@@ -93,19 +93,30 @@ class HookManager {
 
   void RegisterHook() {
     AutoLock auto_lock(lock_);
-    if (init_count_ == 0)
-      WriteHook();
-
     ++init_count_;
+    if (disabled_)
+      return;
+    if (init_count_ == 1)
+      WriteHook();
   }
 
   void UnregisterHook() {
     AutoLock auto_lock(lock_);
     DCHECK_NE(0U, init_count_);
-    if (init_count_ == 1)
-      RevertHook();
-
     --init_count_;
+    if (disabled_)
+      return;
+    if (init_count_ == 0)
+      RevertHook();
+  }
+
+  void DisableCOMChecksForProcess() {
+    AutoLock auto_lock(lock_);
+    if (disabled_)
+      return;
+    disabled_ = true;
+    if (init_count_ > 0)
+      RevertHook();
   }
 
  private:
@@ -267,12 +278,12 @@ class HookManager {
     //
     // If you hit this assert as part of migrating to the Task Scheduler,
     // evaluate your threading guarantees and dispatch your work with
-    // base::CreateCOMSTATaskRunnerWithTraits().
+    // base::CreateCOMSTATaskRunner().
     //
-    // If you need MTA support, ping //base/task/task_scheduler/OWNERS.
+    // If you need MTA support, ping //base/task/thread_pool/OWNERS.
     AssertComInitialized(
         "CoCreateInstance calls in Chromium require explicit COM "
-        "initialization via base::CreateCOMSTATaskRunnerWithTraits() or "
+        "initialization via base::CreateCOMSTATaskRunner() or "
         "ScopedCOMInitializer. See the comment in DCheckedCoCreateInstance for "
         "more details.");
     return original_co_create_instance_body_function_(rclsid, pUnkOuter,
@@ -291,6 +302,7 @@ class HookManager {
   // Synchronizes everything in this class.
   base::Lock lock_;
   size_t init_count_ = 0;
+  bool disabled_ = false;
   HMODULE ole32_library_ = nullptr;
   uint32_t co_create_instance_padded_address_ = 0;
   HotpatchPlaceholderFormat hotpatch_placeholder_format_ =
@@ -321,5 +333,11 @@ ComInitCheckHook::~ComInitCheckHook() {
 #endif  // defined(COM_INIT_CHECK_HOOK_ENABLED)
 }
 
+void ComInitCheckHook::DisableCOMChecksForProcess() {
+#if defined(COM_INIT_CHECK_HOOK_ENABLED)
+  HookManager::GetInstance()->DisableCOMChecksForProcess();
+#endif
+}
+
 }  // namespace win
 }  // namespace base
