@@ -95,9 +95,8 @@ CookiesEventRouter::CookieChangeListener::CookieChangeListener(
 CookiesEventRouter::CookieChangeListener::~CookieChangeListener() = default;
 
 void CookiesEventRouter::CookieChangeListener::OnCookieChange(
-    const net::CanonicalCookie& canonical_cookie,
-    network::mojom::CookieChangeCause cause) {
-  router_->OnCookieChange(otr_, canonical_cookie, cause);
+    const net::CookieChangeInfo& change) {
+  router_->OnCookieChange(otr_, change);
 }
 
 CookiesEventRouter::CookiesEventRouter(content::BrowserContext* context)
@@ -110,50 +109,48 @@ CookiesEventRouter::~CookiesEventRouter() {
   BrowserList::RemoveObserver(this);
 }
 
-void CookiesEventRouter::OnCookieChange(
-    bool otr,
-    const net::CanonicalCookie& canonical_cookie,
-    network::mojom::CookieChangeCause cause) {
+void CookiesEventRouter::OnCookieChange(bool otr,
+                                        const net::CookieChangeInfo& change) {
   DCHECK_CURRENTLY_ON(content::BrowserThread::UI);
 
   std::unique_ptr<base::ListValue> args(new base::ListValue());
   std::unique_ptr<base::DictionaryValue> dict(new base::DictionaryValue());
   dict->SetBoolean(cookies_api_constants::kRemovedKey,
-                   cause != network::mojom::CookieChangeCause::INSERTED);
+                   change.cause != net::CookieChangeCause::INSERTED);
 
   Profile* profile =
       otr ? profile_->GetOffTheRecordProfile() : profile_->GetOriginalProfile();
   api::cookies::Cookie cookie = cookies_helpers::CreateCookie(
-      canonical_cookie, cookies_helpers::GetStoreIdFromProfile(profile));
+      change.cookie, cookies_helpers::GetStoreIdFromProfile(profile));
   dict->Set(cookies_api_constants::kCookieKey, cookie.ToValue());
 
   // Map the internal cause to an external string.
   std::string cause_dict_entry;
-  switch (cause) {
+  switch (change.cause) {
     // Report an inserted cookie as an "explicit" change cause. All other causes
     // only make sense for deletions.
-    case network::mojom::CookieChangeCause::INSERTED:
-    case network::mojom::CookieChangeCause::EXPLICIT:
+    case net::CookieChangeCause::INSERTED:
+    case net::CookieChangeCause::EXPLICIT:
       cause_dict_entry = cookies_api_constants::kExplicitChangeCause;
       break;
 
-    case network::mojom::CookieChangeCause::OVERWRITE:
+    case net::CookieChangeCause::OVERWRITE:
       cause_dict_entry = cookies_api_constants::kOverwriteChangeCause;
       break;
 
-    case network::mojom::CookieChangeCause::EXPIRED:
+    case net::CookieChangeCause::EXPIRED:
       cause_dict_entry = cookies_api_constants::kExpiredChangeCause;
       break;
 
-    case network::mojom::CookieChangeCause::EVICTED:
+    case net::CookieChangeCause::EVICTED:
       cause_dict_entry = cookies_api_constants::kEvictedChangeCause;
       break;
 
-    case network::mojom::CookieChangeCause::EXPIRED_OVERWRITE:
+    case net::CookieChangeCause::EXPIRED_OVERWRITE:
       cause_dict_entry = cookies_api_constants::kExpiredOverwriteChangeCause;
       break;
 
-    case network::mojom::CookieChangeCause::UNKNOWN_DELETION:
+    case net::CookieChangeCause::UNKNOWN_DELETION:
       NOTREACHED();
   }
   dict->SetString(cookies_api_constants::kCauseKey, cause_dict_entry);
@@ -162,7 +159,7 @@ void CookiesEventRouter::OnCookieChange(
 
   DispatchEvent(profile, events::COOKIES_ON_CHANGED,
                 api::cookies::OnChanged::kEventName, std::move(args),
-                cookies_helpers::GetURLFromCanonicalCookie(canonical_cookie));
+                cookies_helpers::GetURLFromCanonicalCookie(change.cookie));
 }
 
 void CookiesEventRouter::OnBrowserAdded(Browser* browser) {
@@ -182,14 +179,14 @@ void CookiesEventRouter::MaybeStartListening() {
                              ? original_profile->GetOffTheRecordProfile()
                              : nullptr;
 
-  if (!binding_)
-    BindToCookieManager(&binding_, original_profile);
-  if (!otr_binding_.is_bound() && otr_profile)
-    BindToCookieManager(&otr_binding_, otr_profile);
+  if (!receiver_.is_bound())
+    BindToCookieManager(&receiver_, original_profile);
+  if (!otr_receiver_.is_bound() && otr_profile)
+    BindToCookieManager(&otr_receiver_, otr_profile);
 }
 
 void CookiesEventRouter::BindToCookieManager(
-    mojo::Binding<network::mojom::CookieChangeListener>* binding,
+    mojo::Receiver<network::mojom::CookieChangeListener>* receiver,
     Profile* profile) {
   network::mojom::CookieManager* cookie_manager =
       content::BrowserContext::GetDefaultStoragePartition(profile)
@@ -197,19 +194,17 @@ void CookiesEventRouter::BindToCookieManager(
   if (!cookie_manager)
     return;
 
-  network::mojom::CookieChangeListenerPtr listener_ptr;
-  binding->Bind(mojo::MakeRequest(&listener_ptr));
-  binding->set_connection_error_handler(base::BindOnce(
-      &CookiesEventRouter::OnConnectionError, base::Unretained(this), binding));
-
-  cookie_manager->AddGlobalChangeListener(std::move(listener_ptr));
+  cookie_manager->AddGlobalChangeListener(receiver->BindNewPipeAndPassRemote());
+  receiver->set_disconnect_handler(
+      base::BindOnce(&CookiesEventRouter::OnConnectionError,
+                     base::Unretained(this), receiver));
 }
 
 void CookiesEventRouter::OnConnectionError(
-    mojo::Binding<network::mojom::CookieChangeListener>* binding) {
+    mojo::Receiver<network::mojom::CookieChangeListener>* receiver) {
   DCHECK_CURRENTLY_ON(content::BrowserThread::UI);
 
-  binding->Close();
+  receiver->reset();
   MaybeStartListening();
 }
 
@@ -251,25 +246,26 @@ ExtensionFunction::ResponseAction CookiesGetFunction::Run() {
   if (!parsed_args_->details.store_id.get())
     parsed_args_->details.store_id.reset(new std::string(store_id));
 
+  DCHECK(!url_.is_empty() && url_.is_valid());
   cookies_helpers::GetCookieListFromManager(
       cookie_manager, url_,
-      base::BindOnce(&CookiesGetFunction::GetCookieCallback, this));
+      base::BindOnce(&CookiesGetFunction::GetCookieListCallback, this));
 
   // Will finish asynchronously.
   return RespondLater();
 }
 
-void CookiesGetFunction::GetCookieCallback(
-    const net::CookieList& cookie_list,
+void CookiesGetFunction::GetCookieListCallback(
+    const net::CookieStatusList& cookie_status_list,
     const net::CookieStatusList& excluded_cookies) {
   DCHECK_CURRENTLY_ON(BrowserThread::UI);
-  for (const net::CanonicalCookie& cookie : cookie_list) {
+  for (const net::CookieWithStatus& cookie_with_status : cookie_status_list) {
     // Return the first matching cookie. Relies on the fact that the
     // CookieManager interface returns them in canonical order (longest path,
     // then earliest creation time).
-    if (cookie.Name() == parsed_args_->details.name) {
+    if (cookie_with_status.cookie.Name() == parsed_args_->details.name) {
       api::cookies::Cookie api_cookie = cookies_helpers::CreateCookie(
-          cookie, *parsed_args_->details.store_id);
+          cookie_with_status.cookie, *parsed_args_->details.store_id);
       Respond(ArgumentList(api::cookies::Get::Results::Create(api_cookie)));
       return;
     }
@@ -308,22 +304,46 @@ ExtensionFunction::ResponseAction CookiesGetAllFunction::Run() {
     parsed_args_->details.store_id.reset(new std::string(store_id));
 
   DCHECK(url_.is_empty() || url_.is_valid());
-  cookies_helpers::GetCookieListFromManager(
-      cookie_manager, url_,
-      base::BindOnce(&CookiesGetAllFunction::GetAllCookiesCallback, this));
+  if (url_.is_empty()) {
+    cookies_helpers::GetAllCookiesFromManager(
+        cookie_manager,
+        base::BindOnce(&CookiesGetAllFunction::GetAllCookiesCallback, this));
+  } else {
+    cookies_helpers::GetCookieListFromManager(
+        cookie_manager, url_,
+        base::BindOnce(&CookiesGetAllFunction::GetCookieListCallback, this));
+  }
 
   return RespondLater();
 }
 
 void CookiesGetAllFunction::GetAllCookiesCallback(
-    const net::CookieList& cookie_list,
+    const net::CookieList& cookie_list) {
+  DCHECK_CURRENTLY_ON(BrowserThread::UI);
+  ResponseValue response;
+  if (extension()) {
+    std::vector<api::cookies::Cookie> match_vector;
+    cookies_helpers::AppendMatchingCookiesFromCookieListToVector(
+        cookie_list, &parsed_args_->details, extension(), &match_vector);
+
+    response =
+        ArgumentList(api::cookies::GetAll::Results::Create(match_vector));
+  } else {
+    // TODO(devlin): When can |extension()| be null for this function?
+    response = NoArguments();
+  }
+  Respond(std::move(response));
+}
+
+void CookiesGetAllFunction::GetCookieListCallback(
+    const net::CookieStatusList& cookie_status_list,
     const net::CookieStatusList& excluded_cookies) {
   DCHECK_CURRENTLY_ON(BrowserThread::UI);
   ResponseValue response;
   if (extension()) {
     std::vector<api::cookies::Cookie> match_vector;
-    cookies_helpers::AppendMatchingCookiesToVector(
-        cookie_list, url_, &parsed_args_->details, extension(), &match_vector);
+    cookies_helpers::AppendMatchingCookiesFromCookieStatusListToVector(
+        cookie_status_list, &parsed_args_->details, extension(), &match_vector);
 
     response =
         ArgumentList(api::cookies::GetAll::Results::Create(match_vector));
@@ -417,7 +437,7 @@ ExtensionFunction::ResponseAction CookiesSetFunction::Run() {
     // is generated.
     success_ = false;
     state_ = SET_COMPLETED;
-    GetCookieListCallback(net::CookieList(), net::CookieStatusList());
+    GetCookieListCallback(net::CookieStatusList(), net::CookieStatusList());
     return AlreadyResponded();
   }
 
@@ -428,6 +448,7 @@ ExtensionFunction::ResponseAction CookiesSetFunction::Run() {
   options.set_include_httponly();
   options.set_same_site_cookie_context(
       net::CookieOptions::SameSiteCookieContext::SAME_SITE_STRICT);
+  DCHECK(!url_.is_empty() && url_.is_valid());
   cookie_manager->SetCanonicalCookie(
       *cc, url_.scheme(), options,
       base::BindOnce(&CookiesSetFunction::SetCanonicalCookieCallback, this));
@@ -444,12 +465,11 @@ void CookiesSetFunction::SetCanonicalCookieCallback(
   DCHECK_CURRENTLY_ON(BrowserThread::UI);
   DCHECK_EQ(NO_RESPONSE, state_);
   state_ = SET_COMPLETED;
-  success_ = (set_cookie_result ==
-              net::CanonicalCookie::CookieInclusionStatus::INCLUDE);
+  success_ = set_cookie_result.IsInclude();
 }
 
 void CookiesSetFunction::GetCookieListCallback(
-    const net::CookieList& cookie_list,
+    const net::CookieStatusList& cookie_list,
     const net::CookieStatusList& excluded_cookies) {
   DCHECK_CURRENTLY_ON(BrowserThread::UI);
   DCHECK_EQ(SET_COMPLETED, state_);
@@ -465,16 +485,16 @@ void CookiesSetFunction::GetCookieListCallback(
   }
 
   ResponseValue value;
-  for (const net::CanonicalCookie& cookie : cookie_list) {
+  for (const net::CookieWithStatus& cookie_with_status : cookie_list) {
     // Return the first matching cookie. Relies on the fact that the
     // CookieMonster returns them in canonical order (longest path, then
     // earliest creation time).
     std::string name =
         parsed_args_->details.name.get() ? *parsed_args_->details.name
                                          : std::string();
-    if (cookie.Name() == name) {
+    if (cookie_with_status.cookie.Name() == name) {
       api::cookies::Cookie api_cookie = cookies_helpers::CreateCookie(
-          cookie, *parsed_args_->details.store_id);
+          cookie_with_status.cookie, *parsed_args_->details.store_id);
       value = ArgumentList(api::cookies::Set::Results::Create(api_cookie));
       break;
     }
