@@ -10,6 +10,7 @@
 #include <set>
 #include <string>
 
+#include "ash/public/cpp/tablet_mode_observer.h"
 #include "ash/public/cpp/wallpaper_controller_observer.h"
 #include "base/callback.h"
 #include "base/compiler_specific.h"
@@ -21,7 +22,6 @@
 #include "chrome/browser/chromeos/login/signin_specifics.h"
 #include "chrome/browser/chromeos/login/ui/login_display.h"
 #include "chrome/browser/chromeos/settings/cros_settings.h"
-#include "chrome/browser/ui/ash/tablet_mode_client_observer.h"
 #include "chrome/browser/ui/webui/chromeos/login/base_webui_handler.h"
 #include "chrome/browser/ui/webui/chromeos/login/network_state_informer.h"
 #include "chrome/browser/ui/webui/chromeos/login/oobe_ui.h"
@@ -183,7 +183,7 @@ class SigninScreenHandler
       public NetworkStateInformer::NetworkStateInformerObserver,
       public PowerManagerClient::Observer,
       public input_method::ImeKeyboard::Observer,
-      public TabletModeClientObserver,
+      public ash::TabletModeObserver,
       public OobeUI::Observer,
       public ash::WallpaperControllerObserver {
  public:
@@ -302,8 +302,11 @@ class SigninScreenHandler
   // PowerManagerClient::Observer implementation:
   void SuspendDone(const base::TimeDelta& sleep_duration) override;
 
-  // TabletModeClientObserver:
-  void OnTabletModeToggled(bool enabled) override;
+  // ash::TabletModeObserver:
+  void OnTabletModeStarted() override;
+  void OnTabletModeEnded() override;
+
+  void OnTabletModeToggled(bool enabled);
 
   // Restore input focus to current user pod.
   void RefocusCurrentPod();
@@ -324,6 +327,7 @@ class SigninScreenHandler
                                            const std::string& password);
   void HandleAttemptUnlock(const std::string& username);
   void HandleLaunchIncognito();
+  void HandleLaunchSAMLPublicSession(const std::string& email);
   void HandleLaunchPublicSession(const AccountId& account_id,
                                  const std::string& locale,
                                  const std::string& input_method);
@@ -503,7 +507,7 @@ class SigninScreenHandler
 
   std::unique_ptr<AccountId> focused_pod_account_id_;
 
-  base::WeakPtrFactory<SigninScreenHandler> weak_factory_;
+  base::WeakPtrFactory<SigninScreenHandler> weak_factory_{this};
 
   DISALLOW_COPY_AND_ASSIGN(SigninScreenHandler);
 };
