@@ -4,9 +4,9 @@
 
 #include "chrome/browser/ui/webui/quota_internals/quota_internals_ui.h"
 
+#include <memory>
 #include <string>
 
-#include "base/memory/ptr_util.h"
 #include "chrome/browser/profiles/profile.h"
 #include "chrome/browser/ui/webui/quota_internals/quota_internals_handler.h"
 #include "chrome/common/url_constants.h"
@@ -23,13 +23,12 @@ content::WebUIDataSource* CreateQuotaInternalsHTMLSource() {
   content::WebUIDataSource* source =
       content::WebUIDataSource::Create(chrome::kChromeUIQuotaInternalsHost);
 
-  source->SetJsonPath("strings.js");
+  source->UseStringsJs();
   source->AddResourcePath(
       "event_handler.js", IDR_QUOTA_INTERNALS_EVENT_HANDLER_JS);
   source->AddResourcePath(
       "message_dispatcher.js", IDR_QUOTA_INTERNALS_MESSAGE_DISPATCHER_JS);
   source->SetDefaultResource(IDR_QUOTA_INTERNALS_MAIN_HTML);
-  source->UseGzip(std::unordered_set<std::string>());
   return source;
 }
 
@@ -38,7 +37,7 @@ content::WebUIDataSource* CreateQuotaInternalsHTMLSource() {
 QuotaInternalsUI::QuotaInternalsUI(content::WebUI* web_ui)
     : WebUIController(web_ui) {
   web_ui->AddMessageHandler(
-      base::MakeUnique<quota_internals::QuotaInternalsHandler>());
+      std::make_unique<quota_internals::QuotaInternalsHandler>());
   Profile* profile = Profile::FromWebUI(web_ui);
   content::WebUIDataSource::Add(profile, CreateQuotaInternalsHTMLSource());
 }
