@@ -109,10 +109,7 @@ void PowerHandler::TestAPI::SetLidClosedBehavior(
   handler_->HandleSetLidClosedBehavior(&args);
 }
 
-PowerHandler::PowerHandler(PrefService* prefs)
-    : prefs_(prefs),
-      power_manager_client_observer_(this),
-      weak_ptr_factory_(this) {}
+PowerHandler::PowerHandler(PrefService* prefs) : prefs_(prefs) {}
 
 PowerHandler::~PowerHandler() {}
 
