@@ -11,7 +11,7 @@
 #include "ash/keyboard/ui/keyboard_ui_controller.h"
 #include "base/strings/string16.h"
 #include "base/strings/utf_string_conversions.h"
-#include "components/arc/common/ime.mojom.h"
+#include "components/arc/mojom/ime.mojom.h"
 #include "components/arc/session/arc_bridge_service.h"
 #include "testing/gtest/include/gtest/gtest.h"
 #include "ui/aura/test/test_window_delegate.h"
@@ -31,12 +31,17 @@ namespace {
 class FakeArcImeBridge : public ArcImeBridge {
  public:
   FakeArcImeBridge()
-      : count_send_insert_text_(0), last_keyboard_availability_(false) {}
+      : count_send_insert_text_(0),
+        last_keyboard_availability_(false),
+        selection_range_(gfx::Range()) {}
 
   void SendSetCompositionText(const ui::CompositionText& composition) override {
   }
   void SendConfirmCompositionText() override {
   }
+  void SendSelectionRange(const gfx::Range& selection_range) override {
+    selection_range_ = selection_range;
+  }
   void SendInsertText(const base::string16& text) override {
     count_send_insert_text_++;
   }
@@ -55,11 +60,13 @@ class FakeArcImeBridge : public ArcImeBridge {
   bool last_keyboard_availability() const {
     return last_keyboard_availability_;
   }
+  gfx::Range selection_range() { return selection_range_; }
 
  private:
   int count_send_insert_text_;
   gfx::Rect last_keyboard_bounds_;
   bool last_keyboard_availability_;
+  gfx::Range selection_range_;
 };
 
 class FakeInputMethod : public ui::DummyInputMethod {
@@ -227,7 +234,7 @@ TEST_F(ArcImeServiceTest, HasCompositionText) {
 
   instance_->SetCompositionText(composition);
   EXPECT_TRUE(instance_->HasCompositionText());
-  instance_->ConfirmCompositionText();
+  instance_->ConfirmCompositionText(/* keep_selection */ false);
   EXPECT_FALSE(instance_->HasCompositionText());
 
   instance_->SetCompositionText(composition);
@@ -241,6 +248,40 @@ TEST_F(ArcImeServiceTest, HasCompositionText) {
   EXPECT_FALSE(instance_->HasCompositionText());
 }
 
+TEST_F(ArcImeServiceTest, SetEditableSelectionRange) {
+  instance_->OnWindowFocused(arc_win_.get(), nullptr);
+  ui::CompositionText composition;
+  instance_->SetCompositionText(composition);
+  EXPECT_TRUE(instance_->SetEditableSelectionRange(gfx::Range(3, 8)));
+  gfx::Range selection;
+  instance_->GetEditableSelectionRange(&selection);
+  EXPECT_EQ(gfx::Range(3, 8), selection);
+
+  EXPECT_TRUE(instance_->SetEditableSelectionRange(gfx::Range(2, 4)));
+  instance_->GetEditableSelectionRange(&selection);
+  EXPECT_EQ(gfx::Range(2, 4), selection);
+}
+
+TEST_F(ArcImeServiceTest, ConfirmCompositionText) {
+  instance_->OnWindowFocused(arc_win_.get(), nullptr);
+
+  ui::CompositionText composition;
+  composition.text = base::UTF8ToUTF16("nonempty text");
+  EXPECT_FALSE(instance_->HasCompositionText());
+  instance_->SetCompositionText(composition);
+  EXPECT_TRUE(instance_->HasCompositionText());
+
+  instance_->SetEditableSelectionRange(gfx::Range(3, 8));
+
+  gfx::Range selection;
+  instance_->GetEditableSelectionRange(&selection);
+  EXPECT_EQ(gfx::Range(3, 8), selection);
+  instance_->ConfirmCompositionText(/* keep_selection */ true);
+  selection = gfx::Range();
+  instance_->GetEditableSelectionRange(&selection);
+  EXPECT_EQ(gfx::Range(3, 8), selection);
+}
+
 TEST_F(ArcImeServiceTest, ShowVirtualKeyboardIfEnabled) {
   instance_->OnWindowFocused(arc_win_.get(), nullptr);
 
