@@ -5,12 +5,21 @@
 #ifndef COMPONENTS_BROWSER_SYNC_BROWSER_SYNC_SWITCHES_H_
 #define COMPONENTS_BROWSER_SYNC_BROWSER_SYNC_SWITCHES_H_
 
+#include "base/feature_list.h"
+#include "build/build_config.h"
+
 namespace switches {
 
 extern const char kDisableSyncTypes[];
 extern const char kEnableLocalSyncBackend[];
 extern const char kLocalSyncBackendDir[];
 
+#if defined(OS_ANDROID)
+extern const base::Feature kSyncManualStartAndroid;
+extern const base::Feature kSyncUseSessionsUnregisterDelay;
+extern const base::Feature kSyncErrorInfoBarAndroid;
+#endif
+
 }  // namespace switches
 
 #endif  // COMPONENTS_BROWSER_SYNC_BROWSER_SYNC_SWITCHES_H_
