@@ -12,8 +12,8 @@
 #include "base/strings/string_number_conversions.h"
 #include "base/test/metrics/histogram_tester.h"
 #include "base/test/scoped_feature_list.h"
-#include "base/test/scoped_task_environment.h"
 #include "base/test/simple_test_clock.h"
+#include "base/test/task_environment.h"
 #include "base/threading/thread_task_runner_handle.h"
 #include "base/time/default_clock.h"
 #include "components/data_reduction_proxy/core/common/data_reduction_proxy_features.h"
@@ -29,54 +29,32 @@ namespace {
 
 class TestNetworkPropertiesManager : public NetworkPropertiesManager {
  public:
-  TestNetworkPropertiesManager(
-      PrefService* pref_service,
-      scoped_refptr<base::SingleThreadTaskRunner> ui_task_runner)
+  TestNetworkPropertiesManager(PrefService* pref_service)
       : TestNetworkPropertiesManager(base::DefaultClock::GetInstance(),
-                                     pref_service,
-                                     ui_task_runner) {}
-  TestNetworkPropertiesManager(
-      base::Clock* clock,
-      PrefService* pref_service,
-      scoped_refptr<base::SingleThreadTaskRunner> ui_task_runner)
-      : NetworkPropertiesManager(clock, pref_service, ui_task_runner) {}
+                                     pref_service) {}
+  TestNetworkPropertiesManager(base::Clock* clock, PrefService* pref_service)
+      : NetworkPropertiesManager(clock, pref_service) {}
   ~TestNetworkPropertiesManager() override {}
-
-  static void InitDiscardCanaryCheckResultExperiment(
-      base::test::ScopedFeatureList* scoped_feature_list) {
-    std::map<std::string, std::string> params;
-    params[params::GetDiscardCanaryCheckResultParam()] = "true";
-    scoped_feature_list->InitAndEnableFeatureWithParameters(
-        features::kDataReductionProxyRobustConnection, params);
-  }
 };
 
 TEST(NetworkPropertyTest, TestSetterGetterCaptivePortal) {
   base::HistogramTester histogram_tester;
   TestingPrefServiceSimple test_prefs;
   test_prefs.registry()->RegisterDictionaryPref(prefs::kNetworkProperties);
-  base::test::ScopedTaskEnvironment task_environment(
-      base::test::ScopedTaskEnvironment::MainThreadType::IO);
-  TestNetworkPropertiesManager network_properties_manager(
-      &test_prefs, base::ThreadTaskRunnerHandle::Get());
+  base::test::SingleThreadTaskEnvironment task_environment(
+      base::test::SingleThreadTaskEnvironment::MainThreadType::IO);
+  TestNetworkPropertiesManager network_properties_manager(&test_prefs);
 
   std::string network_id("test");
   network_properties_manager.OnChangeInNetworkID(network_id);
 
-  EXPECT_TRUE(network_properties_manager.IsInsecureProxyAllowed(false));
   EXPECT_TRUE(network_properties_manager.IsInsecureProxyAllowed(true));
-  EXPECT_TRUE(network_properties_manager.IsSecureProxyAllowed(false));
   EXPECT_TRUE(network_properties_manager.IsSecureProxyAllowed(true));
 
   network_properties_manager.SetIsCaptivePortal(true);
-  EXPECT_TRUE(network_properties_manager.IsInsecureProxyAllowed(false));
-  EXPECT_FALSE(network_properties_manager.IsSecureProxyAllowed(false));
   EXPECT_FALSE(network_properties_manager.IsSecureProxyAllowed(true));
   EXPECT_FALSE(network_properties_manager.IsSecureProxyDisallowedByCarrier());
   EXPECT_TRUE(network_properties_manager.IsCaptivePortal());
-  EXPECT_FALSE(
-      network_properties_manager.HasWarmupURLProbeFailed(false, false));
-  EXPECT_FALSE(network_properties_manager.HasWarmupURLProbeFailed(true, false));
   base::RunLoop().RunUntilIdle();
 
   // Verify the prefs.
@@ -86,26 +64,19 @@ TEST(NetworkPropertyTest, TestSetterGetterCaptivePortal) {
   EXPECT_TRUE(
       test_prefs.GetDictionary(prefs::kNetworkProperties)->HasKey(network_id));
   {
-    TestNetworkPropertiesManager network_properties_manager_2(
-        &test_prefs, base::ThreadTaskRunnerHandle::Get());
+    TestNetworkPropertiesManager network_properties_manager_2(&test_prefs);
     network_properties_manager_2.OnChangeInNetworkID(network_id);
     EXPECT_TRUE(network_properties_manager_2.IsCaptivePortal());
   }
 
   network_properties_manager.SetIsCaptivePortal(false);
-  EXPECT_TRUE(network_properties_manager.IsInsecureProxyAllowed(false));
-  EXPECT_TRUE(network_properties_manager.IsSecureProxyAllowed(false));
   EXPECT_TRUE(network_properties_manager.IsSecureProxyAllowed(true));
   EXPECT_FALSE(network_properties_manager.IsSecureProxyDisallowedByCarrier());
   EXPECT_FALSE(network_properties_manager.IsCaptivePortal());
-  EXPECT_FALSE(
-      network_properties_manager.HasWarmupURLProbeFailed(false, false));
-  EXPECT_FALSE(network_properties_manager.HasWarmupURLProbeFailed(true, false));
   base::RunLoop().RunUntilIdle();
 
   {
-    TestNetworkPropertiesManager network_properties_manager_2(
-        &test_prefs, base::ThreadTaskRunnerHandle::Get());
+    TestNetworkPropertiesManager network_properties_manager_2(&test_prefs);
     network_properties_manager_2.OnChangeInNetworkID(network_id);
     EXPECT_FALSE(network_properties_manager_2.IsCaptivePortal());
   }
@@ -121,129 +92,87 @@ TEST(NetworkPropertyTest, TestSetterGetterCaptivePortal) {
 TEST(NetworkPropertyTest, TestSetterGetterDisallowedByCarrier) {
   TestingPrefServiceSimple test_prefs;
   test_prefs.registry()->RegisterDictionaryPref(prefs::kNetworkProperties);
-  base::test::ScopedTaskEnvironment task_environment(
-      base::test::ScopedTaskEnvironment::MainThreadType::IO);
-  TestNetworkPropertiesManager network_properties_manager(
-      &test_prefs, base::ThreadTaskRunnerHandle::Get());
-
-  EXPECT_TRUE(network_properties_manager.IsInsecureProxyAllowed(false));
-  EXPECT_TRUE(network_properties_manager.IsSecureProxyAllowed(false));
+  base::test::SingleThreadTaskEnvironment task_environment(
+      base::test::SingleThreadTaskEnvironment::MainThreadType::IO);
+  TestNetworkPropertiesManager network_properties_manager(&test_prefs);
 
   network_properties_manager.SetIsSecureProxyDisallowedByCarrier(true);
-  EXPECT_TRUE(network_properties_manager.IsInsecureProxyAllowed(false));
-  EXPECT_FALSE(network_properties_manager.IsSecureProxyAllowed(false));
   EXPECT_FALSE(network_properties_manager.IsSecureProxyAllowed(true));
   EXPECT_TRUE(network_properties_manager.IsSecureProxyDisallowedByCarrier());
   EXPECT_FALSE(network_properties_manager.IsCaptivePortal());
-  EXPECT_FALSE(
-      network_properties_manager.HasWarmupURLProbeFailed(false, false));
-  EXPECT_FALSE(network_properties_manager.HasWarmupURLProbeFailed(true, false));
 
   network_properties_manager.SetIsSecureProxyDisallowedByCarrier(false);
-  EXPECT_TRUE(network_properties_manager.IsInsecureProxyAllowed(false));
   EXPECT_TRUE(network_properties_manager.IsInsecureProxyAllowed(true));
-  EXPECT_TRUE(network_properties_manager.IsSecureProxyAllowed(false));
   EXPECT_FALSE(network_properties_manager.IsSecureProxyDisallowedByCarrier());
   EXPECT_FALSE(network_properties_manager.IsCaptivePortal());
-  EXPECT_FALSE(
-      network_properties_manager.HasWarmupURLProbeFailed(false, false));
-  EXPECT_FALSE(network_properties_manager.HasWarmupURLProbeFailed(true, false));
 }
 
 TEST(NetworkPropertyTest, TestWarmupURLFailedOnSecureCoreProxy) {
   TestingPrefServiceSimple test_prefs;
   test_prefs.registry()->RegisterDictionaryPref(prefs::kNetworkProperties);
-  base::test::ScopedTaskEnvironment task_environment(
-      base::test::ScopedTaskEnvironment::MainThreadType::IO);
-  TestNetworkPropertiesManager network_properties_manager(
-      &test_prefs, base::ThreadTaskRunnerHandle::Get());
-
-  EXPECT_TRUE(network_properties_manager.IsInsecureProxyAllowed(false));
-  EXPECT_TRUE(network_properties_manager.IsSecureProxyAllowed(false));
+  base::test::SingleThreadTaskEnvironment task_environment(
+      base::test::SingleThreadTaskEnvironment::MainThreadType::IO);
+  TestNetworkPropertiesManager network_properties_manager(&test_prefs);
 
   network_properties_manager.SetHasWarmupURLProbeFailed(
       true /* secure_proxy */, true /* is_core_proxy */,
       true /* warmup_url_probe_failed */);
-  EXPECT_TRUE(network_properties_manager.IsInsecureProxyAllowed(false));
   EXPECT_TRUE(network_properties_manager.IsInsecureProxyAllowed(true));
-  EXPECT_TRUE(network_properties_manager.IsSecureProxyAllowed(false));
   EXPECT_FALSE(network_properties_manager.IsSecureProxyAllowed(true));
   EXPECT_FALSE(network_properties_manager.IsSecureProxyDisallowedByCarrier());
   EXPECT_FALSE(network_properties_manager.IsCaptivePortal());
-  EXPECT_FALSE(network_properties_manager.HasWarmupURLProbeFailed(
-      false /* secure_proxy */, false /* is_core_proxy */));
-  EXPECT_FALSE(network_properties_manager.HasWarmupURLProbeFailed(true, false));
   EXPECT_FALSE(network_properties_manager.HasWarmupURLProbeFailed(false, true));
   EXPECT_TRUE(network_properties_manager.HasWarmupURLProbeFailed(true, true));
 
   network_properties_manager.SetHasWarmupURLProbeFailed(true, true, false);
-  EXPECT_TRUE(network_properties_manager.IsInsecureProxyAllowed(false));
   EXPECT_TRUE(network_properties_manager.IsInsecureProxyAllowed(true));
-  EXPECT_TRUE(network_properties_manager.IsSecureProxyAllowed(false));
   EXPECT_TRUE(network_properties_manager.IsSecureProxyAllowed(true));
   EXPECT_FALSE(network_properties_manager.IsSecureProxyDisallowedByCarrier());
   EXPECT_FALSE(network_properties_manager.IsCaptivePortal());
-  EXPECT_FALSE(
-      network_properties_manager.HasWarmupURLProbeFailed(false, false));
   EXPECT_FALSE(network_properties_manager.HasWarmupURLProbeFailed(false, true));
-  EXPECT_FALSE(network_properties_manager.HasWarmupURLProbeFailed(true, false));
   EXPECT_FALSE(network_properties_manager.HasWarmupURLProbeFailed(true, true));
 }
 
 TEST(NetworkPropertyTest, TestWarmupURLFailedOnInSecureCoreProxy) {
   TestingPrefServiceSimple test_prefs;
   test_prefs.registry()->RegisterDictionaryPref(prefs::kNetworkProperties);
-  base::test::ScopedTaskEnvironment task_environment(
-      base::test::ScopedTaskEnvironment::MainThreadType::IO);
-  TestNetworkPropertiesManager network_properties_manager(
-      &test_prefs, base::ThreadTaskRunnerHandle::Get());
+  base::test::SingleThreadTaskEnvironment task_environment(
+      base::test::SingleThreadTaskEnvironment::MainThreadType::IO);
+  TestNetworkPropertiesManager network_properties_manager(&test_prefs);
 
-  EXPECT_TRUE(network_properties_manager.IsInsecureProxyAllowed(false));
   EXPECT_TRUE(network_properties_manager.IsInsecureProxyAllowed(true));
-  EXPECT_TRUE(network_properties_manager.IsSecureProxyAllowed(false));
   EXPECT_TRUE(network_properties_manager.IsSecureProxyAllowed(true));
 
   network_properties_manager.SetHasWarmupURLProbeFailed(
       false /* secure_proxy */, true /* is_core_proxy */,
       true /* warmup_url_probe_failed */);
-  EXPECT_TRUE(network_properties_manager.IsInsecureProxyAllowed(false));
   EXPECT_FALSE(network_properties_manager.IsInsecureProxyAllowed(true));
-  EXPECT_TRUE(network_properties_manager.IsSecureProxyAllowed(false));
   EXPECT_TRUE(network_properties_manager.IsSecureProxyAllowed(true));
   EXPECT_FALSE(network_properties_manager.IsSecureProxyDisallowedByCarrier());
   EXPECT_FALSE(network_properties_manager.IsCaptivePortal());
-  EXPECT_FALSE(network_properties_manager.HasWarmupURLProbeFailed(
-      false /* secure_proxy */, false /* is_core_proxy */));
-  EXPECT_FALSE(network_properties_manager.HasWarmupURLProbeFailed(true, false));
   EXPECT_TRUE(network_properties_manager.HasWarmupURLProbeFailed(false, true));
   EXPECT_FALSE(network_properties_manager.HasWarmupURLProbeFailed(true, true));
 
   network_properties_manager.SetHasWarmupURLProbeFailed(false, true, false);
-  EXPECT_TRUE(network_properties_manager.IsInsecureProxyAllowed(false));
   EXPECT_TRUE(network_properties_manager.IsInsecureProxyAllowed(true));
-  EXPECT_TRUE(network_properties_manager.IsSecureProxyAllowed(false));
   EXPECT_TRUE(network_properties_manager.IsSecureProxyAllowed(true));
   EXPECT_FALSE(network_properties_manager.IsSecureProxyDisallowedByCarrier());
   EXPECT_FALSE(network_properties_manager.IsCaptivePortal());
-  EXPECT_FALSE(
-      network_properties_manager.HasWarmupURLProbeFailed(false, false));
   EXPECT_FALSE(network_properties_manager.HasWarmupURLProbeFailed(false, true));
-  EXPECT_FALSE(network_properties_manager.HasWarmupURLProbeFailed(true, false));
   EXPECT_FALSE(network_properties_manager.HasWarmupURLProbeFailed(true, true));
 }
 
 TEST(NetworkPropertyTest, TestLimitPrefSize) {
   TestingPrefServiceSimple test_prefs;
   test_prefs.registry()->RegisterDictionaryPref(prefs::kNetworkProperties);
-  base::test::ScopedTaskEnvironment task_environment(
-      base::test::ScopedTaskEnvironment::MainThreadType::IO);
-  TestNetworkPropertiesManager network_properties_manager(
-      &test_prefs, base::ThreadTaskRunnerHandle::Get());
+  base::test::SingleThreadTaskEnvironment task_environment(
+      base::test::SingleThreadTaskEnvironment::MainThreadType::IO);
+  TestNetworkPropertiesManager network_properties_manager(&test_prefs);
 
   size_t num_network_ids = 100;
 
   for (size_t i = 0; i < num_network_ids; ++i) {
-    std::string network_id("test" + base::IntToString(i));
+    std::string network_id("test" + base::NumberToString(i));
     network_properties_manager.OnChangeInNetworkID(network_id);
 
     // State should be reset when there is a change in the network ID.
@@ -271,14 +200,13 @@ TEST(NetworkPropertyTest, TestLimitPrefSize) {
   // The last 10 network IDs are guaranteed to be present in the prefs.
   for (size_t i = num_network_ids - 10; i < num_network_ids; ++i) {
     EXPECT_TRUE(test_prefs.GetDictionary(prefs::kNetworkProperties)
-                    ->HasKey("test" + base::IntToString(i)));
+                    ->HasKey("test" + base::NumberToString(i)));
   }
 
   {
-    TestNetworkPropertiesManager network_properties_manager_2(
-        &test_prefs, base::ThreadTaskRunnerHandle::Get());
+    TestNetworkPropertiesManager network_properties_manager_2(&test_prefs);
     for (size_t i = 0; i < num_network_ids; ++i) {
-      std::string network_id("test" + base::IntToString(i));
+      std::string network_id("test" + base::NumberToString(i));
       network_properties_manager_2.OnChangeInNetworkID(network_id);
 
       EXPECT_EQ(test_prefs.GetDictionary(prefs::kNetworkProperties)
@@ -297,10 +225,9 @@ TEST(NetworkPropertyTest, TestLimitPrefSize) {
 TEST(NetworkPropertyTest, TestChangeNetworkIDBackAndForth) {
   TestingPrefServiceSimple test_prefs;
   test_prefs.registry()->RegisterDictionaryPref(prefs::kNetworkProperties);
-  base::test::ScopedTaskEnvironment task_environment(
-      base::test::ScopedTaskEnvironment::MainThreadType::IO);
-  TestNetworkPropertiesManager network_properties_manager(
-      &test_prefs, base::ThreadTaskRunnerHandle::Get());
+  base::test::SingleThreadTaskEnvironment task_environment(
+      base::test::SingleThreadTaskEnvironment::MainThreadType::IO);
+  TestNetworkPropertiesManager network_properties_manager(&test_prefs);
 
   // First network ID has a captive portal.
   std::string first_network_id("test1");
@@ -347,10 +274,9 @@ TEST(NetworkPropertyTest, TestChangeNetworkIDBackAndForth) {
 TEST(NetworkPropertyTest, TestNetworkQualitiesOverwrite) {
   TestingPrefServiceSimple test_prefs;
   test_prefs.registry()->RegisterDictionaryPref(prefs::kNetworkProperties);
-  base::test::ScopedTaskEnvironment task_environment(
-      base::test::ScopedTaskEnvironment::MainThreadType::IO);
-  TestNetworkPropertiesManager network_properties_manager(
-      &test_prefs, base::ThreadTaskRunnerHandle::Get());
+  base::test::SingleThreadTaskEnvironment task_environment(
+      base::test::SingleThreadTaskEnvironment::MainThreadType::IO);
+  TestNetworkPropertiesManager network_properties_manager(&test_prefs);
 
   // First network ID has a captive portal.
   std::string first_network_id("test1");
@@ -399,10 +325,9 @@ TEST(NetworkPropertyTest, TestDeleteHistory) {
   base::HistogramTester histogram_tester;
   TestingPrefServiceSimple test_prefs;
   test_prefs.registry()->RegisterDictionaryPref(prefs::kNetworkProperties);
-  base::test::ScopedTaskEnvironment task_environment(
-      base::test::ScopedTaskEnvironment::MainThreadType::IO);
-  TestNetworkPropertiesManager network_properties_manager(
-      &test_prefs, base::ThreadTaskRunnerHandle::Get());
+  base::test::SingleThreadTaskEnvironment task_environment(
+      base::test::SingleThreadTaskEnvironment::MainThreadType::IO);
+  TestNetworkPropertiesManager network_properties_manager(&test_prefs);
 
   std::string network_id("test");
   network_properties_manager.OnChangeInNetworkID(network_id);
@@ -428,8 +353,7 @@ TEST(NetworkPropertyTest, TestDeleteHistory) {
   EXPECT_TRUE(
       test_prefs.GetDictionary(prefs::kNetworkProperties)->HasKey(network_id));
   {
-    TestNetworkPropertiesManager network_properties_manager_2(
-        &test_prefs, base::ThreadTaskRunnerHandle::Get());
+    TestNetworkPropertiesManager network_properties_manager_2(&test_prefs);
     network_properties_manager_2.OnChangeInNetworkID(network_id);
     EXPECT_TRUE(network_properties_manager_2.IsCaptivePortal());
     histogram_tester.ExpectBucketCount(
@@ -442,8 +366,7 @@ TEST(NetworkPropertyTest, TestDeleteHistory) {
   network_properties_manager.DeleteHistory();
   base::RunLoop().RunUntilIdle();
   {
-    TestNetworkPropertiesManager network_properties_manager_2(
-        &test_prefs, base::ThreadTaskRunnerHandle::Get());
+    TestNetworkPropertiesManager network_properties_manager_2(&test_prefs);
     network_properties_manager_2.OnChangeInNetworkID(network_id);
     EXPECT_FALSE(network_properties_manager_2.IsCaptivePortal());
     histogram_tester.ExpectBucketCount(
@@ -460,13 +383,13 @@ TEST(NetworkPropertyTest, TestDeleteOldValues) {
 
   TestingPrefServiceSimple test_prefs;
   test_prefs.registry()->RegisterDictionaryPref(prefs::kNetworkProperties);
-  base::test::ScopedTaskEnvironment task_environment(
-      base::test::ScopedTaskEnvironment::MainThreadType::IO);
-  TestNetworkPropertiesManager network_properties_manager(
-      &test_clock, &test_prefs, base::ThreadTaskRunnerHandle::Get());
+  base::test::SingleThreadTaskEnvironment task_environment(
+      base::test::SingleThreadTaskEnvironment::MainThreadType::IO);
+  TestNetworkPropertiesManager network_properties_manager(&test_clock,
+                                                          &test_prefs);
 
   for (size_t i = 0; i < 5; ++i) {
-    std::string network_id("test" + base::IntToString(i));
+    std::string network_id("test" + base::NumberToString(i));
     network_properties_manager.OnChangeInNetworkID(network_id);
     network_properties_manager.SetIsCaptivePortal(true);
   }
@@ -474,7 +397,7 @@ TEST(NetworkPropertyTest, TestDeleteOldValues) {
   test_clock.Advance(base::TimeDelta::FromDays(20));
 
   for (size_t i = 5; i < 10; ++i) {
-    std::string network_id("test" + base::IntToString(i));
+    std::string network_id("test" + base::NumberToString(i));
     network_properties_manager.OnChangeInNetworkID(network_id);
     network_properties_manager.SetIsCaptivePortal(true);
   }
@@ -484,17 +407,17 @@ TEST(NetworkPropertyTest, TestDeleteOldValues) {
   // Verify the prefs.
   EXPECT_EQ(10u, test_prefs.GetDictionary(prefs::kNetworkProperties)->size());
   for (size_t i = 0; i < 10; ++i) {
-    std::string network_id("test" + base::IntToString(i));
+    std::string network_id("test" + base::NumberToString(i));
     EXPECT_TRUE(test_prefs.GetDictionary(prefs::kNetworkProperties)
                     ->HasKey(network_id));
   }
 
   // Entries should not be cleared since all values are less than 30 days old.
   {
-    TestNetworkPropertiesManager network_properties_manager_2(
-        &test_clock, &test_prefs, base::ThreadTaskRunnerHandle::Get());
+    TestNetworkPropertiesManager network_properties_manager_2(&test_clock,
+                                                              &test_prefs);
     for (size_t i = 0; i < 10; ++i) {
-      std::string network_id("test" + base::IntToString(i));
+      std::string network_id("test" + base::NumberToString(i));
 
       EXPECT_TRUE(test_prefs.GetDictionary(prefs::kNetworkProperties)
                       ->HasKey(network_id));
@@ -504,10 +427,10 @@ TEST(NetworkPropertyTest, TestDeleteOldValues) {
   // Only the entries from 5 to 9 should be cleared since they are 40 days old.
   test_clock.Advance(base::TimeDelta::FromDays(20));
   {
-    TestNetworkPropertiesManager network_properties_manager_3(
-        &test_clock, &test_prefs, base::ThreadTaskRunnerHandle::Get());
+    TestNetworkPropertiesManager network_properties_manager_3(&test_clock,
+                                                              &test_prefs);
     for (size_t i = 0; i < 10; ++i) {
-      std::string network_id("test" + base::IntToString(i));
+      std::string network_id("test" + base::NumberToString(i));
       EXPECT_EQ(i >= 5, test_prefs.GetDictionary(prefs::kNetworkProperties)
                             ->HasKey(network_id));
     }
@@ -518,44 +441,35 @@ TEST(NetworkPropertyTest,
      TestSetterGetterDisallowedByCarrierDiscardingEnabled) {
   base::test::ScopedFeatureList scoped_feature_list;
 
-  for (bool discard : {false, true}) {
-    if (discard) {
-      TestNetworkPropertiesManager::InitDiscardCanaryCheckResultExperiment(
-          &scoped_feature_list);
-    }
+  TestingPrefServiceSimple test_prefs;
+  test_prefs.registry()->RegisterDictionaryPref(prefs::kNetworkProperties);
+  base::test::SingleThreadTaskEnvironment task_environment(
+      base::test::SingleThreadTaskEnvironment::MainThreadType::IO);
+  TestNetworkPropertiesManager network_properties_manager(&test_prefs);
 
-    TestingPrefServiceSimple test_prefs;
-    test_prefs.registry()->RegisterDictionaryPref(prefs::kNetworkProperties);
-    base::test::ScopedTaskEnvironment task_environment(
-        base::test::ScopedTaskEnvironment::MainThreadType::IO);
-    TestNetworkPropertiesManager network_properties_manager(
-        &test_prefs, base::ThreadTaskRunnerHandle::Get());
+  // First network ID has a captive portal and the canary check failed.
+  std::string first_network_id("test1");
+  network_properties_manager.OnChangeInNetworkID(first_network_id);
+  EXPECT_FALSE(network_properties_manager.IsCaptivePortal());
+  network_properties_manager.SetIsSecureProxyDisallowedByCarrier(true);
+  network_properties_manager.SetIsCaptivePortal(true);
+  EXPECT_TRUE(network_properties_manager.IsSecureProxyDisallowedByCarrier());
+  EXPECT_TRUE(network_properties_manager.IsCaptivePortal());
+  base::RunLoop().RunUntilIdle();
 
-    // First network ID has a captive portal and the canary check failed.
-    std::string first_network_id("test1");
-    network_properties_manager.OnChangeInNetworkID(first_network_id);
-    EXPECT_FALSE(network_properties_manager.IsCaptivePortal());
-    network_properties_manager.SetIsSecureProxyDisallowedByCarrier(true);
-    network_properties_manager.SetIsCaptivePortal(true);
-    EXPECT_TRUE(network_properties_manager.IsSecureProxyDisallowedByCarrier());
-    EXPECT_TRUE(network_properties_manager.IsCaptivePortal());
-    base::RunLoop().RunUntilIdle();
+  // Change to a different network. State should be reset when there is a
+  // change in the network ID.
+  std::string second_network_id("test2");
+  network_properties_manager.OnChangeInNetworkID(second_network_id);
+  EXPECT_FALSE(network_properties_manager.IsCaptivePortal());
+  EXPECT_FALSE(network_properties_manager.IsSecureProxyDisallowedByCarrier());
+  base::RunLoop().RunUntilIdle();
 
-    // Change to a different network. State should be reset when there is a
-    // change in the network ID.
-    std::string second_network_id("test2");
-    network_properties_manager.OnChangeInNetworkID(second_network_id);
-    EXPECT_FALSE(network_properties_manager.IsCaptivePortal());
-    EXPECT_FALSE(network_properties_manager.IsSecureProxyDisallowedByCarrier());
-    base::RunLoop().RunUntilIdle();
-
-    // Change back to |first_network_id|. Captive portal state should be
-    // persisted but the canary check state should not be.
-    network_properties_manager.OnChangeInNetworkID(first_network_id);
-    EXPECT_NE(discard,
-              network_properties_manager.IsSecureProxyDisallowedByCarrier());
-    EXPECT_TRUE(network_properties_manager.IsCaptivePortal());
-  }
+  // Change back to |first_network_id|. Captive portal state should be
+  // persisted but the canary check state should not be.
+  network_properties_manager.OnChangeInNetworkID(first_network_id);
+  EXPECT_TRUE(network_properties_manager.IsSecureProxyDisallowedByCarrier());
+  EXPECT_TRUE(network_properties_manager.IsCaptivePortal());
 }
 
 }  // namespace
