@@ -27,6 +27,11 @@ namespace {
 const char kTraceFilename[] = "tracing.zip";
 const char kPerformanceCategoryTag[] = "Performance";
 
+const base::FilePath::CharType kHistogramsFilename[] =
+    FILE_PATH_LITERAL("histograms.txt");
+
+const char kHistogramsAttachmentName[] = "histograms.zip";
+
 }  // namespace
 
 FeedbackData::FeedbackData(feedback::FeedbackUploader* uploader)
@@ -34,7 +39,9 @@ FeedbackData::FeedbackData(feedback::FeedbackUploader* uploader)
       context_(nullptr),
       trace_id_(0),
       pending_op_count_(1),
-      report_sent_(false) {
+      report_sent_(false),
+      from_assistant_(false),
+      assistant_debug_info_allowed_(false) {
   CHECK(uploader_);
 }
 
@@ -46,8 +53,7 @@ void FeedbackData::OnFeedbackPageDataComplete() {
   SendReport();
 }
 
-void FeedbackData::SetAndCompressSystemInfo(
-    std::unique_ptr<FeedbackData::SystemLogsMap> sys_info) {
+void FeedbackData::CompressSystemInfo() {
   DCHECK_CURRENTLY_ON(BrowserThread::UI);
 
   if (trace_id_ != 0) {
@@ -62,27 +68,38 @@ void FeedbackData::SetAndCompressSystemInfo(
     }
   }
 
-  if (sys_info) {
-    ++pending_op_count_;
-    AddLogs(std::move(sys_info));
-    base::PostTaskWithTraitsAndReply(
-        FROM_HERE, {base::MayBlock(), base::TaskPriority::BEST_EFFORT},
-        base::BindOnce(&FeedbackData::CompressLogs, this),
-        base::BindOnce(&FeedbackData::OnCompressComplete, this));
-  }
+  ++pending_op_count_;
+  base::PostTaskAndReply(
+      FROM_HERE,
+      {base::ThreadPool(), base::MayBlock(), base::TaskPriority::BEST_EFFORT},
+      base::BindOnce(&FeedbackData::CompressLogs, this),
+      base::BindOnce(&FeedbackData::OnCompressComplete, this));
 }
 
-void FeedbackData::AttachAndCompressFileData(
-    std::unique_ptr<std::string> attached_filedata) {
+void FeedbackData::SetAndCompressHistograms(std::string histograms) {
   DCHECK_CURRENTLY_ON(BrowserThread::UI);
 
-  if (!attached_filedata || attached_filedata->empty())
+  ++pending_op_count_;
+  base::PostTaskAndReply(
+      FROM_HERE,
+      {base::ThreadPool(), base::MayBlock(), base::TaskPriority::BEST_EFFORT},
+      base::BindOnce(&FeedbackData::CompressFile, this,
+                     base::FilePath(kHistogramsFilename),
+                     kHistogramsAttachmentName, std::move(histograms)),
+      base::BindOnce(&FeedbackData::OnCompressComplete, this));
+}
+
+void FeedbackData::AttachAndCompressFileData(std::string attached_filedata) {
+  DCHECK_CURRENTLY_ON(BrowserThread::UI);
+
+  if (attached_filedata.empty())
     return;
   ++pending_op_count_;
   base::FilePath attached_file =
                   base::FilePath::FromUTF8Unsafe(attached_filename_);
-  base::PostTaskWithTraitsAndReply(
-      FROM_HERE, {base::MayBlock(), base::TaskPriority::BEST_EFFORT},
+  base::PostTaskAndReply(
+      FROM_HERE,
+      {base::ThreadPool(), base::MayBlock(), base::TaskPriority::BEST_EFFORT},
       base::BindOnce(&FeedbackData::CompressFile, this, attached_file,
                      std::string(), std::move(attached_filedata)),
       base::BindOnce(&FeedbackData::OnCompressComplete, this));
@@ -96,10 +113,7 @@ void FeedbackData::OnGetTraceData(
   if (manager)
     manager->DiscardTraceData(trace_id);
 
-  std::unique_ptr<std::string> data(new std::string);
-  data->swap(trace_data->data());
-
-  AddFile(kTraceFilename, std::move(data));
+  AddFile(kTraceFilename, std::move(trace_data->data()));
 
   set_category_tag(kPerformanceCategoryTag);
   --pending_op_count_;
