@@ -11,17 +11,22 @@
 #include <memory>
 #include <set>
 #include <string>
+#include <utility>
 #include <vector>
 
 #include "base/bind.h"
 #include "base/callback.h"
 #include "base/callback_list.h"
+#include "base/containers/flat_set.h"
+#include "base/feature_list.h"
 #include "base/files/file_path.h"
+#include "base/location.h"
 #include "base/logging.h"
 #include "base/macros.h"
 #include "base/memory/ref_counted.h"
 #include "base/memory/weak_ptr.h"
 #include "base/observer_list.h"
+#include "base/optional.h"
 #include "base/sequenced_task_runner.h"
 #include "base/strings/string16.h"
 #include "base/task/cancelable_task_tracker.h"
@@ -30,13 +35,9 @@
 #include "build/build_config.h"
 #include "components/favicon_base/favicon_callback.h"
 #include "components/favicon_base/favicon_usage_data.h"
-#include "components/history/core/browser/delete_directive_handler.h"
 #include "components/history/core/browser/history_types.h"
 #include "components/history/core/browser/keyword_id.h"
-#include "components/history/core/browser/typed_url_sync_bridge.h"
-#include "components/history/core/browser/typed_url_syncable_service.h"
 #include "components/keyed_service/core/keyed_service.h"
-#include "components/sync/model/syncable_service.h"
 #include "sql/init_status.h"
 #include "ui/base/page_transition_types.h"
 
@@ -51,14 +52,24 @@ class TestingProfile;
 namespace base {
 class FilePath;
 class Thread;
-}
+}  // namespace base
 
 namespace favicon {
 class FaviconServiceImpl;
 }
 
+namespace syncer {
+class ModelTypeControllerDelegate;
+class SyncableService;
+}  // namespace syncer
+
+namespace sync_pb {
+class HistoryDeleteDirectiveSpecifics;
+}
+
 namespace history {
 
+class DeleteDirectiveHandler;
 struct DownloadRow;
 struct HistoryAddPageArgs;
 class HistoryBackend;
@@ -73,15 +84,11 @@ class URLDatabase;
 class VisitDelegate;
 class WebHistoryService;
 
-// The history service records page titles, and visit times, as well as
-// (eventually) information about autocomplete.
-//
-// This service is thread safe. Each request callback is invoked in the
-// thread that made the request.
-class HistoryService : public syncer::SyncableService, public KeyedService {
+// The history service records page titles, visit times, and favicons, as well
+// as information about downloads.
+class HistoryService : public KeyedService {
  public:
-  // Callback for value asynchronously returned by TopHosts().
-  typedef base::Callback<void(const TopHostsList&)> TopHostsCallback;
+  static const base::Feature kHistoryServiceUsesTaskScheduler;
 
   // Must call Init after construction. The empty constructor provided only for
   // unit tests. When using the full constructor, |history_client| may only be
@@ -118,6 +125,9 @@ class HistoryService : public syncer::SyncableService, public KeyedService {
   // out any cached data associated with that context.
   void ClearCachedDataForContextID(ContextID context_id);
 
+  // Clears all on-demand favicons from thumbnail database.
+  void ClearAllOnDemandFavicons();
+
   // Triggers the backend to load if it hasn't already, and then returns the
   // in-memory URL database. The returned pointer may be null if the in-memory
   // database has not been loaded yet. This pointer is owned by the history
@@ -130,54 +140,19 @@ class HistoryService : public syncer::SyncableService, public KeyedService {
   // They return false if database is not available (e.g. not loaded yet) or the
   // URL does not exist.
 
-  // Returns a pointer to the TypedURLSyncBridge owned by HistoryBackend.
-  // This method should only be called from the history thread, because the
-  // returned bridge is intended to be accessed only via the history thread.
-  TypedURLSyncBridge* GetTypedURLSyncBridge() const;
-
-  // Returns a pointer to the TypedUrlSyncableService owned by HistoryBackend.
-  // This method should only be called from the history thread, because the
-  // returned service is intended to be accessed only via the history thread.
-  TypedUrlSyncableService* GetTypedUrlSyncableService() const;
-
   // KeyedService:
   void Shutdown() override;
 
   // Callback for value asynchronously returned by
   // GetCountsAndLastVisitForOrigins().
-  typedef base::Callback<void(const OriginCountAndLastVisitMap&)>
-      GetCountsAndLastVisitForOriginsCallback;
-
-  // Computes the |num_hosts| most-visited hostnames in the past 30 days and
-  // returns a list of those hosts paired with their visit counts. The following
-  // caveats apply:
-  // 1. Hostnames are stripped of their 'www.' prefix. Visits to foo.com and
-  //    www.foo.com are summed into the resultant foo.com entry.
-  // 2. Ports and schemes are ignored. Visits to http://foo.com/ and
-  //    https://foo.com:567/ are summed into the resultant foo.com entry.
-  // 3. If the history is abnormally large and diverse, the function will give
-  //    up early and return an approximate list.
-  // 4. Only http://, https://, and ftp:// URLs are counted.
-  //
-  // Note: Virtual needed for mocking.
-  virtual void TopHosts(size_t num_hosts,
-                        const TopHostsCallback& callback) const;
+  using GetCountsAndLastVisitForOriginsCallback =
+      base::OnceCallback<void(OriginCountAndLastVisitMap)>;
 
   // Gets the counts and most recent visit date of URLs that belong to |origins|
   // in the history database.
-  void GetCountsAndLastVisitForOrigins(
+  void GetCountsAndLastVisitForOriginsForTesting(
       const std::set<GURL>& origins,
-      const GetCountsAndLastVisitForOriginsCallback& callback) const;
-
-  // Returns, for the given URL, a 0-based index into the list produced by
-  // TopHosts(), corresponding to that URL's host. If TopHosts() has not
-  // previously been run, or the host is not in the top kMaxTopHosts, returns
-  // kMaxTopHosts.
-  //
-  // Note: Virtual needed for mocking.
-  virtual void HostRankIfAvailable(
-      const GURL& url,
-      const base::Callback<void(int)>& callback) const;
+      GetCountsAndLastVisitForOriginsCallback callback) const;
 
   // Navigation ----------------------------------------------------------------
 
@@ -193,6 +168,10 @@ class HistoryService : public syncer::SyncableService, public KeyedService {
   // should be the unique ID of the current navigation entry in the given
   // process.
   //
+  // TODO(avi): This is no longer true. 'page id' was removed years ago, and
+  // their uses replaced by globally-unique nav_entry_ids. Is ContextID still
+  // needed? https://crbug.com/859902
+  //
   // 'redirects' is an array of redirect URLs leading to this page, with the
   // page itself as the last item (so when there is no redirect, it will have
   // one entry). If there are no redirects, this array may also be empty for
@@ -245,11 +224,7 @@ class HistoryService : public syncer::SyncableService, public KeyedService {
   // empty.
   //
   // If success is false, neither the row nor the vector will be valid.
-  typedef base::OnceCallback<void(
-      bool,  // Success flag, when false, nothing else is valid.
-      const URLRow&,
-      const VisitVector&)>
-      QueryURLCallback;
+  using QueryURLCallback = base::OnceCallback<void(QueryURLResult)>;
 
   // Queries the basic information about the URL in the history database. If
   // the caller is interested in the visits (each time the URL is visited),
@@ -264,7 +239,7 @@ class HistoryService : public syncer::SyncableService, public KeyedService {
   // Provides the result of a query. See QueryResults in history_types.h.
   // The common use will be to use QueryResults.Swap to suck the contents of
   // the results out of the passed in parameter and take ownership of them.
-  typedef base::Callback<void(QueryResults*)> QueryHistoryCallback;
+  using QueryHistoryCallback = base::OnceCallback<void(QueryResults)>;
 
   // Queries all history with the given options (see QueryOptions in
   // history_types.h).  If empty, all results matching the given options
@@ -272,7 +247,7 @@ class HistoryService : public syncer::SyncableService, public KeyedService {
   base::CancelableTaskTracker::TaskId QueryHistory(
       const base::string16& text_query,
       const QueryOptions& options,
-      const QueryHistoryCallback& callback,
+      QueryHistoryCallback callback,
       base::CancelableTaskTracker* tracker);
 
   // Called when the results of QueryRedirectsFrom are available.
@@ -286,48 +261,44 @@ class HistoryService : public syncer::SyncableService, public KeyedService {
   // If there is no such URL in the database or the most recent visit has no
   // redirect, the vector will be empty. If the given page has redirected to
   // multiple destinations, this will pick a random one.
-  typedef base::Callback<void(const RedirectList*)> QueryRedirectsCallback;
+  using QueryRedirectsCallback = base::OnceCallback<void(RedirectList)>;
 
   // Schedules a query for the most recent redirect coming out of the given
   // URL. See the RedirectQuerySource above, which is guaranteed to be called
   // if the request is not canceled.
   base::CancelableTaskTracker::TaskId QueryRedirectsFrom(
       const GURL& from_url,
-      const QueryRedirectsCallback& callback,
+      QueryRedirectsCallback callback,
       base::CancelableTaskTracker* tracker);
 
   // Schedules a query to get the most recent redirects ending at the given
   // URL.
   base::CancelableTaskTracker::TaskId QueryRedirectsTo(
       const GURL& to_url,
-      const QueryRedirectsCallback& callback,
+      QueryRedirectsCallback callback,
       base::CancelableTaskTracker* tracker);
 
   // Requests the number of user-visible visits (i.e. no redirects or subframes)
   // to all urls on the same scheme/host/port as |url|.  This is only valid for
   // HTTP and HTTPS URLs.
-  typedef base::Callback<void(
-      bool,         // Were we able to determine the # of visits?
-      int,          // Number of visits.
-      base::Time)>  // Time of first visit. Only set if bool
-                    // is true and int is > 0.
-      GetVisibleVisitCountToHostCallback;
+  using GetVisibleVisitCountToHostCallback =
+      base::OnceCallback<void(VisibleVisitCountToHostResult)>;
 
   base::CancelableTaskTracker::TaskId GetVisibleVisitCountToHost(
       const GURL& url,
-      const GetVisibleVisitCountToHostCallback& callback,
+      GetVisibleVisitCountToHostCallback callback,
       base::CancelableTaskTracker* tracker);
 
   // Request the |result_count| most visited URLs and the chain of
   // redirects leading to each of these URLs. |days_back| is the
   // number of days of history to use. Used by TopSites.
-  typedef base::Callback<void(const MostVisitedURLList*)>
-      QueryMostVisitedURLsCallback;
+  using QueryMostVisitedURLsCallback =
+      base::OnceCallback<void(MostVisitedURLList)>;
 
   base::CancelableTaskTracker::TaskId QueryMostVisitedURLs(
       int result_count,
       int days_back,
-      const QueryMostVisitedURLsCallback& callback,
+      QueryMostVisitedURLsCallback callback,
       base::CancelableTaskTracker* tracker);
 
   // Statistics ----------------------------------------------------------------
@@ -336,22 +307,37 @@ class HistoryService : public syncer::SyncableService, public KeyedService {
   // [|begin_time|, |end_time|). Each URL is counted only once per day. For
   // determination of the date, timestamps are converted to dates using local
   // time.
-  typedef base::Callback<void(HistoryCountResult)> GetHistoryCountCallback;
+  using GetHistoryCountCallback = base::OnceCallback<void(HistoryCountResult)>;
 
   base::CancelableTaskTracker::TaskId GetHistoryCount(
       const base::Time& begin_time,
       const base::Time& end_time,
-      const GetHistoryCountCallback& callback,
+      GetHistoryCountCallback callback,
+      base::CancelableTaskTracker* tracker);
+
+  // Returns, via a callback, the number of Hosts visited in the last month.
+  void CountUniqueHostsVisitedLastMonth(GetHistoryCountCallback callback,
+                                        base::CancelableTaskTracker* tracker);
+
+  using GetLastVisitToHostCallback =
+      base::OnceCallback<void(HistoryLastVisitToHostResult)>;
+
+  // Gets the last time any webpage on the given host was visited within the
+  // time range [|begin_time|, |end_time|). If the given host has not been
+  // visited in the given time age, the callback will be called with a null
+  // base::Time.
+  base::CancelableTaskTracker::TaskId GetLastVisitToHost(
+      const GURL& host,
+      base::Time begin_time,
+      base::Time end_time,
+      GetLastVisitToHostCallback callback,
       base::CancelableTaskTracker* tracker);
 
   // Database management operations --------------------------------------------
 
-  // Delete all the information related to a single url.
-  void DeleteURL(const GURL& url);
-
   // Delete all the information related to a list of urls.  (Deleting
   // URLs one by one is slow as it has to flush to disk each time.)
-  void DeleteURLsForTest(const std::vector<GURL>& urls);
+  void DeleteURLs(const std::vector<GURL>& urls);
 
   // Removes all visits in the selected time range (including the
   // start time), updating the URLs accordingly. This deletes any
@@ -364,7 +350,8 @@ class HistoryService : public syncer::SyncableService, public KeyedService {
   void ExpireHistoryBetween(const std::set<GURL>& restrict_urls,
                             base::Time begin_time,
                             base::Time end_time,
-                            const base::Closure& callback,
+                            bool user_initiated,
+                            base::OnceClosure callback,
                             base::CancelableTaskTracker* tracker);
 
   // Removes all visits to specified URLs in specific time ranges.
@@ -372,56 +359,63 @@ class HistoryService : public syncer::SyncableService, public KeyedService {
   // vector. The fields of |ExpireHistoryArgs| map directly to the arguments of
   // of ExpireHistoryBetween().
   void ExpireHistory(const std::vector<ExpireHistoryArgs>& expire_list,
-                     const base::Closure& callback,
+                     base::OnceClosure callback,
                      base::CancelableTaskTracker* tracker);
 
+  // Expires all visits before and including the given time, updating the URLs
+  // accordingly.
+  void ExpireHistoryBeforeForTesting(base::Time end_time,
+                                     base::OnceClosure callback,
+                                     base::CancelableTaskTracker* tracker);
+
   // Removes all visits to the given URLs in the specified time range. Calls
   // ExpireHistoryBetween() to delete local visits, and handles deletion of
   // synced visits if appropriate.
-  void ExpireLocalAndRemoteHistoryBetween(WebHistoryService* web_history,
-                                          const std::set<GURL>& restrict_urls,
+  void DeleteLocalAndRemoteHistoryBetween(WebHistoryService* web_history,
                                           base::Time begin_time,
                                           base::Time end_time,
-                                          const base::Closure& callback,
+                                          base::OnceClosure callback,
                                           base::CancelableTaskTracker* tracker);
 
+  // Removes all visits to the given url. Calls DeleteUrl() to delete local
+  // visits and handles deletion of synced visits if appropriate.
+  void DeleteLocalAndRemoteUrl(WebHistoryService* web_history, const GURL& url);
+
   // Processes the given |delete_directive| and sends it to the
-  // SyncChangeProcessor (if it exists).  Returns any error resulting
-  // from sending the delete directive to sync.
-  syncer::SyncError ProcessLocalDeleteDirective(
+  // SyncChangeProcessor (if it exists).
+  void ProcessLocalDeleteDirective(
       const sync_pb::HistoryDeleteDirectiveSpecifics& delete_directive);
 
   // Downloads -----------------------------------------------------------------
 
   // Implemented by the caller of 'CreateDownload' below, and is called when the
   // history service has created a new entry for a download in the history db.
-  typedef base::Callback<void(bool)> DownloadCreateCallback;
+  using DownloadCreateCallback = base::OnceCallback<void(bool)>;
 
   // Begins a history request to create a new row for a download. 'info'
   // contains all the download's creation state, and 'callback' runs when the
   // history service request is complete. The callback is called on the thread
   // that calls CreateDownload().
-  void CreateDownload(const DownloadRow& info,
-                      const DownloadCreateCallback& callback);
+  void CreateDownload(const DownloadRow& info, DownloadCreateCallback callback);
 
   // Implemented by the caller of 'GetNextDownloadId' below, and is called with
   // the maximum id of all downloads records in the database plus 1.
-  typedef base::Callback<void(uint32_t)> DownloadIdCallback;
+  using DownloadIdCallback = base::OnceCallback<void(uint32_t)>;
 
   // Responds on the calling thread with the maximum id of all downloads records
   // in the database plus 1.
-  void GetNextDownloadId(const DownloadIdCallback& callback);
+  void GetNextDownloadId(DownloadIdCallback callback);
 
   // Implemented by the caller of 'QueryDownloads' below, and is called when the
   // history service has retrieved a list of all download state. The call
-  typedef base::Callback<void(std::unique_ptr<std::vector<DownloadRow>>)>
-      DownloadQueryCallback;
+  using DownloadQueryCallback =
+      base::OnceCallback<void(std::vector<DownloadRow>)>;
 
   // Begins a history request to retrieve the state of all downloads in the
   // history db. 'callback' runs when the history service request is complete,
   // at which point 'info' contains an array of DownloadRow, one per
   // download. The callback is called on the thread that calls QueryDownloads().
-  void QueryDownloads(const DownloadQueryCallback& callback);
+  void QueryDownloads(DownloadQueryCallback callback);
 
   // Called to update the history service about the current state of a download.
   // This is a 'fire and forget' query, so just pass the relevant state info to
@@ -467,6 +461,7 @@ class HistoryService : public syncer::SyncableService, public KeyedService {
   // Schedules a HistoryDBTask for running on the history backend thread. See
   // HistoryDBTask for details on what this does. Takes ownership of |task|.
   virtual base::CancelableTaskTracker::TaskId ScheduleDBTask(
+      const base::Location& from_here,
       std::unique_ptr<HistoryDBTask> task,
       base::CancelableTaskTracker* tracker);
 
@@ -475,8 +470,8 @@ class HistoryService : public syncer::SyncableService, public KeyedService {
   // icon URL (e.g. http://www.google.com/favicon.ico) for which the favicon
   // data has changed. It is valid to call the callback with non-empty
   // "page URLs" and no "icon URL" and vice versa.
-  typedef base::Callback<void(const std::set<GURL>&, const GURL&)>
-      OnFaviconsChangedCallback;
+  using OnFaviconsChangedCallback =
+      base::RepeatingCallback<void(const std::set<GURL>&, const GURL&)>;
 
   // Add a callback to the list. The callback will remain registered until the
   // returned Subscription is destroyed. The Subscription must be destroyed
@@ -526,17 +521,26 @@ class HistoryService : public syncer::SyncableService, public KeyedService {
 
   base::WeakPtr<HistoryService> AsWeakPtr();
 
-  // syncer::SyncableService implementation.
-  syncer::SyncMergeResult MergeDataAndStartSyncing(
-      syncer::ModelType type,
-      const syncer::SyncDataList& initial_sync_data,
-      std::unique_ptr<syncer::SyncChangeProcessor> sync_processor,
-      std::unique_ptr<syncer::SyncErrorFactory> error_handler) override;
-  void StopSyncing(syncer::ModelType type) override;
-  syncer::SyncDataList GetAllSyncData(syncer::ModelType type) const override;
-  syncer::SyncError ProcessSyncChanges(
-      const tracked_objects::Location& from_here,
-      const syncer::SyncChangeList& change_list) override;
+  // For sync codebase only: returns the SyncableService API that implements
+  // sync datatype HISTORY_DELETE_DIRECTIVES.
+  base::WeakPtr<syncer::SyncableService> GetDeleteDirectivesSyncableService();
+
+  // For sync codebase only: instantiates a controller delegate to interact with
+  // TypedURLSyncBridge. Must be called from the UI thread.
+  std::unique_ptr<syncer::ModelTypeControllerDelegate>
+  GetTypedURLSyncControllerDelegate();
+
+  // Override |backend_task_runner_| for testing; needs to be called before
+  // Init.
+  void set_backend_task_runner_for_testing(
+      scoped_refptr<base::SequencedTaskRunner> task_runner) {
+    DCHECK(!backend_task_runner_);
+    backend_task_runner_ = std::move(task_runner);
+  }
+
+  void set_origin_queried_closure_for_testing(base::OnceClosure closure) {
+    origin_queried_closure_for_testing_ = std::move(closure);
+  }
 
  protected:
   // These are not currently used, hopefully we can do something in the future
@@ -584,7 +588,7 @@ class HistoryService : public syncer::SyncableService, public KeyedService {
   // will be called back on the internal history thread with the history
   // database so it can query. See history_url_provider.h for a diagram.
   void ScheduleAutocomplete(
-      const base::Callback<void(HistoryBackend*, URLDatabase*)>& callback);
+      base::OnceCallback<void(HistoryBackend*, URLDatabase*)> callback);
 
   // Notification from the backend that it has finished loading. Sends
   // notification (NOTIFY_HISTORY_LOADED) and sets backend_loaded_ to true.
@@ -608,16 +612,8 @@ class HistoryService : public syncer::SyncableService, public KeyedService {
   void NotifyURLsModified(const URLRows& changed_urls);
 
   // Notify all HistoryServiceObservers registered that URLs have been deleted.
-  // |all_history| is set to true, if all the URLs are deleted.
-  //               When set to true, |deleted_rows| and |favicon_urls| are
-  //               undefined.
-  // |expired| is set to true, if the URL deletion is due to expiration.
-  // |deleted_rows| list of the deleted URLs.
-  // |favicon_urls| list of favicon URLs that correspond to the deleted URLs.
-  void NotifyURLsDeleted(bool all_history,
-                         bool expired,
-                         const URLRows& deleted_rows,
-                         const std::set<GURL>& favicon_urls);
+  // |deletion_info| describes the urls that have been removed from history.
+  void NotifyURLsDeleted(const DeletionInfo& deletion_info);
 
   // Notify all HistoryServiceObservers registered that the
   // HistoryService has finished loading.
@@ -655,7 +651,7 @@ class HistoryService : public syncer::SyncableService, public KeyedService {
       const GURL& icon_url,
       favicon_base::IconType icon_type,
       const std::vector<int>& desired_sizes,
-      const favicon_base::FaviconResultsCallback& callback,
+      favicon_base::FaviconResultsCallback callback,
       base::CancelableTaskTracker* tracker);
 
   // Used by the FaviconService to get favicons mapped to |page_url| for
@@ -664,12 +660,18 @@ class HistoryService : public syncer::SyncableService, public KeyedService {
   // |icon_types| is returned. The returned FaviconBitmapResults will have at
   // most one result for each entry in |desired_sizes|. If a favicon bitmap is
   // determined to be the best candidate for multiple |desired_sizes| there
-  // will be fewer results.
+  // will be fewer results. If |fallback_to_host| is true, the host of
+  // |page_url| will be used to search the favicon database if an exact match
+  // cannot be found. Generally, code showing an icon for a full/previously
+  // visited URL should set |fallback_to_host|=false. Otherwise, if only a host
+  // is available, and any icon matching the host is permissible, use
+  // |fallback_to_host|=true.
   base::CancelableTaskTracker::TaskId GetFaviconsForURL(
       const GURL& page_url,
-      int icon_types,
+      const favicon_base::IconTypeSet& icon_types,
       const std::vector<int>& desired_sizes,
-      const favicon_base::FaviconResultsCallback& callback,
+      bool fallback_to_host,
+      favicon_base::FaviconResultsCallback callback,
       base::CancelableTaskTracker* tracker);
 
   // Used by FaviconService to find the first favicon bitmap whose width and
@@ -685,9 +687,9 @@ class HistoryService : public syncer::SyncableService, public KeyedService {
   // long as its size is larger than a specific value.
   base::CancelableTaskTracker::TaskId GetLargestFaviconForURL(
       const GURL& page_url,
-      const std::vector<int>& icon_types,
+      const std::vector<favicon_base::IconTypeSet>& icon_types,
       int minimum_size_in_pixels,
-      const favicon_base::FaviconRawBitmapCallback& callback,
+      favicon_base::FaviconRawBitmapCallback callback,
       base::CancelableTaskTracker* tracker);
 
   // Used by the FaviconService to get the favicon bitmap which most closely
@@ -697,23 +699,27 @@ class HistoryService : public syncer::SyncableService, public KeyedService {
   base::CancelableTaskTracker::TaskId GetFaviconForID(
       favicon_base::FaviconID favicon_id,
       int desired_size,
-      const favicon_base::FaviconResultsCallback& callback,
+      favicon_base::FaviconResultsCallback callback,
       base::CancelableTaskTracker* tracker);
 
-  // Maps |page_url| to the favicon at |icon_url| if there is an entry in the
+  // Maps |page_urls| to the favicon at |icon_url| if there is an entry in the
   // database for |icon_url| and |icon_type|. This occurs when there is a
   // mapping from a different page URL to |icon_url|. The favicon bitmaps whose
   // edge sizes most closely match |desired_sizes| from the favicons which were
-  // just mapped to |page_url| are returned. If |desired_sizes| has a '0' entry,
-  // the largest favicon bitmap is returned.
+  // just mapped to |page_urls| are returned. If |desired_sizes| has a '0'
+  // entry, the largest favicon bitmap is returned.
   base::CancelableTaskTracker::TaskId UpdateFaviconMappingsAndFetch(
-      const GURL& page_url,
+      const base::flat_set<GURL>& page_urls,
       const GURL& icon_url,
       favicon_base::IconType icon_type,
       const std::vector<int>& desired_sizes,
-      const favicon_base::FaviconResultsCallback& callback,
+      favicon_base::FaviconResultsCallback callback,
       base::CancelableTaskTracker* tracker);
 
+  // Deletes favicon mappings for each URL in |page_urls| and their redirects.
+  void DeleteFaviconMappings(const base::flat_set<GURL>& page_urls,
+                             favicon_base::IconType icon_type);
+
   // Used by FaviconService to set a favicon for |page_url| and |icon_url| with
   // |pixel_size|.
   // Example:
@@ -740,20 +746,36 @@ class HistoryService : public syncer::SyncableService, public KeyedService {
                     scoped_refptr<base::RefCountedMemory> bitmap_data,
                     const gfx::Size& pixel_size);
 
-  // Used by the FaviconService to replace all of the favicon bitmaps mapped to
-  // |page_url| for |icon_type|.
+  // Used by the FaviconService to replace the favicon bitmaps mapped to all
+  // URLs in |page_urls| for |icon_type|.
   // Use MergeFavicon() if |bitmaps| is incomplete, and favicon bitmaps in the
   // database should be preserved if possible. For instance, favicon bitmaps
   // from sync are 1x only. MergeFavicon() is used to avoid deleting the 2x
-  // favicon bitmap if it is present in the history backend.
-  void SetFavicons(const GURL& page_url,
+  // favicon bitmap if it is present in the history backend. |page_urls| must
+  // not be empty.
+  void SetFavicons(const base::flat_set<GURL>& page_urls,
                    favicon_base::IconType icon_type,
                    const GURL& icon_url,
                    const std::vector<SkBitmap>& bitmaps);
 
+  // Causes each page in |page_urls_to_write| to be associated to the same
+  // icon as the page |page_url_to_read| for icon types matching |icon_types|.
+  // No-op if |page_url_to_read| has no mappings for |icon_types|.
+  void CloneFaviconMappingsForPages(
+      const GURL& page_url_to_read,
+      const favicon_base::IconTypeSet& icon_types,
+      const base::flat_set<GURL>& page_urls_to_write);
+
+  // Figures out whether an on-demand favicon can be written for provided
+  // |page_url| and returns the result via |callback|. The result is false if
+  // there is an existing cached favicon for |icon_type| or if there is a
+  // non-expired icon of *any* type for |page_url|.
+  void CanSetOnDemandFavicons(const GURL& page_url,
+                              favicon_base::IconType icon_type,
+                              base::OnceCallback<void(bool)> callback);
+
   // Same as SetFavicons with three differences:
-  // 1) It will be a no-op if there is an existing cached favicon for *any* type
-  //    for |page_url|.
+  // 1) It will be a no-op if CanSetOnDemandFavicons() returns false.
   // 2) If |icon_url| is known to the database, |bitmaps| will be ignored (i.e.
   //    the icon won't be overwritten) but the mappings from |page_url| to
   //    |icon_url| will be stored (conditioned to point 1 above).
@@ -773,7 +795,7 @@ class HistoryService : public syncer::SyncableService, public KeyedService {
                            favicon_base::IconType icon_type,
                            const GURL& icon_url,
                            const std::vector<SkBitmap>& bitmaps,
-                           base::Callback<void(bool)> callback);
+                           base::OnceCallback<void(bool)> callback);
 
   // Used by the FaviconService to mark the favicon for the page as being out
   // of date.
@@ -828,7 +850,7 @@ class HistoryService : public syncer::SyncableService, public KeyedService {
 
   // This class has most of the implementation and runs on the 'thread_'.
   // You MUST communicate with this class ONLY through the thread_'s
-  // message_loop().
+  // task_runner().
   //
   // This pointer will be null once Cleanup() has been called, meaning no
   // more calls should be made to the history thread.
@@ -851,14 +873,16 @@ class HistoryService : public syncer::SyncableService, public KeyedService {
   // completed.
   bool backend_loaded_;
 
-  base::ObserverList<HistoryServiceObserver> observers_;
+  base::ObserverList<HistoryServiceObserver>::Unchecked observers_;
   base::CallbackList<void(const std::set<GURL>&, const GURL&)>
       favicon_changed_callback_list_;
 
-  DeleteDirectiveHandler delete_directive_handler_;
+  std::unique_ptr<DeleteDirectiveHandler> delete_directive_handler_;
+
+  base::OnceClosure origin_queried_closure_for_testing_;
 
   // All vended weak pointers are invalidated in Cleanup().
-  base::WeakPtrFactory<HistoryService> weak_ptr_factory_;
+  base::WeakPtrFactory<HistoryService> weak_ptr_factory_{this};
 
   DISALLOW_COPY_AND_ASSIGN(HistoryService);
 };
