@@ -4,12 +4,30 @@
 
 #include "components/optimization_guide/test_hints_component_creator.h"
 
+#include <memory>
+
+#include "base/base64.h"
 #include "base/files/file_util.h"
 #include "base/strings/string_number_conversions.h"
 #include "base/threading/thread_restrictions.h"
 #include "base/version.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
+namespace {
+
+std::string GetDefaultHintVersionString() {
+  optimization_guide::proto::Version hint_version;
+  hint_version.mutable_generation_timestamp()->set_seconds(123);
+  hint_version.set_hint_source(
+      optimization_guide::proto::HINT_SOURCE_OPTIMIZATION_HINTS_COMPONENT);
+  std::string hint_version_string;
+  hint_version.SerializeToString(&hint_version_string);
+  base::Base64Encode(hint_version_string, &hint_version_string);
+  return hint_version_string;
+}
+
+}  // namespace
+
 namespace optimization_guide {
 namespace testing {
 
@@ -33,23 +51,43 @@ TestHintsComponentCreator::CreateHintsComponentInfoWithPageHints(
     optimization_guide::proto::Hint* hint = config.add_hints();
     hint->set_key(page_hint_site);
     hint->set_key_representation(optimization_guide::proto::HOST_SUFFIX);
+    hint->set_version(GetDefaultHintVersionString());
 
     optimization_guide::proto::PageHint* page_hint = hint->add_page_hints();
     page_hint->set_page_pattern(page_pattern);
+    page_hint->set_max_ect_trigger(
+        optimization_guide::proto::EFFECTIVE_CONNECTION_TYPE_2G);
 
     optimization_guide::proto::Optimization* optimization =
         page_hint->add_whitelisted_optimizations();
     optimization->set_optimization_type(optimization_type);
+    optimization->mutable_previews_metadata()->set_max_ect_trigger(
+        optimization_guide::proto::EFFECTIVE_CONNECTION_TYPE_2G);
 
     for (auto resource_blocking_pattern : resource_blocking_patterns) {
       optimization_guide::proto::ResourceLoadingHint* resource_loading_hint =
-          optimization->add_resource_loading_hints();
+          optimization->mutable_previews_metadata()
+              ->add_resource_loading_hints();
       resource_loading_hint->set_loading_optimization_type(
           optimization_guide::proto::LOADING_BLOCK_RESOURCE);
       resource_loading_hint->set_resource_pattern(resource_blocking_pattern);
     }
   }
 
+  // Always stick something with no hint version in here.
+  optimization_guide::proto::Hint* no_version_hint = config.add_hints();
+  no_version_hint->set_key("noversion.com");
+  no_version_hint->set_key_representation(
+      optimization_guide::proto::HOST_SUFFIX);
+  no_version_hint->add_page_hints()->set_page_pattern("*");
+  // Always stick something with a bad hint version in here.
+  optimization_guide::proto::Hint* bad_version_hint = config.add_hints();
+  bad_version_hint->set_key("badversion.com");
+  bad_version_hint->set_key_representation(
+      optimization_guide::proto::HOST_SUFFIX);
+  bad_version_hint->set_version("notaversion");
+  bad_version_hint->add_page_hints()->set_page_pattern("*");
+
   return WriteConfigToFileAndReturnHintsComponentInfo(config);
 }
 
@@ -63,23 +101,42 @@ TestHintsComponentCreator::CreateHintsComponentInfoWithExperimentalPageHints(
     optimization_guide::proto::Hint* hint = config.add_hints();
     hint->set_key(page_hint_site);
     hint->set_key_representation(optimization_guide::proto::HOST_SUFFIX);
+    hint->set_version(GetDefaultHintVersionString());
 
     optimization_guide::proto::PageHint* page_hint = hint->add_page_hints();
     page_hint->set_page_pattern("*");
+    page_hint->set_max_ect_trigger(
+        optimization_guide::proto::EFFECTIVE_CONNECTION_TYPE_2G);
 
     optimization_guide::proto::Optimization* optimization =
         page_hint->add_whitelisted_optimizations();
     optimization->set_optimization_type(optimization_type);
     optimization->set_experiment_name(kFooExperimentName);
+    optimization->mutable_previews_metadata()->set_max_ect_trigger(
+        optimization_guide::proto::EFFECTIVE_CONNECTION_TYPE_2G);
 
     for (auto resource_blocking_pattern : experimental_resource_patterns) {
       optimization_guide::proto::ResourceLoadingHint* resource_loading_hint =
-          optimization->add_resource_loading_hints();
+          optimization->mutable_previews_metadata()
+              ->add_resource_loading_hints();
       resource_loading_hint->set_loading_optimization_type(
           optimization_guide::proto::LOADING_BLOCK_RESOURCE);
       resource_loading_hint->set_resource_pattern(resource_blocking_pattern);
     }
   }
+  // Always stick something with no hint version in here.
+  optimization_guide::proto::Hint* no_version_hint = config.add_hints();
+  no_version_hint->set_key("noversion.com");
+  no_version_hint->set_key_representation(
+      optimization_guide::proto::HOST_SUFFIX);
+  no_version_hint->add_page_hints()->set_page_pattern("*");
+  // Always stick something with a bad hint version in here.
+  optimization_guide::proto::Hint* bad_version_hint = config.add_hints();
+  bad_version_hint->set_key("badversion.com");
+  bad_version_hint->set_key_representation(
+      optimization_guide::proto::HOST_SUFFIX);
+  bad_version_hint->set_version("notaversion");
+  bad_version_hint->add_page_hints()->set_page_pattern("*");
 
   return WriteConfigToFileAndReturnHintsComponentInfo(config);
 }
@@ -95,9 +152,12 @@ TestHintsComponentCreator::CreateHintsComponentInfoWithMixPageHints(
     optimization_guide::proto::Hint* hint = config.add_hints();
     hint->set_key(page_hint_site);
     hint->set_key_representation(optimization_guide::proto::HOST_SUFFIX);
+    hint->set_version(GetDefaultHintVersionString());
 
     optimization_guide::proto::PageHint* page_hint = hint->add_page_hints();
     page_hint->set_page_pattern("*");
+    page_hint->set_max_ect_trigger(
+        optimization_guide::proto::EFFECTIVE_CONNECTION_TYPE_2G);
 
     // Add experimental patterns first so they get higher priority.
     {
@@ -105,10 +165,13 @@ TestHintsComponentCreator::CreateHintsComponentInfoWithMixPageHints(
           page_hint->add_whitelisted_optimizations();
       optimization->set_optimization_type(optimization_type);
       optimization->set_experiment_name(kFooExperimentName);
+      optimization->mutable_previews_metadata()->set_max_ect_trigger(
+          optimization_guide::proto::EFFECTIVE_CONNECTION_TYPE_2G);
 
       for (auto resource_blocking_pattern : experimental_resource_patterns) {
         optimization_guide::proto::ResourceLoadingHint* resource_loading_hint =
-            optimization->add_resource_loading_hints();
+            optimization->mutable_previews_metadata()
+                ->add_resource_loading_hints();
         resource_loading_hint->set_loading_optimization_type(
             optimization_guide::proto::LOADING_BLOCK_RESOURCE);
         resource_loading_hint->set_resource_pattern(resource_blocking_pattern);
@@ -119,16 +182,32 @@ TestHintsComponentCreator::CreateHintsComponentInfoWithMixPageHints(
       optimization_guide::proto::Optimization* optimization =
           page_hint->add_whitelisted_optimizations();
       optimization->set_optimization_type(optimization_type);
+      optimization->mutable_previews_metadata()->set_max_ect_trigger(
+          optimization_guide::proto::EFFECTIVE_CONNECTION_TYPE_2G);
 
       for (auto resource_blocking_pattern : default_resource_patterns) {
         optimization_guide::proto::ResourceLoadingHint* resource_loading_hint =
-            optimization->add_resource_loading_hints();
+            optimization->mutable_previews_metadata()
+                ->add_resource_loading_hints();
         resource_loading_hint->set_loading_optimization_type(
             optimization_guide::proto::LOADING_BLOCK_RESOURCE);
         resource_loading_hint->set_resource_pattern(resource_blocking_pattern);
       }
     }
   }
+  // Always stick something with no hint version in here.
+  optimization_guide::proto::Hint* no_version_hint = config.add_hints();
+  no_version_hint->set_key("noversion.com");
+  no_version_hint->set_key_representation(
+      optimization_guide::proto::HOST_SUFFIX);
+  no_version_hint->add_page_hints()->set_page_pattern("*");
+  // Always stick something with a bad hint version in here.
+  optimization_guide::proto::Hint* bad_version_hint = config.add_hints();
+  bad_version_hint->set_key("badversion.com");
+  bad_version_hint->set_key_representation(
+      optimization_guide::proto::HOST_SUFFIX);
+  bad_version_hint->set_version("notaversion");
+  bad_version_hint->add_page_hints()->set_page_pattern("*");
 
   return WriteConfigToFileAndReturnHintsComponentInfo(config);
 }
