@@ -3,10 +3,7 @@
 // found in the LICENSE file.
 #include "components/password_manager/core/browser/credential_manager_impl.h"
 
-#include <memory>
 #include <string>
-#include <utility>
-#include <vector>
 
 #include "base/bind.h"
 #include "base/metrics/user_metrics.h"
@@ -27,7 +24,7 @@ void RunGetCallback(GetCallback callback, const CredentialInfo& info) {
 }  // namespace
 
 CredentialManagerImpl::CredentialManagerImpl(PasswordManagerClient* client)
-    : client_(client) {
+    : client_(client), leak_delegate_(client) {
   auto_signin_enabled_.Init(prefs::kCredentialsEnableAutosignin,
                             client_->GetPrefs());
 }
@@ -56,6 +53,10 @@ void CredentialManagerImpl::Store(const CredentialInfo& credential,
   std::unique_ptr<autofill::PasswordForm> form(
       CreatePasswordFormFromCredentialInfo(credential, origin));
 
+  // Check whether a stored password credential was leaked.
+  if (credential.type == CredentialType::CREDENTIAL_TYPE_PASSWORD)
+    leak_delegate_.StartLeakCheck(*form);
+
   std::string signon_realm = origin.GetOrigin().spec();
   PasswordStore::FormDigest observed_digest(
       autofill::PasswordForm::Scheme::kHtml, signon_realm, origin);
@@ -206,7 +207,7 @@ PasswordManagerClient* CredentialManagerImpl::client() const {
 }
 
 PasswordStore* CredentialManagerImpl::GetPasswordStore() {
-  return client_ ? client_->GetPasswordStore() : nullptr;
+  return client_ ? client_->GetProfilePasswordStore() : nullptr;
 }
 
 void CredentialManagerImpl::DoneRequiringUserMediation() {
