@@ -6,18 +6,20 @@
 
 #include <stdint.h>
 
-#include <memory>
+#include <algorithm>
+#include <map>
 #include <string>
 #include <tuple>
-#include <utility>
-#include <vector>
 
 #include "base/bind.h"
 #include "base/macros.h"
 #include "base/strings/string16.h"
 #include "base/strings/utf_string_conversions.h"
-#include "base/test/scoped_task_environment.h"
+#include "base/test/task_environment.h"
+#include "build/build_config.h"
 #include "components/password_manager/core/browser/android_affiliation/mock_affiliated_match_helper.h"
+#include "components/password_manager/core/browser/leak_detection/leak_detection_check.h"
+#include "components/password_manager/core/browser/leak_detection/leak_detection_check_factory.h"
 #include "components/password_manager/core/browser/password_manager.h"
 #include "components/password_manager/core/browser/stub_password_manager_client.h"
 #include "components/password_manager/core/browser/test_password_store.h"
@@ -25,6 +27,8 @@
 #include "components/password_manager/core/common/password_manager_pref_names.h"
 #include "components/prefs/pref_registry_simple.h"
 #include "components/prefs/testing_pref_service.h"
+#include "components/safe_browsing/common/safe_browsing_prefs.h"
+#include "services/network/public/cpp/shared_url_loader_factory.h"
 #include "testing/gmock/include/gmock/gmock.h"
 #include "testing/gtest/include/gtest/gtest.h"
 #include "url/gurl.h"
@@ -32,6 +36,7 @@
 
 using testing::_;
 using testing::ElementsAre;
+using testing::Pair;
 using testing::Pointee;
 using testing::UnorderedElementsAre;
 
@@ -43,6 +48,20 @@ const char kTestWebOrigin[] = "https://example.com/";
 const char kTestAndroidRealm1[] = "android://hash@com.example.one.android/";
 const char kTestAndroidRealm2[] = "android://hash@com.example.two.android/";
 
+class MockLeakDetectionCheck : public LeakDetectionCheck {
+ public:
+  MOCK_METHOD3(Start, void(const GURL&, base::string16, base::string16));
+};
+
+class MockLeakDetectionCheckFactory : public LeakDetectionCheckFactory {
+ public:
+  MOCK_CONST_METHOD3(TryCreateLeakCheck,
+                     std::unique_ptr<LeakDetectionCheck>(
+                         LeakDetectionDelegateInterface*,
+                         signin::IdentityManager*,
+                         scoped_refptr<network::SharedURLLoaderFactory>));
+};
+
 class MockPasswordManagerClient : public StubPasswordManagerClient {
  public:
   MOCK_CONST_METHOD1(IsSavingAndFillingEnabled, bool(const GURL&));
@@ -58,6 +77,10 @@ class MockPasswordManagerClient : public StubPasswordManagerClient {
                bool(const std::vector<autofill::PasswordForm*>& local_forms,
                     const GURL& origin,
                     const CredentialsCallback& callback));
+  MOCK_METHOD3(PasswordWasAutofilled,
+               void(const std::vector<const autofill::PasswordForm*>&,
+                    const GURL&,
+                    const std::vector<const autofill::PasswordForm*>*));
 
   explicit MockPasswordManagerClient(PasswordStore* store)
       : store_(store), password_manager_(this) {
@@ -66,6 +89,12 @@ class MockPasswordManagerClient : public StubPasswordManagerClient {
                                             true);
     prefs_->registry()->RegisterBooleanPref(
         prefs::kWasAutoSignInFirstRunExperienceShown, true);
+    prefs_->registry()->RegisterBooleanPref(
+        prefs::kPasswordLeakDetectionEnabled, true);
+#if !defined(OS_IOS)
+    prefs_->registry()->RegisterBooleanPref(::prefs::kSafeBrowsingEnabled,
+                                            true);
+#endif
   }
   ~MockPasswordManagerClient() override {}
 
@@ -82,7 +111,7 @@ class MockPasswordManagerClient : public StubPasswordManagerClient {
     NotifyUserCouldBeAutoSignedInPtr(form.get());
   }
 
-  PasswordStore* GetPasswordStore() const override { return store_; }
+  PasswordStore* GetProfilePasswordStore() const override { return store_; }
 
   PrefService* GetPrefs() const override { return prefs_.get(); }
 
@@ -191,7 +220,7 @@ class CredentialManagerImplTest : public testing::Test {
     form_.password_value = base::ASCIIToUTF16("Password");
     form_.origin = client_->GetLastCommittedEntryURL();
     form_.signon_realm = form_.origin.GetOrigin().spec();
-    form_.scheme = autofill::PasswordForm::SCHEME_HTML;
+    form_.scheme = autofill::PasswordForm::Scheme::kHtml;
     form_.skip_zero_click = false;
 
     affiliated_form1_.username_value = base::ASCIIToUTF16("Affiliated 1");
@@ -199,7 +228,7 @@ class CredentialManagerImplTest : public testing::Test {
     affiliated_form1_.password_value = base::ASCIIToUTF16("Password");
     affiliated_form1_.origin = GURL();
     affiliated_form1_.signon_realm = kTestAndroidRealm1;
-    affiliated_form1_.scheme = autofill::PasswordForm::SCHEME_HTML;
+    affiliated_form1_.scheme = autofill::PasswordForm::Scheme::kHtml;
     affiliated_form1_.skip_zero_click = false;
 
     affiliated_form2_.username_value = base::ASCIIToUTF16("Affiliated 2");
@@ -207,7 +236,7 @@ class CredentialManagerImplTest : public testing::Test {
     affiliated_form2_.password_value = base::ASCIIToUTF16("Password");
     affiliated_form2_.origin = GURL();
     affiliated_form2_.signon_realm = kTestAndroidRealm2;
-    affiliated_form2_.scheme = autofill::PasswordForm::SCHEME_HTML;
+    affiliated_form2_.scheme = autofill::PasswordForm::Scheme::kHtml;
     affiliated_form2_.skip_zero_click = false;
 
     origin_path_form_.username_value = base::ASCIIToUTF16("Username 2");
@@ -216,7 +245,7 @@ class CredentialManagerImplTest : public testing::Test {
     origin_path_form_.origin = GURL("https://example.com/path");
     origin_path_form_.signon_realm =
         origin_path_form_.origin.GetOrigin().spec();
-    origin_path_form_.scheme = autofill::PasswordForm::SCHEME_HTML;
+    origin_path_form_.scheme = autofill::PasswordForm::Scheme::kHtml;
     origin_path_form_.skip_zero_click = false;
 
     subdomain_form_.username_value = base::ASCIIToUTF16("Username 2");
@@ -224,7 +253,7 @@ class CredentialManagerImplTest : public testing::Test {
     subdomain_form_.password_value = base::ASCIIToUTF16("Password 2");
     subdomain_form_.origin = GURL("https://subdomain.example.com/path");
     subdomain_form_.signon_realm = subdomain_form_.origin.GetOrigin().spec();
-    subdomain_form_.scheme = autofill::PasswordForm::SCHEME_HTML;
+    subdomain_form_.scheme = autofill::PasswordForm::Scheme::kHtml;
     subdomain_form_.skip_zero_click = false;
 
     cross_origin_form_.username_value = base::ASCIIToUTF16("Username");
@@ -233,7 +262,7 @@ class CredentialManagerImplTest : public testing::Test {
     cross_origin_form_.origin = GURL("https://example.net/");
     cross_origin_form_.signon_realm =
         cross_origin_form_.origin.GetOrigin().spec();
-    cross_origin_form_.scheme = autofill::PasswordForm::SCHEME_HTML;
+    cross_origin_form_.scheme = autofill::PasswordForm::Scheme::kHtml;
     cross_origin_form_.skip_zero_click = false;
 
     store_->Clear();
@@ -327,10 +356,10 @@ class CredentialManagerImplTest : public testing::Test {
                           std::move(callback));
   }
 
-  void RunAllPendingTasks() { scoped_task_environment_.RunUntilIdle(); }
+  void RunAllPendingTasks() { task_environment_.RunUntilIdle(); }
 
  protected:
-  base::test::ScopedTaskEnvironment scoped_task_environment_;
+  base::test::TaskEnvironment task_environment_;
   autofill::PasswordForm form_;
   autofill::PasswordForm affiliated_form1_;
   autofill::PasswordForm affiliated_form2_;
@@ -375,8 +404,6 @@ TEST_F(CredentialManagerImplTest, CredentialManagerOnStore) {
   RunAllPendingTasks();
 
   EXPECT_TRUE(called);
-  EXPECT_EQ(FormFetcher::State::NOT_WAITING,
-            client_->pending_manager()->GetFormFetcher()->GetState());
 
   autofill::PasswordForm new_form =
       client_->pending_manager()->GetPendingCredentials();
@@ -388,7 +415,7 @@ TEST_F(CredentialManagerImplTest, CredentialManagerOnStore) {
   EXPECT_TRUE(new_form.federation_origin.opaque());
   EXPECT_EQ(form_.icon_url, new_form.icon_url);
   EXPECT_FALSE(form_.skip_zero_click);
-  EXPECT_EQ(autofill::PasswordForm::SCHEME_HTML, new_form.scheme);
+  EXPECT_EQ(autofill::PasswordForm::Scheme::kHtml, new_form.scheme);
 }
 
 TEST_F(CredentialManagerImplTest, CredentialManagerOnStoreFederated) {
@@ -408,8 +435,6 @@ TEST_F(CredentialManagerImplTest, CredentialManagerOnStoreFederated) {
   RunAllPendingTasks();
 
   EXPECT_TRUE(called);
-  EXPECT_EQ(FormFetcher::State::NOT_WAITING,
-            client_->pending_manager()->GetFormFetcher()->GetState());
 
   autofill::PasswordForm new_form =
       client_->pending_manager()->GetPendingCredentials();
@@ -421,7 +446,7 @@ TEST_F(CredentialManagerImplTest, CredentialManagerOnStoreFederated) {
   EXPECT_EQ(form_.federation_origin, new_form.federation_origin);
   EXPECT_EQ(form_.icon_url, new_form.icon_url);
   EXPECT_FALSE(form_.skip_zero_click);
-  EXPECT_EQ(autofill::PasswordForm::SCHEME_HTML, new_form.scheme);
+  EXPECT_EQ(autofill::PasswordForm::Scheme::kHtml, new_form.scheme);
 }
 
 TEST_F(CredentialManagerImplTest, StoreFederatedAfterPassword) {
@@ -430,7 +455,7 @@ TEST_F(CredentialManagerImplTest, StoreFederatedAfterPassword) {
 
   autofill::PasswordForm federated = form_;
   federated.password_value.clear();
-  federated.type = autofill::PasswordForm::TYPE_API;
+  federated.type = autofill::PasswordForm::Type::kApi;
   federated.preferred = true;
   federated.federation_origin =
       url::Origin::Create(GURL("https://google.com/"));
@@ -447,8 +472,6 @@ TEST_F(CredentialManagerImplTest, StoreFederatedAfterPassword) {
   RunAllPendingTasks();
 
   EXPECT_TRUE(called);
-  EXPECT_EQ(FormFetcher::State::NOT_WAITING,
-            client_->pending_manager()->GetFormFetcher()->GetState());
   client_->pending_manager()->Save();
 
   RunAllPendingTasks();
@@ -456,6 +479,8 @@ TEST_F(CredentialManagerImplTest, StoreFederatedAfterPassword) {
   EXPECT_THAT(passwords["https://example.com/"], ElementsAre(form_));
   federated.date_created =
       passwords["federation://example.com/google.com"][0].date_created;
+  federated.date_last_used =
+      passwords["federation://example.com/google.com"][0].date_last_used;
   EXPECT_THAT(passwords["federation://example.com/google.com"],
               ElementsAre(federated));
 }
@@ -970,7 +995,6 @@ TEST_F(CredentialManagerImplTest,
   std::vector<GURL> federations;
 
   EXPECT_CALL(*client_, NotifyUserCouldBeAutoSignedInPtr(_)).Times(0);
-
   ExpectZeroClickSignInFailure(CredentialMediationRequirement::kSilent, false,
                                federations);
 }
@@ -1507,7 +1531,7 @@ TEST_F(CredentialManagerImplTest, GetSynthesizedFormForOrigin) {
       cm_service_impl_->GetSynthesizedFormForOrigin();
   EXPECT_EQ(kTestWebOrigin, synthesized.origin.spec());
   EXPECT_EQ(kTestWebOrigin, synthesized.signon_realm);
-  EXPECT_EQ(autofill::PasswordForm::SCHEME_HTML, synthesized.scheme);
+  EXPECT_EQ(autofill::PasswordForm::Scheme::kHtml, synthesized.scheme);
 }
 
 TEST_F(CredentialManagerImplTest, GetBlacklistedPasswordCredential) {
@@ -1548,7 +1572,6 @@ TEST_F(CredentialManagerImplTest, BlacklistPasswordCredential) {
   blacklisted.blacklisted_by_user = true;
   blacklisted.origin = form_.origin;
   blacklisted.signon_realm = form_.signon_realm;
-  blacklisted.type = autofill::PasswordForm::TYPE_API;
   blacklisted.date_created = passwords[form_.signon_realm][0].date_created;
   EXPECT_THAT(passwords[form_.signon_realm], testing::ElementsAre(blacklisted));
 }
@@ -1577,7 +1600,6 @@ TEST_F(CredentialManagerImplTest, BlacklistFederatedCredential) {
   blacklisted.blacklisted_by_user = true;
   blacklisted.origin = form_.origin;
   blacklisted.signon_realm = blacklisted.origin.spec();
-  blacklisted.type = autofill::PasswordForm::TYPE_API;
   blacklisted.date_created =
       passwords[blacklisted.signon_realm][0].date_created;
   EXPECT_THAT(passwords[blacklisted.signon_realm],
@@ -1623,4 +1645,71 @@ TEST_F(CredentialManagerImplTest, RespectBlacklistingFederatedCredential) {
   EXPECT_TRUE(client_->pending_manager()->IsBlacklisted());
 }
 
+TEST_F(CredentialManagerImplTest,
+       ManagePasswordsUICredentialsUpdatedUnconditionallyInSilentMediation) {
+  autofill::PasswordForm federated = origin_path_form_;
+  federated.federation_origin =
+      url::Origin::Create(GURL("https://google.com/"));
+  federated.signon_realm =
+      "federation://" + federated.origin.host() + "/google.com";
+  store_->AddLogin(federated);
+
+  form_.username_value = base::ASCIIToUTF16("username_value");
+  store_->AddLogin(form_);
+
+  EXPECT_CALL(*client_,
+              PasswordWasAutofilled(ElementsAre(Pointee(form_)), _,
+                                    Pointee(ElementsAre(Pointee(federated)))));
+
+  bool called = false;
+  CredentialManagerError error;
+  base::Optional<CredentialInfo> credential;
+  std::vector<GURL> federations;
+  federations.push_back(GURL("https://google.com/"));
+
+  CallGet(CredentialMediationRequirement::kSilent, true, federations,
+          base::BindOnce(&GetCredentialCallback, &called, &error, &credential));
+
+  RunAllPendingTasks();
+}
+
+#if !defined(OS_IOS)
+// Check that following a call to store() a federated credential is not checked
+// for leaks.
+TEST_F(CredentialManagerImplTest,
+       StoreFederatedCredentialDoesNotStartLeakDetection) {
+  auto mock_factory =
+      std::make_unique<testing::StrictMock<MockLeakDetectionCheckFactory>>();
+  EXPECT_CALL(*mock_factory, TryCreateLeakCheck).Times(0);
+  cm_service_impl()->set_leak_factory(std::move(mock_factory));
+
+  form_.federation_origin = url::Origin::Create(GURL("https://example.com/"));
+  form_.password_value = base::string16();
+  form_.signon_realm = "federation://example.com/example.com";
+  CallStore({form_, CredentialType::CREDENTIAL_TYPE_FEDERATED},
+            base::DoNothing());
+
+  RunAllPendingTasks();
+}
+
+// Check that following a call to store() a password credential is checked for
+// leaks.
+TEST_F(CredentialManagerImplTest, StorePasswordCredentialStartsLeakDetection) {
+  auto mock_factory =
+      std::make_unique<testing::StrictMock<MockLeakDetectionCheckFactory>>();
+  auto* weak_factory = mock_factory.get();
+  cm_service_impl()->set_leak_factory(std::move(mock_factory));
+
+  auto check_instance = std::make_unique<MockLeakDetectionCheck>();
+  EXPECT_CALL(*check_instance,
+              Start(form_.origin, form_.username_value, form_.password_value));
+  EXPECT_CALL(*weak_factory, TryCreateLeakCheck)
+      .WillOnce(testing::Return(testing::ByMove(std::move(check_instance))));
+  CallStore({form_, CredentialType::CREDENTIAL_TYPE_PASSWORD},
+            base::DoNothing());
+
+  RunAllPendingTasks();
+}
+#endif  // !defined(OS_IOS)
+
 }  // namespace password_manager
