@@ -7,6 +7,7 @@
 #include <string>
 #include <utility>
 
+#include "base/bind.h"
 #include "base/memory/weak_ptr.h"
 #include "base/stl_util.h"
 #include "base/strings/strcat.h"
@@ -46,10 +47,10 @@ HttpPasswordStoreMigrator::HttpPasswordStoreMigrator(
   GURL::Replacements rep;
   rep.SetSchemeStr(url::kHttpScheme);
   GURL http_origin = https_origin.ReplaceComponents(rep);
-  PasswordStore::FormDigest form(autofill::PasswordForm::SCHEME_HTML,
+  PasswordStore::FormDigest form(autofill::PasswordForm::Scheme::kHtml,
                                  http_origin.GetOrigin().spec(), http_origin);
   http_origin_domain_ = http_origin.GetOrigin();
-  client_->GetPasswordStore()->GetLogins(form, this);
+  client_->GetProfilePasswordStore()->GetLogins(form, this);
   client_->PostHSTSQueryForHost(
       https_origin, base::Bind(&OnHSTSQueryResultHelper, GetWeakPtr()));
 }
@@ -78,7 +79,8 @@ autofill::PasswordForm HttpPasswordStoreMigrator::MigrateHttpFormToHttps(
   if (!http_form.action.SchemeIs(url::kHttpsScheme))
     https_form.action = https_form.origin;
   https_form.form_data = autofill::FormData();
-  https_form.generation_upload_status = autofill::PasswordForm::NO_SIGNAL_SENT;
+  https_form.generation_upload_status =
+      autofill::PasswordForm::GenerationUploadStatus::kNoSignalSent;
   https_form.skip_zero_click = false;
   return https_form;
 }
@@ -100,7 +102,7 @@ void HttpPasswordStoreMigrator::OnHSTSQueryResult(HSTSResult is_hsts) {
   got_hsts_query_result_ = true;
 
   if (is_hsts == HSTSResult::kYes)
-    client_->GetPasswordStore()->RemoveSiteStats(http_origin_domain_);
+    client_->GetProfilePasswordStore()->RemoveSiteStats(http_origin_domain_);
 
   if (got_password_store_results_)
     ProcessPasswordStoreResults();
@@ -118,10 +120,10 @@ void HttpPasswordStoreMigrator::ProcessPasswordStoreResults() {
   for (const auto& form : results_) {
     autofill::PasswordForm new_form =
         HttpPasswordStoreMigrator::MigrateHttpFormToHttps(*form);
-    client_->GetPasswordStore()->AddLogin(new_form);
+    client_->GetProfilePasswordStore()->AddLogin(new_form);
 
     if (mode_ == MigrationMode::MOVE)
-      client_->GetPasswordStore()->RemoveLogin(*form);
+      client_->GetProfilePasswordStore()->RemoveLogin(*form);
     *form = std::move(new_form);
   }
 
