@@ -14,13 +14,16 @@
 #include "base/macros.h"
 #include "base/pickle.h"
 #include "base/strings/string16.h"
+#include "base/util/type_safety/strong_alias.h"
 #include "build/build_config.h"
+#include "components/password_manager/core/browser/compromised_credentials_table.h"
+#include "components/password_manager/core/browser/field_info_table.h"
 #include "components/password_manager/core/browser/password_store.h"
 #include "components/password_manager/core/browser/password_store_change.h"
+#include "components/password_manager/core/browser/password_store_sync.h"
 #include "components/password_manager/core/browser/psl_matching_helper.h"
 #include "components/password_manager/core/browser/statistics_table.h"
 #include "components/sync/model/metadata_batch.h"
-#include "components/sync/model/sync_metadata_store.h"
 #include "components/sync/protocol/model_type_state.pb.h"
 #include "sql/database.h"
 #include "sql/meta_table.h"
@@ -40,14 +43,23 @@ class SQLTableBuilder;
 extern const int kCurrentVersionNumber;
 extern const int kCompatibleVersionNumber;
 
+using IsAccountStore = util::StrongAlias<class IsAccountStoreTag, bool>;
+
 // Interface to the database storage of login information, intended as a helper
 // for PasswordStore on platforms that need internal storage of some or all of
 // the login information.
-class LoginDatabase : public syncer::SyncMetadataStore {
+class LoginDatabase : public PasswordStoreSync::MetadataStore {
  public:
-  explicit LoginDatabase(const base::FilePath& db_path);
+  LoginDatabase(const base::FilePath& db_path, IsAccountStore is_account_store);
   ~LoginDatabase() override;
 
+  // Returns whether this is the profile-scoped or the account-scoped storage:
+  // true:  Gaia-account-scoped store, which is used for signed-in but not
+  //        syncing users.
+  // false: Profile-scoped store, which is used for local storage and for
+  //        syncing users.
+  bool is_account_store() const { return is_account_store_.value(); }
+
   // Actually creates/opens the database. If false is returned, no other method
   // should be called.
   virtual bool Init();
@@ -65,8 +77,10 @@ class LoginDatabase : public syncer::SyncMetadataStore {
   // Adds |form| to the list of remembered password forms. Returns the list of
   // changes applied ({}, {ADD}, {REMOVE, ADD}). If it returns {REMOVE, ADD}
   // then the REMOVE is associated with the form that was added. Thus only the
-  // primary key columns contain the values associated with the removed form.
-  PasswordStoreChangeList AddLogin(const autofill::PasswordForm& form)
+  // primary key columns contain the values associated with the removed form. In
+  // case of error, it sets |error| if |error| isn't null.
+  PasswordStoreChangeList AddLogin(const autofill::PasswordForm& form,
+                                   AddLoginError* error = nullptr)
       WARN_UNUSED_RESULT;
 
   // This function does the same thing as AddLogin() with the difference that
@@ -75,11 +89,13 @@ class LoginDatabase : public syncer::SyncMetadataStore {
   PasswordStoreChangeList AddBlacklistedLoginForTesting(
       const autofill::PasswordForm& form) WARN_UNUSED_RESULT;
 
-  // Updates existing password form. Returns the list of applied changes
-  // ({}, {UPDATE}). The password is looked up by the tuple {origin,
-  // username_element, username_value, password_element, signon_realm}.
-  // These columns stay intact.
-  PasswordStoreChangeList UpdateLogin(const autofill::PasswordForm& form)
+  // Updates existing password form. Returns the list of applied changes ({},
+  // {UPDATE}). The password is looked up by the tuple {origin,
+  // username_element, username_value, password_element, signon_realm}. These
+  // columns stay intact. In case of error, it sets |error| if |error| isn't
+  // null.
+  PasswordStoreChangeList UpdateLogin(const autofill::PasswordForm& form,
+                                      UpdateLoginError* error = nullptr)
       WARN_UNUSED_RESULT;
 
   // Removes |form| from the list of remembered password forms. Returns true if
@@ -103,14 +119,6 @@ class LoginDatabase : public syncer::SyncMetadataStore {
                                   base::Time delete_end,
                                   PasswordStoreChangeList* changes);
 
-  // Removes all logins synced from |delete_begin| onwards (inclusive) and
-  // before |delete_end|. You may use a null Time value to do an unbounded
-  // delete in either direction. If |changes| is not be null, it will be used to
-  // populate the change list of the removed forms if any.
-  bool RemoveLoginsSyncedBetween(base::Time delete_begin,
-                                 base::Time delete_end,
-                                 PasswordStoreChangeList* changes);
-
   // Sets the 'skip_zero_click' flag on all forms on |origin| to 'true'.
   bool DisableAutoSignInForOrigin(const GURL& origin);
 
@@ -123,17 +131,10 @@ class LoginDatabase : public syncer::SyncMetadataStore {
                  std::vector<std::unique_ptr<autofill::PasswordForm>>* forms)
       WARN_UNUSED_RESULT;
 
-  // Retrieves all stored credentials with SCHEME_HTTP that have a realm whose
-  // organization-identifying name -- that is, the first domain name label below
-  // the effective TLD -- matches that of |signon_realm|. Return value indicates
-  // a successful query (but potentially no results).
-  //
-  // For example, the organization-identifying name of "https://foo.example.org"
-  // is `example`, and logins will be returned for "http://bar.example.co.uk",
-  // but not for "http://notexample.com" or "https://example.foo.com".
-  bool GetLoginsForSameOrganizationName(
-      const std::string& signon_realm,
-      std::vector<std::unique_ptr<autofill::PasswordForm>>* forms);
+  // Gets a list of credentials with password_value=|plain_text_password|.
+  bool GetLoginsByPassword(const base::string16& plain_text_password,
+                           std::vector<std::unique_ptr<autofill::PasswordForm>>*
+                               forms) WARN_UNUSED_RESULT;
 
   // Gets all logins created from |begin| onwards (inclusive) and before |end|.
   // You may use a null Time value to do an unbounded search in either
@@ -145,7 +146,8 @@ class LoginDatabase : public syncer::SyncMetadataStore {
       WARN_UNUSED_RESULT;
 
   // Gets the complete list of all credentials.
-  bool GetAllLogins(PrimaryKeyToFormMap* key_to_form_map) WARN_UNUSED_RESULT;
+  FormRetrievalResult GetAllLogins(PrimaryKeyToFormMap* key_to_form_map)
+      WARN_UNUSED_RESULT;
 
   // Gets the complete list of not blacklisted credentials.
   bool GetAutofillableLogins(
@@ -157,8 +159,8 @@ class LoginDatabase : public syncer::SyncMetadataStore {
                               forms) WARN_UNUSED_RESULT;
 
   // Gets the list of auto-sign-inable credentials.
-  bool GetAutoSignInLogins(std::vector<std::unique_ptr<autofill::PasswordForm>>*
-                               forms) WARN_UNUSED_RESULT;
+  bool GetAutoSignInLogins(PrimaryKeyToFormMap* key_to_form_map)
+      WARN_UNUSED_RESULT;
 
   // Deletes the login database file on disk, and creates a new, empty database.
   // This can be used after migrating passwords to some other store, to ensure
@@ -167,6 +169,8 @@ class LoginDatabase : public syncer::SyncMetadataStore {
   // whether further use of this login database will succeed is unspecified.
   bool DeleteAndRecreateDatabaseFile();
 
+  bool IsEmpty();
+
   // On MacOS, it deletes all logins from the database that cannot be decrypted
   // when encryption key from Keychain is available. If the Keychain is locked,
   // it does nothing and returns ENCRYPTION_UNAVAILABLE. If it's not running on
@@ -180,7 +184,9 @@ class LoginDatabase : public syncer::SyncMetadataStore {
   // empty string if the row for this |form| is not found.
   std::string GetEncryptedPassword(const autofill::PasswordForm& form) const;
 
-  // syncer::SyncMetadataStore implementation.
+  // PasswordStoreSync::MetadataStore implementation.
+  std::unique_ptr<syncer::MetadataBatch> GetAllSyncMetadata() override;
+  void DeleteAllSyncMetadata() override;
   bool UpdateSyncMetadata(syncer::ModelType model_type,
                           const std::string& storage_key,
                           const sync_pb::EntityMetadata& metadata) override;
@@ -191,21 +197,26 @@ class LoginDatabase : public syncer::SyncMetadataStore {
       const sync_pb::ModelTypeState& model_type_state) override;
   bool ClearModelTypeState(syncer::ModelType model_type) override;
 
-  // Callers that requires transaction support should call there methods to
-  // begin and commit transactions. They delegate to the transaction support of
-  // the underlying database. Only one transaction may exist at a time.
+  // Callers that requires transaction support should call these methods to
+  // begin, rollback and commit transactions. They delegate to the transaction
+  // support of the underlying database. Only one transaction may exist at a
+  // time.
   bool BeginTransaction();
+  void RollbackTransaction();
   bool CommitTransaction();
 
-  // Returns all the stored sync metadata. Returns null in case of failure.
-  std::unique_ptr<syncer::MetadataBatch> GetAllSyncMetadataForTesting();
-
   StatisticsTable& stats_table() { return stats_table_; }
+  CompromisedCredentialsTable& compromised_credentials_table() {
+    return compromised_credentials_table_;
+  }
+
+  FieldInfoTable& field_info_table() { return field_info_table_; }
 
 #if defined(OS_POSIX) && !defined(OS_MACOSX)
+  void enable_encryption() { use_encryption_ = true; }
   // This instance should not encrypt/decrypt password values using OSCrypt.
   void disable_encryption() { use_encryption_ = false; }
-#endif  // defined(OS_POSIX)
+#endif  // defined(OS_POSIX) && !defined(OS_MACOSX)
 
  private:
 #if defined(OS_IOS)
@@ -257,11 +268,15 @@ class LoginDatabase : public syncer::SyncMetadataStore {
 
   // Fills |form| from the values in the given statement (which is assumed to be
   // of the form used by the Get*Logins methods). Fills the corresponding DB
-  // primary key in |primary_key|. Returns the EncryptionResult from decrypting
-  // the password in |s|; if not ENCRYPTION_RESULT_SUCCESS, |form| is not
-  // filled.
+  // primary key in |primary_key|. If |decrypt_and_fill_password_value| is set
+  // to true, it tries to decrypt the stored password and returns the
+  // EncryptionResult from decrypting the password in |s|; if not
+  // ENCRYPTION_RESULT_SUCCESS, |form| is not filled. If
+  // |decrypt_and_fill_password_value| is set to false, it always returns
+  // ENCRYPTION_RESULT_SUCCESS.
   EncryptionResult InitPasswordFormFromStatement(
       const sql::Statement& s,
+      bool decrypt_and_fill_password_value,
       int* primary_key,
       autofill::PasswordForm* form) const WARN_UNUSED_RESULT;
 
@@ -272,37 +287,29 @@ class LoginDatabase : public syncer::SyncMetadataStore {
       bool blacklisted,
       std::vector<std::unique_ptr<autofill::PasswordForm>>* forms);
 
-  // Gets all logins synced from |begin| onwards (inclusive) and before |end|.
-  // You may use a null Time value to do an unbounded search in either
-  // direction. |key_to_form_map| must not be null and will be used to return
-  // the results. The key of the map is the DB primary key.
-  bool GetLoginsSyncedBetween(base::Time begin,
-                              base::Time end,
-                              PrimaryKeyToFormMap* key_to_form_map)
-      WARN_UNUSED_RESULT;
-
   // Returns the DB primary key for the specified |form|.  Returns -1 if the row
   // for this |form| is not found.
   int GetPrimaryKey(const autofill::PasswordForm& form) const;
 
   // Reads all the stored sync entities metadata in a MetadataBatch. Returns
   // nullptr in case of failure.
-  std::unique_ptr<syncer::MetadataBatch> GetAllSyncEntityMetadataForTesting();
+  std::unique_ptr<syncer::MetadataBatch> GetAllSyncEntityMetadata();
 
   // Reads the stored ModelTypeState. Returns nullptr in case of failure.
-  std::unique_ptr<sync_pb::ModelTypeState> GetModelTypeStateForTesting();
+  std::unique_ptr<sync_pb::ModelTypeState> GetModelTypeState();
 
-  // Overwrites |forms| with credentials retrieved from |statement|. If
-  // |matched_form| is not null, filters out all results but those PSL-matching
+  // Overwrites |key_to_form_map| with credentials retrieved from |statement|.
+  // If |matched_form| is not null, filters out all results but those
+  // PSL-matching
   // |*matched_form| or federated credentials for it. If feature for recovering
   // passwords is enabled, it removes all passwords that couldn't be decrypted
   // when encryption was available from the database. On success returns true.
   // |key_to_form_map| must not be null and will be used to return the results.
   // The key of the map is the DB primary key.
-  bool StatementToForms(sql::Statement* statement,
-                        const PasswordStore::FormDigest* matched_form,
-                        PrimaryKeyToFormMap* key_to_form_map)
-      WARN_UNUSED_RESULT;
+  FormRetrievalResult StatementToForms(
+      sql::Statement* statement,
+      const PasswordStore::FormDigest* matched_form,
+      PrimaryKeyToFormMap* key_to_form_map) WARN_UNUSED_RESULT;
 
   // Initializes all the *_statement_ data members with appropriate SQL
   // fragments based on |builder|.
@@ -312,10 +319,14 @@ class LoginDatabase : public syncer::SyncMetadataStore {
   // enabled, or false otherwise. On all other platforms it returns false.
   bool IsUsingCleanupMechanism() const;
 
-  base::FilePath db_path_;
+  const base::FilePath db_path_;
+  const IsAccountStore is_account_store_;
+
   mutable sql::Database db_;
   sql::MetaTable meta_table_;
   StatisticsTable stats_table_;
+  FieldInfoTable field_info_table_;
+  CompromisedCredentialsTable compromised_credentials_table_;
 
   // These cached strings are used to build SQL statements.
   std::string add_statement_;
@@ -328,9 +339,7 @@ class LoginDatabase : public syncer::SyncMetadataStore {
   std::string get_statement_psl_;
   std::string get_statement_federated_;
   std::string get_statement_psl_federated_;
-  std::string get_same_organization_name_logins_statement_;
   std::string created_statement_;
-  std::string synced_statement_;
   std::string blacklisted_statement_;
   std::string encrypted_statement_;
   std::string encrypted_password_statement_by_id_;
