@@ -5,10 +5,13 @@
 #ifndef COMPONENTS_PASSWORD_MANAGER_CORE_BROWSER_MOCK_PASSWORD_STORE_H_
 #define COMPONENTS_PASSWORD_MANAGER_CORE_BROWSER_MOCK_PASSWORD_STORE_H_
 
+#include <memory>
 #include <string>
 #include <vector>
 
 #include "components/autofill/core/common/password_form.h"
+#include "components/password_manager/core/browser/compromised_credentials_table.h"
+#include "components/password_manager/core/browser/field_info_table.h"
 #include "components/password_manager/core/browser/password_store.h"
 #include "components/password_manager/core/browser/statistics_table.h"
 #include "testing/gmock/include/gmock/gmock.h"
@@ -22,6 +25,8 @@ class MockPasswordStore : public PasswordStore {
   MOCK_METHOD1(RemoveLogin, void(const autofill::PasswordForm&));
   MOCK_METHOD2(GetLogins,
                void(const PasswordStore::FormDigest&, PasswordStoreConsumer*));
+  MOCK_METHOD2(GetLoginsByPassword,
+               void(const base::string16&, PasswordStoreConsumer*));
   MOCK_METHOD1(AddLogin, void(const autofill::PasswordForm&));
   MOCK_METHOD1(UpdateLogin, void(const autofill::PasswordForm&));
   MOCK_METHOD2(UpdateLoginWithPrimaryKey,
@@ -54,6 +59,9 @@ class MockPasswordStore : public PasswordStore {
       const PasswordStore::FormDigest& form) override {
     return std::vector<std::unique_ptr<autofill::PasswordForm>>();
   }
+  MOCK_METHOD1(FillMatchingLoginsByPassword,
+               std::vector<std::unique_ptr<autofill::PasswordForm>>(
+                   const base::string16&));
   MOCK_METHOD1(FillAutofillableLogins,
                bool(std::vector<std::unique_ptr<autofill::PasswordForm>>*));
   MOCK_METHOD1(FillBlacklistLogins,
@@ -65,17 +73,32 @@ class MockPasswordStore : public PasswordStore {
                std::vector<InteractionsStats>(const GURL& origin_domain));
   MOCK_METHOD1(AddSiteStatsImpl, void(const InteractionsStats&));
   MOCK_METHOD1(RemoveSiteStatsImpl, void(const GURL&));
+  MOCK_METHOD1(AddCompromisedCredentialsImpl,
+               void(const CompromisedCredentials&));
+  MOCK_METHOD2(RemoveCompromisedCredentialsImpl,
+               void(const GURL&, const base::string16&));
+  MOCK_METHOD0(GetAllCompromisedCredentialsImpl,
+               std::vector<CompromisedCredentials>());
+  MOCK_METHOD3(RemoveCompromisedCredentialsByUrlAndTimeImpl,
+               void(const base::RepeatingCallback<bool(const GURL&)>&,
+                    base::Time,
+                    base::Time));
+  MOCK_METHOD1(AddFieldInfoImpl, void(const FieldInfo&));
+  MOCK_METHOD0(GetAllFieldInfoImpl, std::vector<FieldInfo>());
+  MOCK_METHOD2(RemoveFieldInfoByTimeImpl, void(base::Time, base::Time));
+
   MOCK_CONST_METHOD0(IsAbleToSavePasswords, bool());
-// TODO(crbug.com/706392): Fix password reuse detection for Android.
+
 #if defined(SYNC_PASSWORD_REUSE_DETECTION_ENABLED)
   MOCK_METHOD3(CheckReuse,
                void(const base::string16&,
                     const std::string&,
                     PasswordReuseDetectorConsumer*));
-  MOCK_METHOD3(SaveGaiaPasswordHash,
+  MOCK_METHOD4(SaveGaiaPasswordHash,
                void(const std::string&,
                     const base::string16&,
-                    metrics_util::SyncPasswordHashChange));
+                    bool,
+                    metrics_util::GaiaPasswordHashChange));
   MOCK_METHOD2(SaveEnterprisePasswordHash,
                void(const std::string&, const base::string16&));
   MOCK_METHOD1(ClearGaiaPasswordHash, void(const std::string&));
@@ -88,6 +111,8 @@ class MockPasswordStore : public PasswordStore {
   MOCK_METHOD1(ReadAllLogins, FormRetrievalResult(PrimaryKeyToFormMap*));
   MOCK_METHOD1(RemoveLoginByPrimaryKeySync, PasswordStoreChangeList(int));
   MOCK_METHOD0(GetMetadataStore, PasswordStoreSync::MetadataStore*());
+  MOCK_CONST_METHOD0(IsAccountStore, bool());
+  MOCK_METHOD0(DeleteAndRecreateDatabaseFile, bool());
 
   PasswordStoreSync* GetSyncInterface() { return this; }
 
