@@ -8,13 +8,12 @@
 
 #include "base/bind.h"
 #include "base/logging.h"
-#include "base/strings/stringprintf.h"
 #include "base/trace_event/memory_dump_manager.h"
 #include "base/trace_event/process_memory_dump.h"
 #include "gpu/ipc/common/gpu_memory_buffer_impl.h"
 #include "gpu/ipc/common/gpu_memory_buffer_impl_shared_memory.h"
 #include "gpu/ipc/common/gpu_memory_buffer_support.h"
-#include "services/viz/privileged/interfaces/gl/gpu_service.mojom.h"
+#include "services/viz/privileged/mojom/gl/gpu_service.mojom.h"
 #include "ui/gfx/buffer_format_util.h"
 #include "ui/gfx/buffer_usage_util.h"
 
@@ -37,11 +36,6 @@ HostGpuMemoryBufferManager::PendingBufferInfo::PendingBufferInfo(
     PendingBufferInfo&&) = default;
 HostGpuMemoryBufferManager::PendingBufferInfo::~PendingBufferInfo() = default;
 
-HostGpuMemoryBufferManager::AllocatedBufferInfo::AllocatedBufferInfo() =
-    default;
-HostGpuMemoryBufferManager::AllocatedBufferInfo::~AllocatedBufferInfo() =
-    default;
-
 HostGpuMemoryBufferManager::HostGpuMemoryBufferManager(
     GpuServiceProvider gpu_service_provider,
     int client_id,
@@ -75,8 +69,8 @@ void HostGpuMemoryBufferManager::DestroyGpuMemoryBuffer(
   auto buffer_iter = buffers.find(id);
   if (buffer_iter == buffers.end())
     return;
-  DCHECK_NE(gfx::EMPTY_BUFFER, buffer_iter->second.type);
-  if (buffer_iter->second.type != gfx::SHARED_MEMORY_BUFFER) {
+  DCHECK_NE(gfx::EMPTY_BUFFER, buffer_iter->second.type());
+  if (buffer_iter->second.type() != gfx::SHARED_MEMORY_BUFFER) {
     auto* gpu_service = GetGpuService();
     DCHECK(gpu_service);
     gpu_service->DestroyGpuMemoryBuffer(id, client_id, sync_token);
@@ -91,8 +85,8 @@ void HostGpuMemoryBufferManager::DestroyAllGpuMemoryBufferForClient(
   if (client_iter != allocated_buffers_.end()) {
     auto& buffers = client_iter->second;
     for (const auto& pair : buffers) {
-      DCHECK_NE(gfx::EMPTY_BUFFER, pair.second.type);
-      if (pair.second.type != gfx::SHARED_MEMORY_BUFFER) {
+      DCHECK_NE(gfx::EMPTY_BUFFER, pair.second.type());
+      if (pair.second.type() != gfx::SHARED_MEMORY_BUFFER) {
         auto* gpu_service = GetGpuService();
         DCHECK(gpu_service);
         gpu_service->DestroyGpuMemoryBuffer(pair.first, client_id,
@@ -153,12 +147,8 @@ void HostGpuMemoryBufferManager::AllocateGpuMemoryBuffer(
                                                                  format)) {
     buffer_handle = gpu::GpuMemoryBufferImplSharedMemory::CreateGpuMemoryBuffer(
         id, size, format, usage);
-    AllocatedBufferInfo buffer_info;
     DCHECK_EQ(gfx::SHARED_MEMORY_BUFFER, buffer_handle.type);
-    buffer_info.type = gfx::SHARED_MEMORY_BUFFER;
-    buffer_info.buffer_size_in_bytes =
-        gfx::BufferSizeForBufferFormat(size, format);
-    buffer_info.shared_memory_guid = buffer_handle.region.GetGUID();
+    AllocatedBufferInfo buffer_info(buffer_handle, size, format);
     allocated_buffers_[client_id].insert(
         std::make_pair(buffer_handle.id, buffer_info));
   }
@@ -230,33 +220,11 @@ bool HostGpuMemoryBufferManager::OnMemoryDump(
   DCHECK(task_runner_->BelongsToCurrentThread());
   for (const auto& pair : allocated_buffers_) {
     int client_id = pair.first;
+    uint64_t client_tracing_process_id = ClientIdToTracingId(client_id);
     for (const auto& buffer_pair : pair.second) {
-      gfx::GpuMemoryBufferId buffer_id = buffer_pair.first;
-      const AllocatedBufferInfo& buffer_info = buffer_pair.second;
-      base::trace_event::MemoryAllocatorDump* dump =
-          pmd->CreateAllocatorDump(base::StringPrintf(
-              "gpumemorybuffer/client_%d/buffer_%d", client_id, buffer_id.id));
-      if (!dump)
+      auto& buffer_info = buffer_pair.second;
+      if (!buffer_info.OnMemoryDump(pmd, client_id, client_tracing_process_id))
         return false;
-      dump->AddScalar(base::trace_event::MemoryAllocatorDump::kNameSize,
-                      base::trace_event::MemoryAllocatorDump::kUnitsBytes,
-                      buffer_info.buffer_size_in_bytes);
-
-      // Create the cross-process ownership edge. If the client creates a
-      // corresponding dump for the same buffer, this will avoid to
-      // double-count them in tracing. If, instead, no other process will emit a
-      // dump with the same guid, the segment will be accounted to the browser.
-      uint64_t client_tracing_process_id = ClientIdToTracingId(client_id);
-
-      if (buffer_info.type == gfx::SHARED_MEMORY_BUFFER) {
-        pmd->CreateSharedMemoryOwnershipEdge(
-            dump->guid(), buffer_info.shared_memory_guid, 0 /* importance */);
-      } else {
-        auto shared_buffer_guid = gfx::GetGenericSharedGpuMemoryGUIDForTracing(
-            client_tracing_process_id, buffer_id);
-        pmd->CreateSharedGlobalAllocatorDump(shared_buffer_guid);
-        pmd->AddOwnershipEdge(dump->guid(), shared_buffer_guid);
-      }
     }
   }
   return true;
@@ -311,7 +279,9 @@ uint64_t HostGpuMemoryBufferManager::ClientIdToTracingId(int client_id) const {
   // TODO(sad|ssid): Find a better way once https://crbug.com/661257 is
   // resolved.  The hash value is incremented so that the tracing id is never
   // equal to MemoryDumpManager::kInvalidTracingProcessId.
-  return static_cast<uint64_t>(base::Hash(&client_id, sizeof(client_id))) + 1;
+  return static_cast<uint64_t>(base::PersistentHash(
+             base::as_bytes(base::make_span(&client_id, 1)))) +
+         1;
 }
 
 void HostGpuMemoryBufferManager::OnGpuMemoryBufferAllocated(
@@ -348,10 +318,8 @@ void HostGpuMemoryBufferManager::OnGpuMemoryBufferAllocated(
   if (!handle.is_null()) {
     DCHECK(handle.id == id);
 
-    AllocatedBufferInfo buffer_info;
-    buffer_info.type = handle.type;
-    buffer_info.buffer_size_in_bytes = gfx::BufferSizeForBufferFormat(
-        pending_buffer.size, pending_buffer.format);
+    AllocatedBufferInfo buffer_info(handle, pending_buffer.size,
+                                    pending_buffer.format);
     allocated_buffers_[client_id].insert(std::make_pair(id, buffer_info));
   }
   std::move(pending_buffer.callback).Run(std::move(handle));
