@@ -14,7 +14,11 @@
 #include "base/strings/utf_string_conversions.h"
 #include "base/task/post_task.h"
 #include "content/browser/cache_storage/cache_storage_context_impl.h"
+#include "content/browser/devtools/protocol/browser_handler.h"
+#include "content/browser/devtools/protocol/network.h"
+#include "content/browser/devtools/protocol/network_handler.h"
 #include "content/browser/indexed_db/indexed_db_context_impl.h"
+#include "content/browser/storage_partition_impl.h"
 #include "content/public/browser/browser_task_traits.h"
 #include "content/public/browser/browser_thread.h"
 #include "content/public/browser/render_process_host.h"
@@ -28,6 +32,10 @@
 namespace content {
 namespace protocol {
 
+using ClearCookiesCallback = Storage::Backend::ClearCookiesCallback;
+using GetCookiesCallback = Storage::Backend::GetCookiesCallback;
+using SetCookiesCallback = Storage::Backend::SetCookiesCallback;
+
 struct UsageListInitializer {
   const char* type;
   int64_t blink::mojom::UsageBreakdown::*usage_member;
@@ -48,6 +56,7 @@ UsageListInitializer initializers[] = {
 };
 
 namespace {
+
 void ReportUsageAndQuotaDataOnUIThread(
     std::unique_ptr<StorageHandler::GetUsageAndQuotaCallback> callback,
     blink::mojom::QuotaStatusCode code,
@@ -82,7 +91,7 @@ void GotUsageAndQuotaDataCallback(
     int64_t quota,
     blink::mojom::UsageBreakdownPtr usage_breakdown) {
   DCHECK_CURRENTLY_ON(BrowserThread::IO);
-  base::PostTaskWithTraits(
+  base::PostTask(
       FROM_HERE, {BrowserThread::UI},
       base::BindOnce(ReportUsageAndQuotaDataOnUIThread, std::move(callback),
                      code, usage, quota, std::move(usage_breakdown)));
@@ -97,6 +106,7 @@ void GetUsageAndQuotaOnIOThread(
       origin, blink::mojom::StorageType::kTemporary,
       base::BindOnce(&GotUsageAndQuotaDataCallback, std::move(callback)));
 }
+
 }  // namespace
 
 // Observer that listens on the IO thread for cache storage notifications and
@@ -193,10 +203,9 @@ class StorageHandler::IndexedDBObserver : IndexedDBContextImpl::Observer {
     auto found = origins_.find(origin);
     if (found == origins_.end())
       return;
-    base::PostTaskWithTraits(
-        FROM_HERE, {BrowserThread::UI},
-        base::BindOnce(&StorageHandler::NotifyIndexedDBListChanged, owner_,
-                       origin.Serialize()));
+    base::PostTask(FROM_HERE, {BrowserThread::UI},
+                   base::BindOnce(&StorageHandler::NotifyIndexedDBListChanged,
+                                  owner_, origin.Serialize()));
   }
 
   void OnIndexedDBContentChanged(
@@ -206,7 +215,7 @@ class StorageHandler::IndexedDBObserver : IndexedDBContextImpl::Observer {
     auto found = origins_.find(origin);
     if (found == origins_.end())
       return;
-    base::PostTaskWithTraits(
+    base::PostTask(
         FROM_HERE, {BrowserThread::UI},
         base::BindOnce(&StorageHandler::NotifyIndexedDBContentChanged, owner_,
                        origin.Serialize(), database_name, object_store_name));
@@ -263,6 +272,69 @@ Response StorageHandler::Disable() {
   return Response::OK();
 }
 
+void StorageHandler::GetCookies(Maybe<std::string> browser_context_id,
+                                std::unique_ptr<GetCookiesCallback> callback) {
+  StoragePartition* storage_partition = nullptr;
+  Response response = StorageHandler::FindStoragePartition(browser_context_id,
+                                                           &storage_partition);
+  if (!response.isSuccess()) {
+    callback->sendFailure(std::move(response));
+    return;
+  }
+
+  storage_partition->GetCookieManagerForBrowserProcess()->GetAllCookies(
+      base::BindOnce(
+          [](std::unique_ptr<GetCookiesCallback> callback,
+             const std::vector<net::CanonicalCookie>& cookies) {
+            callback->sendSuccess(NetworkHandler::BuildCookieArray(cookies));
+          },
+          std::move(callback)));
+}
+
+void StorageHandler::SetCookies(
+    std::unique_ptr<protocol::Array<Network::CookieParam>> cookies,
+    Maybe<std::string> browser_context_id,
+    std::unique_ptr<SetCookiesCallback> callback) {
+  StoragePartition* storage_partition = nullptr;
+  Response response = StorageHandler::FindStoragePartition(browser_context_id,
+                                                           &storage_partition);
+  if (!response.isSuccess()) {
+    callback->sendFailure(std::move(response));
+    return;
+  }
+
+  NetworkHandler::SetCookies(
+      storage_partition, std::move(cookies),
+      base::BindOnce(
+          [](std::unique_ptr<SetCookiesCallback> callback, bool success) {
+            if (success) {
+              callback->sendSuccess();
+            } else {
+              callback->sendFailure(
+                  Response::InvalidParams("Invalid cookie fields"));
+            }
+          },
+          std::move(callback)));
+}
+
+void StorageHandler::ClearCookies(
+    Maybe<std::string> browser_context_id,
+    std::unique_ptr<ClearCookiesCallback> callback) {
+  StoragePartition* storage_partition = nullptr;
+  Response response = StorageHandler::FindStoragePartition(browser_context_id,
+                                                           &storage_partition);
+  if (!response.isSuccess()) {
+    callback->sendFailure(std::move(response));
+    return;
+  }
+
+  storage_partition->GetCookieManagerForBrowserProcess()->DeleteCookies(
+      network::mojom::CookieDeletionFilter::New(),
+      base::BindOnce([](std::unique_ptr<ClearCookiesCallback> callback,
+                        uint32_t) { callback->sendSuccess(); },
+                     std::move(callback)));
+}
+
 void StorageHandler::ClearDataForOrigin(
     const std::string& origin,
     const std::string& storage_types,
@@ -320,7 +392,7 @@ void StorageHandler::GetUsageAndQuota(
   }
 
   storage::QuotaManager* manager = storage_partition_->GetQuotaManager();
-  base::PostTaskWithTraits(
+  base::PostTask(
       FROM_HERE, {BrowserThread::IO},
       base::BindOnce(&GetUsageAndQuotaOnIOThread, base::RetainedRef(manager),
                      url::Origin::Create(origin_url), std::move(callback)));
@@ -429,5 +501,20 @@ void StorageHandler::NotifyIndexedDBContentChanged(
                                      base::UTF16ToUTF8(object_store_name));
 }
 
+Response StorageHandler::FindStoragePartition(
+    const Maybe<std::string>& browser_context_id,
+    StoragePartition** storage_partition) {
+  BrowserContext* browser_context = nullptr;
+  Response response =
+      BrowserHandler::FindBrowserContext(browser_context_id, &browser_context);
+  if (!response.isSuccess())
+    return response;
+  *storage_partition =
+      BrowserContext::GetDefaultStoragePartition(browser_context);
+  if (!*storage_partition)
+    return Response::InternalError();
+  return Response::OK();
+}
+
 }  // namespace protocol
 }  // namespace content
