@@ -182,12 +182,14 @@ bool TouchEmulator::HandleMouseEvent(const WebMouseEvent& mouse_event,
   if (!enabled() || mode_ != Mode::kEmulatingTouchFromMouse)
     return false;
 
+  UpdateCursor();
+
   if (mouse_event.button == WebMouseEvent::Button::kRight &&
       mouse_event.GetType() == WebInputEvent::kMouseDown) {
     client_->ShowContextMenuAtPoint(
         gfx::Point(mouse_event.PositionInWidget().x,
                    mouse_event.PositionInWidget().y),
-        ui::MENU_SOURCE_MOUSE);
+        ui::MENU_SOURCE_MOUSE, target_view);
   }
 
   if (mouse_event.button != WebMouseEvent::Button::kLeft)
