@@ -38,6 +38,12 @@ gfx::PointF ComputeEventLocation(const blink::WebInputEvent& event) {
   return gfx::PointF();
 }
 
+bool IsMouseMiddleClick(const blink::WebInputEvent& event) {
+  return (event.GetType() == blink::WebInputEvent::Type::kMouseDown &&
+          static_cast<const blink::WebMouseEvent&>(event).button ==
+              blink::WebPointerProperties::Button::kMiddle);
+}
+
 constexpr const char kTracingCategory[] = "input,latency";
 
 }  // namespace
@@ -56,14 +62,10 @@ class TracingUmaTracker {
   TracingUmaTracker(TracingUmaTracker&& tracker) = default;
 
   void StopAndRecord() {
-    StopButNotRecord();
-    UmaHistogramTimes(metric_name_, base::TimeTicks::Now() - start_time_);
-  }
-
-  void StopButNotRecord() {
     TRACE_EVENT_ASYNC_END0(
         kTracingCategory, metric_name_,
         TRACE_ID_WITH_SCOPE("UmaTracker", TRACE_ID_LOCAL(id_)));
+    UmaHistogramTimes(metric_name_, base::TimeTicks::Now() - start_time_);
   }
 
  private:
@@ -81,13 +83,6 @@ class TracingUmaTracker {
 
 int TracingUmaTracker::next_id_ = 1;
 
-enum class HitTestResultsMatch {
-  kDoNotMatch = 0,
-  kMatch = 1,
-  kHitTestResultChanged = 2,
-  kMaxValue = kHitTestResultChanged,
-};
-
 RenderWidgetTargetResult::RenderWidgetTargetResult() = default;
 
 RenderWidgetTargetResult::RenderWidgetTargetResult(
@@ -97,13 +92,11 @@ RenderWidgetTargetResult::RenderWidgetTargetResult(
     RenderWidgetHostViewBase* in_view,
     bool in_should_query_view,
     base::Optional<gfx::PointF> in_location,
-    bool in_latched_target,
-    bool in_should_verify_result)
+    bool in_latched_target)
     : view(in_view),
       should_query_view(in_should_query_view),
       target_location(in_location),
-      latched_target(in_latched_target),
-      should_verify_result(in_should_verify_result) {}
+      latched_target(in_latched_target) {}
 
 RenderWidgetTargetResult::~RenderWidgetTargetResult() = default;
 
@@ -167,9 +160,8 @@ bool RenderWidgetTargeter::TargetingRequest::IsWebInputEventRequest() const {
   return !!event;
 }
 
-const blink::WebInputEvent& RenderWidgetTargeter::TargetingRequest::GetEvent()
-    const {
-  return *event.get();
+blink::WebInputEvent* RenderWidgetTargeter::TargetingRequest::GetEvent() {
+  return event.get();
 }
 
 RenderWidgetHostViewBase* RenderWidgetTargeter::TargetingRequest::GetRootView()
@@ -181,16 +173,6 @@ gfx::PointF RenderWidgetTargeter::TargetingRequest::GetLocation() const {
   return location;
 }
 
-viz::FrameSinkId
-RenderWidgetTargeter::TargetingRequest::GetExpectedFrameSinkId() const {
-  return expected_frame_sink_id;
-}
-
-void RenderWidgetTargeter::TargetingRequest::SetExpectedFrameSinkId(
-    const viz::FrameSinkId& id) {
-  expected_frame_sink_id = id;
-}
-
 const ui::LatencyInfo& RenderWidgetTargeter::TargetingRequest::GetLatency()
     const {
   return latency;
@@ -247,16 +229,26 @@ void RenderWidgetTargeter::ResolveTargetingRequest(TargetingRequest request) {
   }
 
   RenderWidgetTargetResult result;
+  auto* request_target = request.GetRootView();
+  auto request_target_location = request.GetLocation();
+
   if (request.IsWebInputEventRequest()) {
-    result = delegate_->FindTargetSynchronously(request.GetRootView(),
-                                                request.GetEvent());
+    result = is_autoscroll_in_progress_
+                 ? middle_click_result_
+                 : delegate_->FindTargetSynchronously(request_target,
+                                                      *request.GetEvent());
+    if (!is_autoscroll_in_progress_ &&
+        IsMouseMiddleClick(*request.GetEvent())) {
+      if (!result.should_query_view)
+        middle_click_result_ = result;
+    }
   } else {
-    result = delegate_->FindTargetSynchronouslyAtPoint(request.GetRootView(),
-                                                       request.GetLocation());
+    result = delegate_->FindTargetSynchronouslyAtPoint(request_target,
+                                                       request_target_location);
   }
   RenderWidgetHostViewBase* target = result.view;
   async_depth_ = 0;
-  if (result.should_query_view) {
+  if (!is_autoscroll_in_progress_ && result.should_query_view) {
     TRACE_EVENT_WITH_FLOW2(
         "viz,benchmark", "Event.Pipeline", TRACE_ID_GLOBAL(trace_id_),
         TRACE_EVENT_FLAG_FLOW_OUT, "step", "QueryClient(Start)",
@@ -270,39 +262,41 @@ void RenderWidgetTargeter::ResolveTargetingRequest(TargetingRequest request) {
     // root_view and the original event location for the initial query.
     // Do not compare hit test results if we are forced to do async hit testing
     // by HitTestQuery.
-    QueryClient(std::move(request));
+    QueryClient(request_target, request_target_location, nullptr, gfx::PointF(),
+                std::move(request));
   } else {
     FoundTarget(target, result.target_location, result.latched_target,
                 &request);
-    // Verify the event targeting results from surface layer viz hit testing if
-    // --use-viz-hit-test-surface-layer is enabled.
-    if (result.should_verify_result && !target->IsRenderWidgetHostViewGuest()) {
-      request.SetExpectedFrameSinkId(target->GetFrameSinkId());
-      QueryAndVerifyClient(std::move(request));
-    }
   }
 }
 
 void RenderWidgetTargeter::ViewWillBeDestroyed(RenderWidgetHostViewBase* view) {
   unresponsive_views_.erase(view);
+
+  if (is_autoscroll_in_progress_ && middle_click_result_.view == view) {
+    SetIsAutoScrollInProgress(false);
+  }
 }
 
 bool RenderWidgetTargeter::HasEventsPendingDispatch() const {
   return request_in_flight_ || !requests_.empty();
 }
 
-void RenderWidgetTargeter::QueryClientInternal(
+void RenderWidgetTargeter::SetIsAutoScrollInProgress(
+    bool autoscroll_in_progress) {
+  is_autoscroll_in_progress_ = autoscroll_in_progress;
+
+  // If middle click autoscroll ends, reset |middle_click_result_|.
+  if (!autoscroll_in_progress)
+    middle_click_result_ = RenderWidgetTargetResult();
+}
+
+void RenderWidgetTargeter::QueryClient(
     RenderWidgetHostViewBase* target,
     const gfx::PointF& target_location,
     RenderWidgetHostViewBase* last_request_target,
     const gfx::PointF& last_target_location,
     TargetingRequest request) {
-  // Async event targeting and verifying use two different queues, so they don't
-  // block each other.
-  bool is_verifying = request.GetExpectedFrameSinkId().is_valid();
-  DCHECK((!is_verifying && !request_in_flight_) ||
-         (is_verifying && !verify_request_in_flight_));
-
   auto* target_client = target->host()->input_target_client();
   // |target_client| may not be set yet for this |target| on Mac, need to
   // understand why this happens. https://crbug.com/859492.
@@ -312,21 +306,16 @@ void RenderWidgetTargeter::QueryClientInternal(
     return;
   }
 
-  if (is_verifying) {
-    verify_request_in_flight_ = std::move(request);
-  } else {
-    request_in_flight_ = std::move(request);
-    async_depth_++;
-  }
+  request_in_flight_ = std::move(request);
+  async_depth_++;
+
   TracingUmaTracker tracker("Event.AsyncTargeting.ResponseTime");
-  auto& hit_test_timeout =
-      is_verifying ? async_verify_hit_test_timeout_ : async_hit_test_timeout_;
-  hit_test_timeout.reset(new OneShotTimeoutMonitor(
+  async_hit_test_timeout_.reset(new OneShotTimeoutMonitor(
       base::BindOnce(
           &RenderWidgetTargeter::AsyncHitTestTimedOut,
           weak_ptr_factory_.GetWeakPtr(), target->GetWeakPtr(), target_location,
           last_request_target ? last_request_target->GetWeakPtr() : nullptr,
-          last_target_location, is_verifying),
+          last_target_location),
       async_hit_test_timeout_delay_));
 
   TRACE_EVENT_WITH_FLOW2(
@@ -336,58 +325,30 @@ void RenderWidgetTargeter::QueryClientInternal(
 
   target_client->FrameSinkIdAt(
       target_location, trace_id_,
-      base::BindOnce(
-          &RenderWidgetTargeter::FoundFrameSinkId,
-          weak_ptr_factory_.GetWeakPtr(), target->GetWeakPtr(),
-          is_verifying ? ++last_verify_request_id_ : ++last_request_id_,
-          target_location, std::move(tracker), is_verifying));
+      base::BindOnce(&RenderWidgetTargeter::FoundFrameSinkId,
+                     weak_ptr_factory_.GetWeakPtr(), target->GetWeakPtr(),
+                     ++last_request_id_, target_location, std::move(tracker)));
 }
 
-void RenderWidgetTargeter::QueryClient(TargetingRequest request) {
-  auto* target = request.GetRootView();
-  auto target_location = request.GetLocation();
-  QueryClientInternal(target, target_location, nullptr, gfx::PointF(),
-                      std::move(request));
-}
-
-void RenderWidgetTargeter::QueryAndVerifyClient(TargetingRequest request) {
-  if (verify_request_in_flight_) {
-    verify_requests_.push(std::move(request));
-    return;
-  }
-  auto* target = request.GetRootView();
-  auto target_location = request.GetLocation();
-  QueryClientInternal(target, target_location, nullptr, gfx::PointF(),
-                      std::move(request));
-}
-
-void RenderWidgetTargeter::FlushEventQueue(bool is_verifying) {
+void RenderWidgetTargeter::FlushEventQueue() {
   bool events_being_flushed = false;
-  base::Optional<TargetingRequest>& request_in_flight =
-      is_verifying ? verify_request_in_flight_ : request_in_flight_;
-  auto* requests = is_verifying ? &verify_requests_ : &requests_;
-  while (!request_in_flight && !requests->empty()) {
-    auto request = std::move(requests->front());
-    requests->pop();
+  while (!request_in_flight_ && !requests_.empty()) {
+    auto request = std::move(requests_.front());
+    requests_.pop();
     // The root-view has gone away. Ignore this event, and try to process the
     // next event.
-    if (!request.GetRootView()) {
+    if (!request.GetRootView())
       continue;
-    }
+
     request.StopQueueingTimeTracker();
     // Only notify the delegate once that the current event queue is being
     // flushed. Once all the events are flushed, notify the delegate again.
-    if (!is_verifying && !events_being_flushed) {
+    if (!events_being_flushed) {
       delegate_->SetEventsBeingFlushed(true);
       events_being_flushed = true;
     }
-    if (is_verifying) {
-      QueryAndVerifyClient(std::move(request));
-    } else {
       ResolveTargetingRequest(std::move(request));
-    }
   }
-  if (!is_verifying)
     delegate_->SetEventsBeingFlushed(false);
 }
 
@@ -396,20 +357,12 @@ void RenderWidgetTargeter::FoundFrameSinkId(
     uint32_t request_id,
     const gfx::PointF& target_location,
     TracingUmaTracker tracker,
-    const bool is_verification_request,
     const viz::FrameSinkId& frame_sink_id,
     const gfx::PointF& transformed_location) {
-  if (is_verification_request) {
-    tracker.StopButNotRecord();
-  } else {
-    tracker.StopAndRecord();
-  }
+  tracker.StopAndRecord();
 
-  uint32_t last_id =
-      is_verification_request ? last_verify_request_id_ : last_request_id_;
-  bool in_flight = is_verification_request
-                       ? verify_request_in_flight_.has_value()
-                       : request_in_flight_.has_value();
+  uint32_t last_id = last_request_id_;
+  bool in_flight = request_in_flight_.has_value();
   if (request_id != last_id || !in_flight) {
     // This is a response to a request that already timed out, so the event
     // should have already been dispatched. Mark the renderer as responsive
@@ -418,23 +371,16 @@ void RenderWidgetTargeter::FoundFrameSinkId(
     return;
   }
 
-  TargetingRequest request = is_verification_request
-                                 ? std::move(verify_request_in_flight_.value())
-                                 : std::move(request_in_flight_.value());
+  TargetingRequest request = std::move(request_in_flight_.value());
 
-  if (request.GetExpectedFrameSinkId().is_valid()) {
-    verify_request_in_flight_.reset();
-    async_verify_hit_test_timeout_.reset(nullptr);
-  } else {
-    request_in_flight_.reset();
-    async_hit_test_timeout_.reset(nullptr);
+  request_in_flight_.reset();
+  async_hit_test_timeout_.reset(nullptr);
 
-    if (is_viz_hit_testing_debug_enabled_ && request.IsWebInputEventRequest() &&
-        request.GetEvent().GetType() ==
-            blink::WebInputEvent::Type::kMouseDown) {
-      hit_test_async_queried_debug_queue_.push_back(target->GetFrameSinkId());
-    }
+  if (is_viz_hit_testing_debug_enabled_ && request.IsWebInputEventRequest() &&
+      request.GetEvent()->GetType() == blink::WebInputEvent::Type::kMouseDown) {
+    hit_test_async_queried_debug_queue_.push_back(target->GetFrameSinkId());
   }
+
   auto* view = delegate_->FindViewFromFrameSinkId(frame_sink_id);
   if (!view)
     view = target.get();
@@ -452,10 +398,15 @@ void RenderWidgetTargeter::FoundFrameSinkId(
                              TRACE_EVENT_FLAG_FLOW_IN, "step", "FoundTarget");
     }
 
+    if (request.IsWebInputEventRequest() &&
+        IsMouseMiddleClick(*request.GetEvent())) {
+      middle_click_result_ = {view, false, transformed_location, false};
+    }
+
     FoundTarget(view, transformed_location, false, &request);
   } else {
-    QueryClientInternal(view, transformed_location, target.get(),
-                        target_location, std::move(request));
+    QueryClient(view, transformed_location, target.get(), target_location,
+                std::move(request));
   }
 }
 
@@ -467,7 +418,7 @@ void RenderWidgetTargeter::FoundTarget(
   DCHECK(request);
 
   if (SiteIsolationPolicy::UseDedicatedProcessesForAllSites() &&
-      !latched_target && !request->GetExpectedFrameSinkId().is_valid()) {
+      !latched_target) {
     UMA_HISTOGRAM_COUNTS_100("Event.AsyncTargeting.AsyncClientDepth",
                              async_depth_);
   }
@@ -485,45 +436,6 @@ void RenderWidgetTargeter::FoundTarget(
     hit_test_async_queried_debug_queue_.clear();
   }
 
-  if (features::IsVizHitTestingSurfaceLayerEnabled() &&
-      request->GetExpectedFrameSinkId().is_valid()) {
-    static const char* kResultsMatchHistogramName =
-        "Event.VizHitTestSurfaceLayer.ResultsMatch";
-    bool results_match =
-        target->GetFrameSinkId() == request->GetExpectedFrameSinkId();
-    HitTestResultsMatch match_result =
-        HitTestResultsMatch::kHitTestResultChanged;
-    if (results_match) {
-      match_result = HitTestResultsMatch::kMatch;
-    } else {
-      // If the results do not match, it is possible that the hit test data
-      // changed during verification. We do synchronous hit test again to make
-      // sure the result is reliable.
-      RenderWidgetTargetResult result;
-      if (request->IsWebInputEventRequest()) {
-        result = delegate_->FindTargetSynchronously(request->GetRootView(),
-                                                    request->GetEvent());
-      } else {
-        result = delegate_->FindTargetSynchronouslyAtPoint(
-            request->GetRootView(), request->GetLocation());
-      }
-
-      if (!result.should_query_view && result.view &&
-          request->GetExpectedFrameSinkId() == result.view->GetFrameSinkId()) {
-        // If the result did not change, it is likely that viz hit test finds
-        // the wrong target.
-        match_result = HitTestResultsMatch::kDoNotMatch;
-      } else {
-        // Hit test data changed, so the result is no longer reliable.
-        match_result = HitTestResultsMatch::kHitTestResultChanged;
-      }
-    }
-    UMA_HISTOGRAM_ENUMERATION(kResultsMatchHistogramName, match_result,
-                              HitTestResultsMatch::kMaxValue);
-    FlushEventQueue(true);
-    return;
-  }
-
   if (request->IsWebInputEventRequest()) {
     delegate_->DispatchEventToTarget(request->GetRootView(), target,
                                      request->GetEvent(), request->GetLatency(),
@@ -532,29 +444,18 @@ void RenderWidgetTargeter::FoundTarget(
     request->RunCallback(target, target_location);
   }
 
-  FlushEventQueue(false);
+  FlushEventQueue();
 }
 
 void RenderWidgetTargeter::AsyncHitTestTimedOut(
     base::WeakPtr<RenderWidgetHostViewBase> current_request_target,
     const gfx::PointF& current_target_location,
     base::WeakPtr<RenderWidgetHostViewBase> last_request_target,
-    const gfx::PointF& last_target_location,
-    const bool is_verification_request) {
-  DCHECK(request_in_flight_ || verify_request_in_flight_);
+    const gfx::PointF& last_target_location) {
+  DCHECK(request_in_flight_);
 
-  TargetingRequest request = is_verification_request
-                                 ? std::move(verify_request_in_flight_.value())
-                                 : std::move(request_in_flight_.value());
-
-  // If we time out during a verification, we early out to avoid dispatching
-  // event to root frame.
-  if (request.GetExpectedFrameSinkId().is_valid()) {
-    verify_request_in_flight_.reset();
-    return;
-  } else {
-    request_in_flight_.reset();
-  }
+  TargetingRequest request = std::move(request_in_flight_.value());
+  request_in_flight_.reset();
 
   if (!request.GetRootView())
     return;
