@@ -9,28 +9,33 @@
 #include <memory>
 #include <utility>
 
-#include "base/memory/ptr_util.h"
+#include "base/bind.h"
+#include "base/command_line.h"
 #include "base/strings/string_number_conversions.h"
+#include "base/task/post_task.h"
 #include "build/build_config.h"
 #include "content/browser/renderer_host/render_process_host_impl.h"
 #include "content/browser/web_contents/web_contents_view.h"
+#include "content/browser/webrtc/webrtc_internals_connections_observer.h"
 #include "content/browser/webrtc/webrtc_internals_ui_observer.h"
+#include "content/public/browser/browser_task_traits.h"
 #include "content/public/browser/browser_thread.h"
 #include "content/public/browser/content_browser_client.h"
+#include "content/public/browser/system_connector.h"
 #include "content/public/browser/web_contents.h"
-#include "content/public/common/service_manager_connection.h"
+#include "content/public/browser/webrtc_event_logger.h"
+#include "content/public/common/content_client.h"
+#include "content/public/common/content_switches.h"
 #include "ipc/ipc_platform_file.h"
+#include "media/audio/audio_debug_recording_session.h"
 #include "media/audio/audio_manager.h"
-#include "media/media_features.h"
-#include "services/device/public/interfaces/constants.mojom.h"
-#include "services/device/public/interfaces/wake_lock_provider.mojom.h"
+#include "media/media_buildflags.h"
+#include "mojo/public/cpp/bindings/pending_receiver.h"
+#include "services/audio/public/cpp/debug_recording_session_factory.h"
+#include "services/device/public/mojom/constants.mojom.h"
+#include "services/device/public/mojom/wake_lock_provider.mojom.h"
 #include "services/service_manager/public/cpp/connector.h"
-
-#if defined(OS_WIN)
-#define IntToStringType base::IntToString16
-#else
-#define IntToStringType base::IntToString
-#endif
+#include "ui/shell_dialogs/select_file_policy.h"
 
 using base::ProcessId;
 using std::string;
@@ -39,26 +44,32 @@ namespace content {
 
 namespace {
 
-base::LazyInstance<WebRTCInternals>::Leaky g_webrtc_internals =
-    LAZY_INSTANCE_INITIALIZER;
+const base::FilePath::CharType kEventLogFilename[] =
+    FILE_PATH_LITERAL("event_log");
+
+// This is intended to limit DoS attacks against the browser process consisting
+// of many getUserMedia() calls. See https://crbug.com/804440.
+const size_t kMaxGetUserMediaEntries = 1000;
 
 // Makes sure that |dict| has a ListValue under path "log".
 base::ListValue* EnsureLogList(base::DictionaryValue* dict) {
-  base::ListValue* log = NULL;
+  base::ListValue* log = nullptr;
   if (!dict->GetList("log", &log))
-    log = dict->SetList("log", base::MakeUnique<base::ListValue>());
+    log = dict->SetList("log", std::make_unique<base::ListValue>());
   return log;
 }
 
 // Removes the log entry associated with a given record.
 void FreeLogList(base::Value* value) {
-  DCHECK(value->IsType(base::Value::Type::DICTIONARY));
+  DCHECK(value->is_dict());
   auto* dict = static_cast<base::DictionaryValue*>(value);
   dict->Remove("log", nullptr);
 }
 
 }  // namespace
 
+WebRTCInternals* WebRTCInternals::g_webrtc_internals = nullptr;
+
 WebRTCInternals::PendingUpdate::PendingUpdate(
     const char* command,
     std::unique_ptr<base::Value> value)
@@ -86,15 +97,20 @@ WebRTCInternals::WebRTCInternals() : WebRTCInternals(500, true) {}
 
 WebRTCInternals::WebRTCInternals(int aggregate_updates_ms,
                                  bool should_block_power_saving)
-    : audio_debug_recordings_(false),
+    : selection_type_(SelectionType::kAudioDebugRecordings),
+      command_line_derived_logging_path_(
+          base::CommandLine::ForCurrentProcess()->GetSwitchValuePath(
+              switches::kWebRtcLocalEventLogging)),
       event_log_recordings_(false),
-      selecting_event_log_(false),
-      num_open_connections_(0),
+      num_connected_connections_(0),
       should_block_power_saving_(should_block_power_saving),
-      aggregate_updates_ms_(aggregate_updates_ms),
-      weak_factory_(this) {
+      aggregate_updates_ms_(aggregate_updates_ms) {
+  DCHECK_CURRENTLY_ON(BrowserThread::UI);
+  DCHECK(!g_webrtc_internals);
+
 // TODO(grunell): Shouldn't all the webrtc_internals* files be excluded from the
 // build if WebRTC is disabled?
+// https://crbug.com/817446
 #if BUILDFLAG(ENABLE_WEBRTC)
   audio_debug_recordings_file_path_ =
       GetContentClient()->browser()->GetDefaultDownloadDirectory();
@@ -111,16 +127,40 @@ WebRTCInternals::WebRTCInternals(int aggregate_updates_ms,
         audio_debug_recordings_file_path_.Append(
             FILE_PATH_LITERAL("audio_debug"));
     event_log_recordings_file_path_ =
-        event_log_recordings_file_path_.Append(FILE_PATH_LITERAL("event_log"));
+        event_log_recordings_file_path_.Append(kEventLogFilename);
+  }
+
+  // Allow command-line based setting of (local) WebRTC event logging.
+  if (!command_line_derived_logging_path_.empty()) {
+    const base::FilePath local_logs_path =
+        command_line_derived_logging_path_.Append(kEventLogFilename);
+    WebRtcEventLogger* const logger = WebRtcEventLogger::Get();
+    if (logger) {
+      logger->EnableLocalLogging(local_logs_path,
+                                 base::OnceCallback<void(bool)>());
+    }
+    // For clarity's sake, though these aren't supposed to be regarded now:
+    event_log_recordings_ = true;
+    event_log_recordings_file_path_.clear();
   }
 #endif  // BUILDFLAG(ENABLE_WEBRTC)
+
+  g_webrtc_internals = this;
 }
 
 WebRTCInternals::~WebRTCInternals() {
+  DCHECK(g_webrtc_internals);
+  g_webrtc_internals = nullptr;
+}
+
+WebRTCInternals* WebRTCInternals::CreateSingletonInstance() {
+  DCHECK(!g_webrtc_internals);
+  g_webrtc_internals = new WebRTCInternals;
+  return g_webrtc_internals;
 }
 
 WebRTCInternals* WebRTCInternals::GetInstance() {
-  return g_webrtc_internals.Pointer();
+  return g_webrtc_internals;
 }
 
 void WebRTCInternals::OnAddPeerConnection(int render_process_id,
@@ -142,13 +182,12 @@ void WebRTCInternals::OnAddPeerConnection(int render_process_id,
   dict->SetString("constraints", constraints);
   dict->SetString("url", url);
   dict->SetBoolean("isOpen", true);
+  dict->SetBoolean("connected", false);
 
   if (observers_.might_have_observers())
     SendUpdate("addPeerConnection", dict->CreateDeepCopy());
 
   peer_connection_data_.Append(std::move(dict));
-  ++num_open_connections_;
-  UpdateWakeLock();
 
   if (render_process_id_set_.insert(render_process_id).second) {
     RenderProcessHost* host = RenderProcessHost::FromID(render_process_id);
@@ -164,7 +203,7 @@ void WebRTCInternals::OnRemovePeerConnection(ProcessId pid, int lid) {
   base::DictionaryValue* dict = FindRecord(pid, lid, &index);
   if (dict) {
     MaybeClosePeerConnection(dict);
-    peer_connection_data_.Remove(index, NULL);
+    peer_connection_data_.Remove(index, nullptr);
   }
 
   if (observers_.might_have_observers()) {
@@ -183,22 +222,30 @@ void WebRTCInternals::OnUpdatePeerConnection(
   if (!record)
     return;
 
-  if (type == "stop")
+  if (type == "iceConnectionStateChange") {
+    if (value == "connected" || value == "checking" || value == "completed") {
+      MaybeMarkPeerConnectionAsConnected(record);
+    } else if (value == "failed" || value == "disconnected" ||
+               value == "closed" || value == "new") {
+      MaybeMarkPeerConnectionAsNotConnected(record);
+    }
+  } else if (type == "stop") {
     MaybeClosePeerConnection(record);
+  }
 
   // Don't update entries if there aren't any observers.
   if (!observers_.might_have_observers())
     return;
 
-  auto log_entry = base::MakeUnique<base::DictionaryValue>();
+  auto log_entry = std::make_unique<base::DictionaryValue>();
 
   double epoch_time = base::Time::Now().ToJsTime();
-  string time = base::DoubleToString(epoch_time);
+  string time = base::NumberToString(epoch_time);
   log_entry->SetString("time", time);
   log_entry->SetString("type", type);
   log_entry->SetString("value", value);
 
-  auto update = base::MakeUnique<base::DictionaryValue>();
+  auto update = std::make_unique<base::DictionaryValue>();
   update->SetInteger("pid", static_cast<int>(pid));
   update->SetInteger("lid", lid);
   update->MergeDictionary(log_entry.get());
@@ -209,18 +256,34 @@ void WebRTCInternals::OnUpdatePeerConnection(
   EnsureLogList(record)->Append(std::move(log_entry));
 }
 
-void WebRTCInternals::OnAddStats(base::ProcessId pid, int lid,
-                                 const base::ListValue& value) {
+void WebRTCInternals::OnAddStandardStats(base::ProcessId pid,
+                                         int lid,
+                                         base::Value value) {
   if (!observers_.might_have_observers())
     return;
 
-  auto dict = base::MakeUnique<base::DictionaryValue>();
+  auto dict = std::make_unique<base::DictionaryValue>();
   dict->SetInteger("pid", static_cast<int>(pid));
   dict->SetInteger("lid", lid);
 
-  dict->Set("reports", base::MakeUnique<base::Value>(value));
+  dict->SetKey("reports", std::move(value));
 
-  SendUpdate("addStats", std::move(dict));
+  SendUpdate("addStandardStats", std::move(dict));
+}
+
+void WebRTCInternals::OnAddLegacyStats(base::ProcessId pid,
+                                       int lid,
+                                       base::Value value) {
+  if (!observers_.might_have_observers())
+    return;
+
+  auto dict = std::make_unique<base::DictionaryValue>();
+  dict->SetInteger("pid", static_cast<int>(pid));
+  dict->SetInteger("lid", lid);
+
+  dict->SetKey("reports", std::move(value));
+
+  SendUpdate("addLegacyStats", std::move(dict));
 }
 
 void WebRTCInternals::OnGetUserMedia(int rid,
@@ -232,10 +295,17 @@ void WebRTCInternals::OnGetUserMedia(int rid,
                                      const std::string& video_constraints) {
   DCHECK_CURRENTLY_ON(BrowserThread::UI);
 
-  auto dict = base::MakeUnique<base::DictionaryValue>();
+  if (get_user_media_requests_.GetList().size() >= kMaxGetUserMediaEntries) {
+    LOG(WARNING) << "Maximum number of tracked getUserMedia() requests reached "
+                    "in webrtc-internals.";
+    return;
+  }
+
+  auto dict = std::make_unique<base::DictionaryValue>();
   dict->SetInteger("rid", rid);
   dict->SetInteger("pid", static_cast<int>(pid));
   dict->SetString("origin", origin);
+  dict->SetDouble("timestamp", base::Time::Now().ToJsTime());
   if (audio)
     dict->SetString("audio", audio_constraints);
   if (video)
@@ -267,13 +337,25 @@ void WebRTCInternals::RemoveObserver(WebRTCInternalsUIObserver* observer) {
   // Disables event log and audio debug recordings if enabled and the last
   // webrtc-internals page is going away.
   DisableAudioDebugRecordings();
-  DisableEventLogRecordings();
+  DisableLocalEventLogRecordings();
 
   // TODO(tommi): Consider removing all the peer_connection_data_.
   for (auto& dictionary : peer_connection_data_)
     FreeLogList(&dictionary);
 }
 
+void WebRTCInternals::AddConnectionsObserver(
+    WebRtcInternalsConnectionsObserver* observer) {
+  DCHECK_CURRENTLY_ON(BrowserThread::UI);
+  connections_observers_.AddObserver(observer);
+}
+
+void WebRTCInternals::RemoveConnectionsObserver(
+    WebRtcInternalsConnectionsObserver* observer) {
+  DCHECK_CURRENTLY_ON(BrowserThread::UI);
+  connections_observers_.RemoveObserver(observer);
+}
+
 void WebRTCInternals::UpdateObserver(WebRTCInternalsUIObserver* observer) {
   DCHECK_CURRENTLY_ON(BrowserThread::UI);
   if (peer_connection_data_.GetSize() > 0)
@@ -291,18 +373,14 @@ void WebRTCInternals::EnableAudioDebugRecordings(
 #if defined(OS_ANDROID)
   EnableAudioDebugRecordingsOnAllRenderProcessHosts();
 #else
-  selecting_event_log_ = false;
+  selection_type_ = SelectionType::kAudioDebugRecordings;
   DCHECK(!select_file_dialog_);
-  select_file_dialog_ = ui::SelectFileDialog::Create(this, NULL);
+  select_file_dialog_ = ui::SelectFileDialog::Create(this, nullptr);
   select_file_dialog_->SelectFile(
-      ui::SelectFileDialog::SELECT_SAVEAS_FILE,
-      base::string16(),
-      audio_debug_recordings_file_path_,
-      NULL,
-      0,
-      FILE_PATH_LITERAL(""),
-      web_contents->GetTopLevelNativeWindow(),
-      NULL);
+      ui::SelectFileDialog::SELECT_SAVEAS_FILE, base::string16(),
+      audio_debug_recordings_file_path_, nullptr, 0,
+      base::FilePath::StringType(), web_contents->GetTopLevelNativeWindow(),
+      nullptr);
 #endif
 #endif
 }
@@ -310,10 +388,9 @@ void WebRTCInternals::EnableAudioDebugRecordings(
 void WebRTCInternals::DisableAudioDebugRecordings() {
   DCHECK_CURRENTLY_ON(BrowserThread::UI);
 #if BUILDFLAG(ENABLE_WEBRTC)
-  if (!audio_debug_recordings_)
+  if (!audio_debug_recording_session_)
     return;
-
-  audio_debug_recordings_ = false;
+  audio_debug_recording_session_.reset();
 
   // Tear down the dialog since the user has unchecked the audio debug
   // recordings box.
@@ -324,21 +401,12 @@ void WebRTCInternals::DisableAudioDebugRecordings() {
        !i.IsAtEnd(); i.Advance()) {
     i.GetCurrentValue()->DisableAudioDebugRecordings();
   }
-
-  // It's safe to get the AudioManager pointer here. That pointer is invalidated
-  // on the UI thread, which we're on.
-  // AudioManager is deleted on the audio thread, and the AudioManager outlives
-  // this object, so it's safe to post unretained to the audio thread.
-  media::AudioManager* audio_manager = media::AudioManager::Get();
-  audio_manager->GetTaskRunner()->PostTask(
-      FROM_HERE, base::Bind(&media::AudioManager::DisableOutputDebugRecording,
-                            base::Unretained(audio_manager)));
 #endif
 }
 
 bool WebRTCInternals::IsAudioDebugRecordingsEnabled() const {
   DCHECK_CURRENTLY_ON(BrowserThread::UI);
-  return audio_debug_recordings_;
+  return !!audio_debug_recording_session_;
 }
 
 const base::FilePath& WebRTCInternals::GetAudioDebugRecordingsFilePath() const {
@@ -346,16 +414,21 @@ const base::FilePath& WebRTCInternals::GetAudioDebugRecordingsFilePath() const {
   return audio_debug_recordings_file_path_;
 }
 
-void WebRTCInternals::EnableEventLogRecordings(
+void WebRTCInternals::EnableLocalEventLogRecordings(
     content::WebContents* web_contents) {
   DCHECK_CURRENTLY_ON(BrowserThread::UI);
+  DCHECK(CanToggleEventLogRecordings());
 #if BUILDFLAG(ENABLE_WEBRTC)
 #if defined(OS_ANDROID)
-  EnableEventLogRecordingsOnAllRenderProcessHosts();
+  WebRtcEventLogger* const logger = WebRtcEventLogger::Get();
+  if (logger) {
+    logger->EnableLocalLogging(event_log_recordings_file_path_,
+                               base::OnceCallback<void(bool)>());
+  }
 #else
   DCHECK(web_contents);
   DCHECK(!select_file_dialog_);
-  selecting_event_log_ = true;
+  selection_type_ = SelectionType::kRtcEventLogs;
   select_file_dialog_ = ui::SelectFileDialog::Create(this, nullptr);
   select_file_dialog_->SelectFile(
       ui::SelectFileDialog::SELECT_SAVEAS_FILE, base::string16(),
@@ -365,15 +438,16 @@ void WebRTCInternals::EnableEventLogRecordings(
 #endif
 }
 
-void WebRTCInternals::DisableEventLogRecordings() {
+void WebRTCInternals::DisableLocalEventLogRecordings() {
 #if BUILDFLAG(ENABLE_WEBRTC)
   event_log_recordings_ = false;
   // Tear down the dialog since the user has unchecked the event log checkbox.
   select_file_dialog_ = nullptr;
-  for (RenderProcessHost::iterator i(
-           content::RenderProcessHost::AllHostsIterator());
-       !i.IsAtEnd(); i.Advance())
-    i.GetCurrentValue()->StopWebRTCEventLog();
+  DCHECK(CanToggleEventLogRecordings());
+  WebRtcEventLogger* const logger = WebRtcEventLogger::Get();
+  if (logger) {
+    logger->DisableLocalLogging(base::OnceCallback<void(bool)>());
+  }
 #endif
 }
 
@@ -382,9 +456,8 @@ bool WebRTCInternals::IsEventLogRecordingsEnabled() const {
   return event_log_recordings_;
 }
 
-const base::FilePath& WebRTCInternals::GetEventLogFilePath() const {
-  DCHECK_CURRENTLY_ON(BrowserThread::UI);
-  return event_log_recordings_file_path_;
+bool WebRTCInternals::CanToggleEventLogRecordings() const {
+  return command_line_derived_logging_path_.empty();
 }
 
 void WebRTCInternals::SendUpdate(const char* command,
@@ -396,16 +469,17 @@ void WebRTCInternals::SendUpdate(const char* command,
   pending_updates_.push(PendingUpdate(command, std::move(value)));
 
   if (queue_was_empty) {
-    BrowserThread::PostDelayedTask(BrowserThread::UI, FROM_HERE,
-        base::Bind(&WebRTCInternals::ProcessPendingUpdates,
-                   weak_factory_.GetWeakPtr()),
-                   base::TimeDelta::FromMilliseconds(aggregate_updates_ms_));
+    base::PostDelayedTask(
+        FROM_HERE, {BrowserThread::UI},
+        base::BindOnce(&WebRTCInternals::ProcessPendingUpdates,
+                       weak_factory_.GetWeakPtr()),
+        base::TimeDelta::FromMilliseconds(aggregate_updates_ms_));
   }
 }
 
-void WebRTCInternals::RenderProcessExited(RenderProcessHost* host,
-                                          base::TerminationStatus status,
-                                          int exit_code) {
+void WebRTCInternals::RenderProcessExited(
+    RenderProcessHost* host,
+    const ChildProcessTerminationInfo& info) {
   DCHECK_CURRENTLY_ON(BrowserThread::UI);
   OnRendererExit(host->GetID());
   render_process_id_set_.erase(host->GetID());
@@ -417,12 +491,22 @@ void WebRTCInternals::FileSelected(const base::FilePath& path,
                                    void* /*unused_params */) {
 #if BUILDFLAG(ENABLE_WEBRTC)
   DCHECK_CURRENTLY_ON(BrowserThread::UI);
-  if (selecting_event_log_) {
-    event_log_recordings_file_path_ = path;
-    EnableEventLogRecordingsOnAllRenderProcessHosts();
-  } else {
-    audio_debug_recordings_file_path_ = path;
-    EnableAudioDebugRecordingsOnAllRenderProcessHosts();
+  switch (selection_type_) {
+    case SelectionType::kRtcEventLogs: {
+      event_log_recordings_file_path_ = path;
+      event_log_recordings_ = true;
+      WebRtcEventLogger* const logger = WebRtcEventLogger::Get();
+      if (logger) {
+        logger->EnableLocalLogging(path, base::OnceCallback<void(bool)>());
+      }
+      break;
+    }
+    case SelectionType::kAudioDebugRecordings: {
+      audio_debug_recordings_file_path_ = path;
+      EnableAudioDebugRecordingsOnAllRenderProcessHosts();
+      break;
+    }
+    default: { NOTREACHED(); }
   }
 #endif
 }
@@ -430,11 +514,17 @@ void WebRTCInternals::FileSelected(const base::FilePath& path,
 void WebRTCInternals::FileSelectionCanceled(void* params) {
 #if BUILDFLAG(ENABLE_WEBRTC)
   DCHECK_CURRENTLY_ON(BrowserThread::UI);
-  if (selecting_event_log_) {
-    SendUpdate("eventLogRecordingsFileSelectionCancelled", nullptr);
-  } else {
-    SendUpdate("audioDebugRecordingsFileSelectionCancelled", nullptr);
+  switch (selection_type_) {
+    case SelectionType::kRtcEventLogs:
+      SendUpdate("eventLogRecordingsFileSelectionCancelled", nullptr);
+      break;
+    case SelectionType::kAudioDebugRecordings:
+      SendUpdate("audioDebugRecordingsFileSelectionCancelled", nullptr);
+      break;
+    default:
+      NOTREACHED();
   }
+  select_file_dialog_ = nullptr;
 #endif
 }
 
@@ -444,7 +534,7 @@ void WebRTCInternals::OnRendererExit(int render_process_id) {
   // Iterates from the end of the list to remove the PeerConnections created
   // by the exitting renderer.
   for (int i = peer_connection_data_.GetSize() - 1; i >= 0; --i) {
-    base::DictionaryValue* record = NULL;
+    base::DictionaryValue* record = nullptr;
     peer_connection_data_.GetDictionary(i, &record);
 
     int this_rid = 0;
@@ -463,7 +553,7 @@ void WebRTCInternals::OnRendererExit(int render_process_id) {
         SendUpdate("removePeerConnection", std::move(update));
       }
       MaybeClosePeerConnection(record);
-      peer_connection_data_.Remove(i, NULL);
+      peer_connection_data_.Remove(i, nullptr);
     }
   }
   UpdateWakeLock();
@@ -472,14 +562,14 @@ void WebRTCInternals::OnRendererExit(int render_process_id) {
   // Iterates from the end of the list to remove the getUserMedia requests
   // created by the exiting renderer.
   for (int i = get_user_media_requests_.GetSize() - 1; i >= 0; --i) {
-    base::DictionaryValue* record = NULL;
+    base::DictionaryValue* record = nullptr;
     get_user_media_requests_.GetDictionary(i, &record);
 
     int this_rid = 0;
     record->GetInteger("rid", &this_rid);
 
     if (this_rid == render_process_id) {
-      get_user_media_requests_.Remove(i, NULL);
+      get_user_media_requests_.Remove(i, nullptr);
       found_any = true;
     }
   }
@@ -494,8 +584,9 @@ void WebRTCInternals::OnRendererExit(int render_process_id) {
 #if BUILDFLAG(ENABLE_WEBRTC)
 void WebRTCInternals::EnableAudioDebugRecordingsOnAllRenderProcessHosts() {
   DCHECK_CURRENTLY_ON(BrowserThread::UI);
-
-  audio_debug_recordings_ = true;
+  DCHECK(!audio_debug_recording_session_);
+  audio_debug_recording_session_ = audio::CreateAudioDebugRecordingSession(
+      audio_debug_recordings_file_path_, GetSystemConnector()->Clone());
 
   for (RenderProcessHost::iterator i(
            content::RenderProcessHost::AllHostsIterator());
@@ -503,26 +594,6 @@ void WebRTCInternals::EnableAudioDebugRecordingsOnAllRenderProcessHosts() {
     i.GetCurrentValue()->EnableAudioDebugRecordings(
         audio_debug_recordings_file_path_);
   }
-
-  // It's safe to get the AudioManager pointer here. That pointer is invalidated
-  // on the UI thread, which we're on.
-  // AudioManager is deleted on the audio thread, and the AudioManager outlives
-  // this object, so it's safe to post unretained to the audio thread.
-  media::AudioManager* audio_manager = media::AudioManager::Get();
-  audio_manager->GetTaskRunner()->PostTask(
-      FROM_HERE, base::Bind(&media::AudioManager::EnableOutputDebugRecording,
-                            base::Unretained(audio_manager),
-                            audio_debug_recordings_file_path_));
-}
-
-void WebRTCInternals::EnableEventLogRecordingsOnAllRenderProcessHosts() {
-  DCHECK_CURRENTLY_ON(BrowserThread::UI);
-
-  event_log_recordings_ = true;
-  for (RenderProcessHost::iterator i(
-           content::RenderProcessHost::AllHostsIterator());
-       !i.IsAtEnd(); i.Advance())
-    i.GetCurrentValue()->StartWebRTCEventLog(event_log_recordings_file_path_);
 }
 #endif
 
@@ -534,9 +605,34 @@ void WebRTCInternals::MaybeClosePeerConnection(base::DictionaryValue* record) {
     return;
 
   record->SetBoolean("isOpen", false);
-  --num_open_connections_;
-  DCHECK_GE(num_open_connections_, 0);
-  UpdateWakeLock();
+  MaybeMarkPeerConnectionAsNotConnected(record);
+}
+
+void WebRTCInternals::MaybeMarkPeerConnectionAsConnected(
+    base::DictionaryValue* record) {
+  bool was_connected = true;
+  record->GetBoolean("connected", &was_connected);
+  if (!was_connected) {
+    ++num_connected_connections_;
+    record->SetBoolean("connected", true);
+    UpdateWakeLock();
+    for (auto& observer : connections_observers_)
+      observer.OnConnectionsCountChange(num_connected_connections_);
+  }
+}
+
+void WebRTCInternals::MaybeMarkPeerConnectionAsNotConnected(
+    base::DictionaryValue* record) {
+  bool was_connected = false;
+  record->GetBoolean("connected", &was_connected);
+  if (was_connected) {
+    record->SetBoolean("connected", false);
+    --num_connected_connections_;
+    DCHECK_GE(num_connected_connections_, 0);
+    UpdateWakeLock();
+    for (auto& observer : connections_observers_)
+      observer.OnConnectionsCountChange(num_connected_connections_);
+  }
 }
 
 void WebRTCInternals::UpdateWakeLock() {
@@ -544,12 +640,13 @@ void WebRTCInternals::UpdateWakeLock() {
   if (!should_block_power_saving_)
     return;
 
-  if (num_open_connections_ == 0) {
+  if (num_connected_connections_ == 0) {
     DVLOG(1)
         << ("Cancel the wake lock on application suspension since no "
             "PeerConnections are active anymore.");
     GetWakeLock()->CancelWakeLock();
-  } else if (num_open_connections_ != 0) {
+  } else {
+    DCHECK_GT(num_connected_connections_, 0);
     DVLOG(1) << ("Preventing the application from being suspended while one or "
                  "more PeerConnections are active.");
     GetWakeLock()->RequestWakeLock();
@@ -559,20 +656,18 @@ void WebRTCInternals::UpdateWakeLock() {
 device::mojom::WakeLock* WebRTCInternals::GetWakeLock() {
   // Here is a lazy binding, and will not reconnect after connection error.
   if (!wake_lock_) {
-    device::mojom::WakeLockRequest request = mojo::MakeRequest(&wake_lock_);
-    // In some testing contexts, the service manager connection isn't
-    // initialized.
-    if (ServiceManagerConnection::GetForProcess()) {
-      service_manager::Connector* connector =
-          ServiceManagerConnection::GetForProcess()->GetConnector();
-      DCHECK(connector);
-      device::mojom::WakeLockProviderPtr wake_lock_provider;
-      connector->BindInterface(device::mojom::kServiceName,
-                               mojo::MakeRequest(&wake_lock_provider));
+    mojo::PendingReceiver<device::mojom::WakeLock> receiver =
+        wake_lock_.BindNewPipeAndPassReceiver();
+    // In some testing environments, the system Connector isn't initialized.
+    service_manager::Connector* connector = GetSystemConnector();
+    if (connector) {
+      mojo::Remote<device::mojom::WakeLockProvider> wake_lock_provider;
+      connector->Connect(device::mojom::kServiceName,
+                         wake_lock_provider.BindNewPipeAndPassReceiver());
       wake_lock_provider->GetWakeLockWithoutContext(
-          device::mojom::WakeLockType::PreventAppSuspension,
-          device::mojom::WakeLockReason::ReasonOther,
-          "WebRTC has active PeerConnections", std::move(request));
+          device::mojom::WakeLockType::kPreventAppSuspension,
+          device::mojom::WakeLockReason::kOther,
+          "WebRTC has active PeerConnections", std::move(receiver));
     }
   }
   return wake_lock_.get();
