@@ -11,14 +11,12 @@
 #include <vector>
 
 #include "base/bind.h"
-#include "base/feature_list.h"
 #include "base/metrics/histogram_functions.h"
 #include "base/stl_util.h"
 #include "build/build_config.h"
+#include "components/cbor/diagnostic_writer.h"
 #include "components/device_event_log/device_event_log.h"
-#include "device/fido/authenticator_get_assertion_response.h"
 #include "device/fido/cable/fido_cable_discovery.h"
-#include "device/fido/features.h"
 #include "device/fido/fido_authenticator.h"
 #include "device/fido/fido_discovery_factory.h"
 #include "device/fido/fido_parsing_utils.h"
@@ -29,10 +27,56 @@
 #include "device/fido/mac/authenticator.h"
 #endif  // defined(OS_MACOSX)
 
+#if defined(OS_WIN)
+#include "device/fido/win/authenticator.h"
+#include "device/fido/win/type_conversions.h"
+#endif
+
 namespace device {
 
 namespace {
 
+base::Optional<GetAssertionStatus> ConvertDeviceResponseCode(
+    CtapDeviceResponseCode device_response_code) {
+  switch (device_response_code) {
+    case CtapDeviceResponseCode::kSuccess:
+      return GetAssertionStatus::kSuccess;
+
+    // Only returned after the user interacted with the
+    // authenticator.
+    case CtapDeviceResponseCode::kCtap2ErrNoCredentials:
+      return GetAssertionStatus::kUserConsentButCredentialNotRecognized;
+
+    // The user explicitly denied the operation. Touch ID returns this error
+    // when the user cancels the macOS prompt. External authenticators may
+    // return it e.g. after the user fails fingerprint verification.
+    case CtapDeviceResponseCode::kCtap2ErrOperationDenied:
+      return GetAssertionStatus::kUserConsentDenied;
+
+    // External authenticators may return this error if internal user
+    // verification fails for a make credential request or if the pin token is
+    // not valid.
+    case CtapDeviceResponseCode::kCtap2ErrPinAuthInvalid:
+      return GetAssertionStatus::kUserConsentDenied;
+
+    // This error is returned by some authenticators (e.g. the "Yubico FIDO
+    // 2" CTAP2 USB keys) during GetAssertion **before the user interacted
+    // with the device**. The authenticator does this to avoid blinking (and
+    // possibly asking the user for their PIN) for requests it knows
+    // beforehand it cannot handle.
+    //
+    // Ignore this error to avoid canceling the request without user
+    // interaction.
+    case CtapDeviceResponseCode::kCtap2ErrInvalidCredential:
+      return base::nullopt;
+
+    // For all other errors, the authenticator will be dropped, and other
+    // authenticators may continue.
+    default:
+      return base::nullopt;
+  }
+}
+
 bool ResponseValid(const FidoAuthenticator& authenticator,
                    const CtapGetAssertionRequest& request,
                    const AuthenticatorGetAssertionResponse& response) {
@@ -108,6 +152,16 @@ bool ResponseValid(const FidoAuthenticator& authenticator,
     return false;
   }
 
+  // No extensions are supported when getting assertions therefore no extensions
+  // are permitted in the response.
+  const base::Optional<cbor::Value>& extensions =
+      response.auth_data().extensions();
+  if (extensions) {
+    FIDO_LOG(ERROR) << "assertion response invalid due to extensions block: "
+                    << cbor::DiagnosticWriter::Write(*extensions);
+    return false;
+  }
+
   return true;
 }
 
@@ -122,30 +176,6 @@ void SetCredentialIdForResponseWithEmptyCredential(
   }
 }
 
-// Checks UserVerificationRequirement enum passed from the relying party is
-// compatible with the authenticator.
-bool CheckUserVerificationCompatible(FidoAuthenticator* authenticator,
-                                     const CtapGetAssertionRequest& request,
-                                     bool have_observer) {
-  const auto& opt_options = authenticator->Options();
-  if (!opt_options) {
-    // This authenticator doesn't know its capabilities yet, so we need
-    // to assume it can handle the request. This is the case for Windows,
-    // where we proxy the request to the native API.
-    return true;
-  }
-
-  const bool pin_support =
-      base::FeatureList::IsEnabled(device::kWebAuthPINSupport) && have_observer;
-  return request.user_verification != UserVerificationRequirement::kRequired ||
-         opt_options->user_verification_availability ==
-             AuthenticatorSupportedOptions::UserVerificationAvailability::
-                 kSupportedAndConfigured ||
-         (pin_support && opt_options->client_pin_availability ==
-                             AuthenticatorSupportedOptions::
-                                 ClientPinAvailability::kSupportedAndPinSet);
-}
-
 base::flat_set<FidoTransportProtocol> GetTransportsAllowedByRP(
     const CtapGetAssertionRequest& request) {
   const base::flat_set<FidoTransportProtocol> kAllTransports = {
@@ -171,14 +201,6 @@ base::flat_set<FidoTransportProtocol> GetTransportsAllowedByRP(
   return transports;
 }
 
-base::flat_set<FidoTransportProtocol> GetTransportsAllowedAndConfiguredByRP(
-    const CtapGetAssertionRequest& request) {
-  auto transports = GetTransportsAllowedByRP(request);
-  if (!request.cable_extension)
-    transports.erase(FidoTransportProtocol::kCloudAssistedBluetoothLowEnergy);
-  return transports;
-}
-
 void ReportGetAssertionRequestTransport(FidoAuthenticator* authenticator) {
   if (authenticator->AuthenticatorTransport()) {
     base::UmaHistogramEnumeration(
@@ -202,30 +224,22 @@ GetAssertionRequestHandler::GetAssertionRequestHandler(
     FidoDiscoveryFactory* fido_discovery_factory,
     const base::flat_set<FidoTransportProtocol>& supported_transports,
     CtapGetAssertionRequest request,
+    bool allow_skipping_pin_touch,
     CompletionCallback completion_callback)
-    : FidoRequestHandler(
+    : FidoRequestHandlerBase(
           connector,
           fido_discovery_factory,
           base::STLSetIntersection<base::flat_set<FidoTransportProtocol>>(
               supported_transports,
-              GetTransportsAllowedAndConfiguredByRP(request)),
-          std::move(completion_callback)),
+              GetTransportsAllowedByRP(request))),
+      completion_callback_(std::move(completion_callback)),
       request_(std::move(request)),
-      weak_factory_(this) {
+      allow_skipping_pin_touch_(allow_skipping_pin_touch) {
   transport_availability_info().request_type =
       FidoRequestHandlerBase::RequestType::kGetAssertion;
   transport_availability_info().has_empty_allow_list =
       request_.allow_list.empty();
 
-  if (base::Contains(transport_availability_info().available_transports,
-                     FidoTransportProtocol::kCloudAssistedBluetoothLowEnergy)) {
-    DCHECK(request_.cable_extension);
-    auto discovery =
-        fido_discovery_factory_->CreateCable(*request_.cable_extension);
-    discovery->set_observer(this);
-    discoveries().push_back(std::move(discovery));
-  }
-
   if (request_.allow_list.empty()) {
     // Resident credential requests always involve user verification.
     request_.user_verification = UserVerificationRequirement::kRequired;
@@ -248,42 +262,34 @@ void GetAssertionRequestHandler::DispatchRequest(
     return;
   }
 
-  if (!base::FeatureList::IsEnabled(device::kWebAuthPINSupport) &&
-      !CheckUserVerificationCompatible(authenticator, request_, observer())) {
-    // Don't flash authenticator without PIN support. This maintains
-    // previous behaviour and avoids adding UI unprotected by a feature
-    // flag without increasing the number of feature flags.
-    FIDO_LOG(DEBUG) << "Not dispatching request to "
-                    << authenticator->GetDisplayName()
-                    << " because authenticator is not UV-compatible";
-    return;
-  }
-
-  if (base::FeatureList::IsEnabled(device::kWebAuthPINSupport)) {
-    switch (authenticator->WillNeedPINToGetAssertion(request_, observer())) {
-      case FidoAuthenticator::GetAssertionPINDisposition::kUsePIN:
-        // A PIN will be needed. Just request a touch to let the user select
-        // this authenticator if they wish.
-        FIDO_LOG(DEBUG) << "Asking for touch from "
-                        << authenticator->GetDisplayName()
-                        << " because a PIN will be required";
-        authenticator->GetTouch(
-            base::BindOnce(&GetAssertionRequestHandler::HandleTouch,
-                           weak_factory_.GetWeakPtr(), authenticator));
+  switch (authenticator->WillNeedPINToGetAssertion(request_, observer())) {
+    case FidoAuthenticator::GetAssertionPINDisposition::kUsePIN:
+      // Skip asking for touch if this is the only available authenticator.
+      if (active_authenticators().size() == 1 && allow_skipping_pin_touch_) {
+        HandleTouch(authenticator);
         return;
+      }
+      // A PIN will be needed. Just request a touch to let the user select
+      // this authenticator if they wish.
+      FIDO_LOG(DEBUG) << "Asking for touch from "
+                      << authenticator->GetDisplayName()
+                      << " because a PIN will be required";
+      authenticator->GetTouch(
+          base::BindOnce(&GetAssertionRequestHandler::HandleTouch,
+                         weak_factory_.GetWeakPtr(), authenticator));
+      return;
 
-      case FidoAuthenticator::GetAssertionPINDisposition::kUnsatisfiable:
-        FIDO_LOG(DEBUG) << authenticator->GetDisplayName()
-                        << " cannot satisfy assertion request. Requesting "
-                           "touch in order to handle error case.";
-        authenticator->GetTouch(base::BindOnce(
-            &GetAssertionRequestHandler::HandleInapplicableAuthenticator,
-            weak_factory_.GetWeakPtr(), authenticator));
-        return;
+    case FidoAuthenticator::GetAssertionPINDisposition::kUnsatisfiable:
+      FIDO_LOG(DEBUG) << authenticator->GetDisplayName()
+                      << " cannot satisfy assertion request. Requesting "
+                         "touch in order to handle error case.";
+      authenticator->GetTouch(base::BindOnce(
+          &GetAssertionRequestHandler::HandleAuthenticatorMissingUV,
+          weak_factory_.GetWeakPtr(), authenticator));
+      return;
 
-      case FidoAuthenticator::GetAssertionPINDisposition::kNoPIN:
-        break;
-    }
+    case FidoAuthenticator::GetAssertionPINDisposition::kNoPIN:
+      break;
   }
 
   CtapGetAssertionRequest request(request_);
@@ -315,8 +321,10 @@ void GetAssertionRequestHandler::AuthenticatorAdded(
   DCHECK_CALLED_ON_VALID_SEQUENCE(my_sequence_checker_);
 
 #if defined(OS_MACOSX)
-  if (authenticator->AuthenticatorTransport() ==
-      FidoTransportProtocol::kInternal) {
+  // Indicate to the UI whether a GetAssertion call to Touch ID would succeed
+  // or not. This needs to happen before the base AuthenticatorAdded()
+  // implementation runs |notify_observer_callback_| for this callback.
+  if (authenticator->IsTouchIdAuthenticator()) {
     transport_availability_info().has_recognized_mac_touch_id_credential =
         static_cast<fido::mac::TouchIdAuthenticator*>(authenticator)
             ->HasCredentialForGetAssertionRequest(request_);
@@ -339,7 +347,7 @@ void GetAssertionRequestHandler::AuthenticatorRemoved(
         state_ == State::kWaitingForSecondTouch) {
       state_ = State::kFinished;
       std::move(completion_callback_)
-          .Run(FidoReturnCode::kAuthenticatorRemovedDuringPINEntry,
+          .Run(GetAssertionStatus::kAuthenticatorRemovedDuringPINEntry,
                base::nullopt, nullptr);
     }
   }
@@ -359,19 +367,37 @@ void GetAssertionRequestHandler::HandleResponse(
     return;
   }
 
+#if defined(OS_WIN)
+  if (authenticator->IsWinNativeApiAuthenticator()) {
+    state_ = State::kFinished;
+    CancelActiveAuthenticators(authenticator->GetId());
+    if (status != CtapDeviceResponseCode::kSuccess) {
+      std::move(completion_callback_)
+          .Run(WinCtapDeviceResponseCodeToGetAssertionStatus(status),
+               base::nullopt, authenticator);
+      return;
+    }
+    DCHECK(responses_.empty());
+    responses_.emplace_back(std::move(*response));
+    std::move(completion_callback_)
+        .Run(WinCtapDeviceResponseCodeToGetAssertionStatus(status),
+             std::move(responses_), authenticator);
+    return;
+  }
+#endif
+
   // Requests that require a PIN should follow the |GetTouch| path initially.
   DCHECK(state_ == State::kWaitingForSecondTouch ||
-         !base::FeatureList::IsEnabled(device::kWebAuthPINSupport) ||
          authenticator->WillNeedPINToGetAssertion(request_, observer()) ==
              FidoAuthenticator::GetAssertionPINDisposition::kNoPIN);
 
-  const base::Optional<FidoReturnCode> maybe_result =
-      ConvertDeviceResponseCodeToFidoReturnCode(status);
+  const base::Optional<GetAssertionStatus> maybe_result =
+      ConvertDeviceResponseCode(status);
   if (!maybe_result) {
     if (state_ == State::kWaitingForSecondTouch) {
-      OnAuthenticatorResponse(authenticator,
-                              FidoReturnCode::kAuthenticatorResponseInvalid,
-                              base::nullopt);
+      std::move(completion_callback_)
+          .Run(GetAssertionStatus::kAuthenticatorResponseInvalid, base::nullopt,
+               authenticator);
     } else {
       FIDO_LOG(ERROR) << "Ignoring status " << static_cast<int>(status)
                       << " from " << authenticator->GetDisplayName();
@@ -386,16 +412,17 @@ void GetAssertionRequestHandler::HandleResponse(
     FIDO_LOG(ERROR) << "Failing assertion request due to status "
                     << static_cast<int>(status) << " from "
                     << authenticator->GetDisplayName();
-    OnAuthenticatorResponse(authenticator, *maybe_result, base::nullopt);
+    std::move(completion_callback_)
+        .Run(*maybe_result, base::nullopt, authenticator);
     return;
   }
 
   if (!response || !ResponseValid(*authenticator, request_, *response)) {
     FIDO_LOG(ERROR) << "Failing assertion request due to bad response from "
                     << authenticator->GetDisplayName();
-    OnAuthenticatorResponse(authenticator,
-                            FidoReturnCode::kAuthenticatorResponseInvalid,
-                            base::nullopt);
+    std::move(completion_callback_)
+        .Run(GetAssertionStatus::kAuthenticatorResponseInvalid, base::nullopt,
+             authenticator);
     return;
   }
 
@@ -403,9 +430,9 @@ void GetAssertionRequestHandler::HandleResponse(
   const size_t num_responses = response->num_credentials().value_or(1);
   if (num_responses == 0 ||
       (num_responses > 1 && !request_.allow_list.empty())) {
-    OnAuthenticatorResponse(authenticator,
-                            FidoReturnCode::kAuthenticatorResponseInvalid,
-                            base::nullopt);
+    std::move(completion_callback_)
+        .Run(GetAssertionStatus::kAuthenticatorResponseInvalid, base::nullopt,
+             authenticator);
     return;
   }
 
@@ -423,8 +450,8 @@ void GetAssertionRequestHandler::HandleResponse(
 
   ReportGetAssertionResponseTransport(authenticator);
 
-  OnAuthenticatorResponse(authenticator, FidoReturnCode::kSuccess,
-                          std::move(responses_));
+  std::move(completion_callback_)
+      .Run(GetAssertionStatus::kSuccess, std::move(responses_), authenticator);
 }
 
 void GetAssertionRequestHandler::HandleNextResponse(
@@ -440,18 +467,18 @@ void GetAssertionRequestHandler::HandleNextResponse(
     FIDO_LOG(ERROR) << "Failing assertion request due to status "
                     << static_cast<int>(status) << " from "
                     << authenticator->GetDisplayName();
-    OnAuthenticatorResponse(authenticator,
-                            FidoReturnCode::kAuthenticatorResponseInvalid,
-                            base::nullopt);
+    std::move(completion_callback_)
+        .Run(GetAssertionStatus::kAuthenticatorResponseInvalid, base::nullopt,
+             authenticator);
     return;
   }
 
   if (!ResponseValid(*authenticator, request_, *response)) {
     FIDO_LOG(ERROR) << "Failing assertion request due to bad response from "
                     << authenticator->GetDisplayName();
-    OnAuthenticatorResponse(authenticator,
-                            FidoReturnCode::kAuthenticatorResponseInvalid,
-                            base::nullopt);
+    std::move(completion_callback_)
+        .Run(GetAssertionStatus::kAuthenticatorResponseInvalid, base::nullopt,
+             authenticator);
     return;
   }
 
@@ -468,8 +495,8 @@ void GetAssertionRequestHandler::HandleNextResponse(
 
   ReportGetAssertionResponseTransport(authenticator);
 
-  OnAuthenticatorResponse(authenticator, FidoReturnCode::kSuccess,
-                          std::move(responses_));
+  std::move(completion_callback_)
+      .Run(GetAssertionStatus::kSuccess, std::move(responses_), authenticator);
 }
 
 void GetAssertionRequestHandler::HandleTouch(FidoAuthenticator* authenticator) {
@@ -477,9 +504,8 @@ void GetAssertionRequestHandler::HandleTouch(FidoAuthenticator* authenticator) {
     return;
   }
 
-  DCHECK(base::FeatureList::IsEnabled(device::kWebAuthPINSupport) &&
-         authenticator->WillNeedPINToGetAssertion(request_, observer()) !=
-             FidoAuthenticator::GetAssertionPINDisposition::kNoPIN);
+  DCHECK(authenticator->WillNeedPINToGetAssertion(request_, observer()) !=
+         FidoAuthenticator::GetAssertionPINDisposition::kNoPIN);
 
   DCHECK(observer());
   state_ = State::kGettingRetries;
@@ -490,13 +516,13 @@ void GetAssertionRequestHandler::HandleTouch(FidoAuthenticator* authenticator) {
                      weak_factory_.GetWeakPtr()));
 }
 
-void GetAssertionRequestHandler::HandleInapplicableAuthenticator(
+void GetAssertionRequestHandler::HandleAuthenticatorMissingUV(
     FidoAuthenticator* authenticator) {
   // User touched an authenticator that cannot handle this request.
   state_ = State::kFinished;
   CancelActiveAuthenticators(authenticator->GetId());
   std::move(completion_callback_)
-      .Run(FidoReturnCode::kUserConsentButCredentialNotRecognized,
+      .Run(GetAssertionStatus::kAuthenticatorMissingUserVerification,
            base::nullopt, nullptr);
 }
 
@@ -508,14 +534,14 @@ void GetAssertionRequestHandler::OnRetriesResponse(
   if (status != CtapDeviceResponseCode::kSuccess) {
     state_ = State::kFinished;
     std::move(completion_callback_)
-        .Run(FidoReturnCode::kAuthenticatorResponseInvalid, base::nullopt,
+        .Run(GetAssertionStatus::kAuthenticatorResponseInvalid, base::nullopt,
              nullptr);
     return;
   }
   if (response->retries == 0) {
     state_ = State::kFinished;
     std::move(completion_callback_)
-        .Run(FidoReturnCode::kHardPINBlock, base::nullopt, nullptr);
+        .Run(GetAssertionStatus::kHardPINBlock, base::nullopt, nullptr);
     return;
   }
   state_ = State::kWaitingForPIN;
@@ -532,7 +558,6 @@ void GetAssertionRequestHandler::OnHavePIN(std::string pin) {
   if (authenticator_ == nullptr) {
     // Authenticator was detached. The request will already have been canceled
     // but this callback may have been waiting in a queue.
-    DCHECK(!completion_callback_);
     return;
   }
 
@@ -552,7 +577,7 @@ void GetAssertionRequestHandler::OnHaveEphemeralKey(
   if (status != CtapDeviceResponseCode::kSuccess) {
     state_ = State::kFinished;
     std::move(completion_callback_)
-        .Run(FidoReturnCode::kAuthenticatorResponseInvalid, base::nullopt,
+        .Run(GetAssertionStatus::kAuthenticatorResponseInvalid, base::nullopt,
              nullptr);
     return;
   }
@@ -580,16 +605,16 @@ void GetAssertionRequestHandler::OnHavePINToken(
 
   if (status != CtapDeviceResponseCode::kSuccess) {
     state_ = State::kFinished;
-    FidoReturnCode ret;
+    GetAssertionStatus ret;
     switch (status) {
       case CtapDeviceResponseCode::kCtap2ErrPinAuthBlocked:
-        ret = FidoReturnCode::kSoftPINBlock;
+        ret = GetAssertionStatus::kSoftPINBlock;
         break;
       case CtapDeviceResponseCode::kCtap2ErrPinBlocked:
-        ret = FidoReturnCode::kHardPINBlock;
+        ret = GetAssertionStatus::kHardPINBlock;
         break;
       default:
-        ret = FidoReturnCode::kAuthenticatorResponseInvalid;
+        ret = GetAssertionStatus::kAuthenticatorResponseInvalid;
         break;
     }
     std::move(completion_callback_).Run(ret, base::nullopt, nullptr);
