@@ -26,7 +26,11 @@
 namespace base {
 class SingleThreadTaskRunner;
 class Thread;
-}
+}  // namespace base
+
+namespace service_manager {
+class Connector;
+}  // namespace service_manager
 
 namespace device {
 
@@ -43,14 +47,17 @@ class DEVICE_GAMEPAD_EXPORT GamepadProvider
     : public GamepadPadStateProvider,
       public base::SystemMonitor::DevicesChangedObserver {
  public:
-  explicit GamepadProvider(
-      GamepadConnectionChangeClient* connection_change_client);
+  GamepadProvider(
+      GamepadConnectionChangeClient* connection_change_client,
+      std::unique_ptr<service_manager::Connector> service_manager_connector);
 
   // Manually specifies the data fetcher and polling thread. The polling thread
   // will be created normally if |polling_thread| is nullptr. Used for testing.
-  GamepadProvider(GamepadConnectionChangeClient* connection_change_client,
-                  std::unique_ptr<GamepadDataFetcher> fetcher,
-                  std::unique_ptr<base::Thread> polling_thread);
+  GamepadProvider(
+      GamepadConnectionChangeClient* connection_change_client,
+      std::unique_ptr<service_manager::Connector> service_manager_connector,
+      std::unique_ptr<GamepadDataFetcher> fetcher,
+      std::unique_ptr<base::Thread> polling_thread);
 
   ~GamepadProvider() override;
 
@@ -76,7 +83,7 @@ class DEVICE_GAMEPAD_EXPORT GamepadProvider
 
   // Registers the given closure for calling when the user has interacted with
   // the device. This callback will only be issued once.
-  void RegisterForUserGesture(const base::Closure& closure);
+  void RegisterForUserGesture(base::OnceClosure closure);
 
   // base::SystemMonitor::DevicesChangedObserver implementation.
   void OnDevicesChanged(base::SystemMonitor::DeviceType type) override;
@@ -115,6 +122,10 @@ class DEVICE_GAMEPAD_EXPORT GamepadProvider
   // true if any user gesture observers were notified.
   bool CheckForUserGesture();
 
+  // GamepadPadStateProvider implementation.
+  void DisconnectUnrecognizedGamepad(GamepadSource source,
+                                     int source_id) override;
+
   void PlayEffectOnPollingThread(
       uint32_t pad_index,
       mojom::GamepadHapticEffectType,
@@ -133,12 +144,12 @@ class DEVICE_GAMEPAD_EXPORT GamepadProvider
   // Keeps track of when the background thread is paused. Access to is_paused_
   // must be guarded by is_paused_lock_.
   base::Lock is_paused_lock_;
-  bool is_paused_;
+  bool is_paused_ = true;
 
   // Keep track of when a polling task is schedlued, so as to prevent us from
   // accidentally scheduling more than one at any time, when rapidly toggling
   // |is_paused_|.
-  bool have_scheduled_do_poll_;
+  bool have_scheduled_do_poll_ = false;
 
   // Lists all observers registered for user gestures, and the thread which
   // to issue the callbacks on. Since we always issue the callback on the
@@ -146,15 +157,8 @@ class DEVICE_GAMEPAD_EXPORT GamepadProvider
   // thread, the message loop proxies will normally just be the I/O thread.
   // However, this will be the main thread for unit testing.
   base::Lock user_gesture_lock_;
-  struct ClosureAndThread {
-    ClosureAndThread(const base::Closure& c,
-                     const scoped_refptr<base::SingleThreadTaskRunner>& m);
-    ClosureAndThread(const ClosureAndThread& other);
-    ~ClosureAndThread();
-
-    base::Closure closure;
-    scoped_refptr<base::SingleThreadTaskRunner> task_runner;
-  };
+  using ClosureAndThread =
+      std::pair<base::OnceClosure, scoped_refptr<base::SingleThreadTaskRunner>>;
   using UserGestureObserverVector = std::vector<ClosureAndThread>;
   UserGestureObserverVector user_gesture_observers_;
 
@@ -164,10 +168,10 @@ class DEVICE_GAMEPAD_EXPORT GamepadProvider
   // tests. Access to devices_changed_ must be guarded by
   // devices_changed_lock_.
   base::Lock devices_changed_lock_;
-  bool devices_changed_;
+  bool devices_changed_ = true;
 
-  bool ever_had_user_gesture_;
-  bool sanitize_;
+  bool ever_had_user_gesture_ = false;
+  bool sanitize_ = true;
 
   // Only used on the polling thread.
   using GamepadFetcherVector = std::vector<std::unique_ptr<GamepadDataFetcher>>;
@@ -181,6 +185,10 @@ class DEVICE_GAMEPAD_EXPORT GamepadProvider
 
   GamepadConnectionChangeClient* connection_change_client_;
 
+  // Service manager connector, to allow data fetchers to access the device
+  // service from the polling thread.
+  std::unique_ptr<service_manager::Connector> service_manager_connector_;
+
   DISALLOW_COPY_AND_ASSIGN(GamepadProvider);
 };
 
